/**
 * \file gre/datatip.h
 * \brief GRE DataTip definitions
 *
 * \if NODOC
 * $Id: datatip.h_v 1.17 2013/11/01 19:16:57 mju Exp $
 *
 * $Log: datatip.h_v $
 * Revision 1.17  2013/11/01 19:16:57  mju
 * Remove legacy method.
 *
 * Revision 1.16  2013/10/02 20:58:22  mju
 * Add setUnit and make setPredefined non-inline.
 *
 * Revision 1.15  2013/10/02 16:57:57  mju
 * Add predefined_lidar values.
 *
 * Revision 1.14  2013/10/01 16:28:00  mju
 * Move datatipgeneral to own hdr.
 *
 * Revision 1.13  2013/09/30 22:17:08  mju
 * Add members for image size, flags, fieldtype.
 *
 * Revision 1.12  2013/09/27 21:12:02  mju
 * Add more datatipparms.set... methods.
 * Add scaleVisibleMin/Max members/methods.
 *
 * Revision 1.11  2013/09/27 19:40:40  mju
 * Add addObjNamePrefix.
 *
 * Revision 1.10  2013/09/27 17:43:53  mju
 * Add general setting to show object names.
 *
 * Revision 1.9  2013/09/27 17:18:02  mju
 * Use maxLines, not maxLayers.
 *
 * Revision 1.8  2013/09/27 15:38:43  mju
 * Add datatipgeneral class.
 *
 * Revision 1.7  2012/07/23 22:45:07  bcolombe
 * Add UnitDisplayed
 *
 * Revision 1.6  2012/06/12 21:44:11  mju
 * Add formatValue for numeric.
 *
 * Revision 1.5  2012/06/12 20:17:58  mju
 * Add thousands separator.
 *
 * Revision 1.4  2006/12/06 22:09:45  dwilliss
 * Added support for images mixed with text
 *
 * Revision 1.3  2006/10/02 20:52:02  mju
 * Add checkField.
 *
 * Revision 1.1  2006/09/28 13:04:23  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_DATATIP_H
#define	INC_GRE_DATATIP_H

#ifndef	INC_RVC_OBJECTNAME_H
	#include <rvc/objectname.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
class SERIALIZERITEM;
namespace RVC {
	class DBTABLE;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE_ARRAY;
		}
	}
#endif


namespace GRE {

//! DataTip parameters.
struct DATATIPPARMS {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class FORM;
		#endif

		//! Built-in datatip values.
		enum PREDEFINED {
			PREDEFINED_None =								0,
			PREDEFINED_ImageCellValue =				1,
			PREDEFINED_LinkTileName =					2,
			PREDEFINED_LidarClassName =				3,
			PREDEFINED_LidarReturnNumberCount =		4,
			};

		enum FIELDTYPE {
			FIELDTYPE_Unspecified = 0,
			FIELDTYPE_Numeric,
			FIELDTYPE_Image
			};

		enum FLAGS {
			FLAG_None =						0,
			FLAG_AddStyleAttribute =	0x0001,
			};

		RVC::OBJECTNAME TableName;					//!< Table name
		RVC::OBJECTNAME FieldName;					//!< Field name
		MISTRING Prefix;								//!< Label to place before value in string, defaults to table.field name
		MISTRING Suffix;								//!< Label to place after value in string, defaults to unit name
		INT32 TableNum;								//!< Table number
		mutable INT32 FieldNum;						//!< Field number
		UINT32 UnitDisplayed;						//!< Unit ID to display string in
		PREDEFINED m_Predefined;					//!< Predefined value to show
		UINT8 Visible;									//!< Show this DataTip
		UINT8 HighlightElem;							//!< Set to highlight element in view when datatip shown
		UINT8 UnitType;								//!< Field unit type, here for validation against DBFIELDINFO
		UINT8 UnitShown;								//!< Unit to show field in
		UINT8 DecPlaces;								//!< Decimal places for floating-point fields

		//! Get serialization item definition array.
		static const SERIALIZERITEM* SerialGetItemDef();

		//! Default constructor.
		DATATIPPARMS ();

		//! Copy constructor.
		DATATIPPARMS (
			const DATATIPPARMS& rhs
			);

		//! Assignment.
		DATATIPPARMS& operator= (
			const DATATIPPARMS& rhs
			);

		//! Add object name prefix if specified.
		void AddObjNamePrefix (
			MISTRING& string,						//!< String to add to
			const MISTRING& name					//!< Object name
			) const;

		//! Check field name for validity in specified table
		ERRVALUE CheckField (
			RVC::DBTABLE& ObjTable
			);

		//! Do string formatting with prefix/suffix.
		void FormatString (
			MISTRING& ostr,						//!< String to append to
			const MISTRING& istr					//!< DataTip value
			) const;

		//! Do formatting of numeric value with prefix/suffix.
		void FormatValue (
			MISTRING& ostr,						//!< String to append to
			double value,							//!< DataTip value
			UINT8 unitnum = 0						//!< Value unit number
			) const;

		FIELDTYPE GetFieldType () const
			{ return (m_FieldType); }

		PREDEFINED GetPredefined () const
			{ return (m_Predefined); }

		//! Get "prefix" setting.
		const MISTRING& GetPrefix () const
			{ return (this->Prefix); }

		//! Get maximum visible (reciprocal) scale.
		double GetScaleVisibleMax () const
			{ return (m_ScaleVisibleMax); }

		//! Get minimum visible (reciprocal) scale.
		double GetScaleVisibleMin () const
			{ return (m_ScaleVisibleMin); }

		UINT8 GetUnitShown () const
			{ return (this->UnitShown); }

		UINT8 GetUnitType () const
			{ return (this->UnitType); }

		//! Get "suffix" setting.
		const MISTRING& GetSuffix () const
			{ return (this->Suffix); }

		bool IsDefined () const;

		bool IsEnabled () const
			{ return (this->Visible != 0); }

		//! Determine if is enabled at specified scale.
		bool IsEnabledAtScale (
			double scale
			) const;

		bool IsNumeric () const
			{ return (m_FieldType == FIELDTYPE_Numeric); }

		MISTRING NumericToString (double value, UINT8 unitnum) const;

		//! Read string from table with minimal formatting
		//! @return TRUE if have value, FALSE if not, < 0 if error.
		int ReadString (
			RVC::DBTABLE& ObjTable,				//!< Table to read from
			ELEMTYPE ElemType,					//!< Element type
			INT32 ElemNum,							//!< Element number
			MISTRING& string,						//!< String to append result to
			SPATMOD::IMAGE::STAGE_ARRAY& images, //!< Images to insert into the string in place of {~IMG #} tags
			RVC::DBTABLE *pSourceTable = 0	//!< Source table if required
			) const;

		//! Read string from table and format with prefix/suffix.
		//! @return TRUE if have value, FALSE if not, < 0 if error.
		int ReadStringFormatted (
			RVC::DBTABLE& ObjTable,				//!< Table to read from
			ELEMTYPE ElemType,					//!< Element type
			INT32 ElemNum,							//!< Element number
			MISTRING& string,						//!< String to append result to
			SPATMOD::IMAGE::STAGE_ARRAY& images, //!< Images to insert into the string in place of {~IMG #} tags
			RVC::DBTABLE *pSourceTable = 0	//!< Source table if required
			) const;

		void SetDecPlaces (
			UINT8 DecPlaces
			) { this->DecPlaces = DecPlaces; }

		//! Set suffix to default based on unit type/shown.
		void SetDefaultSuffix ();

		void SetFieldType (
			FIELDTYPE FieldType
			) { m_FieldType = FieldType; }

		void SetFieldNum (
			INT32 FieldNum
			) { this->FieldNum = FieldNum; this->FieldName.Clear(); }

		void SetEnabled (
			bool bEnabled
			) { this->Visible = bEnabled; }

		void SetHighlightElem (
			bool bHighlightElem
			) { this->HighlightElem = bHighlightElem; }

		void SetImageSizeRange (
			UINT32 MinColumns,
			UINT32 MinRows,
			UINT32 MaxColumns,
			UINT32 MaxRows
			);

		//! Set as numeric or not.
		//! If false is specified and field type is not numeric the type will remain unchanged.
		void SetNumeric (
			bool bIsNumeric
			);

		void SetPredefined (
			PREDEFINED predefined
			);

		void SetPrefix (
			const MISTRING& prefix
			) { this->Prefix = prefix; }

		void SetScaleVisibleRange (
			double ScaleVisibleMin,
			double ScaleVisibleMax
			) { m_ScaleVisibleMin = ScaleVisibleMin; m_ScaleVisibleMax = ScaleVisibleMax; }

		void SetSuffix (
			const MISTRING& suffix
			) { this->Suffix = suffix; }

		void SetUnit (
			UINT8 iUnitType,
			UINT8 iUnitShown
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DATATIPPARMS THISCLASS;

		double m_ScaleVisibleMin;
		double m_ScaleVisibleMax;
		UINT32 m_ImageMinColumns;
		UINT32 m_ImageMinRows;
		UINT32 m_ImageMaxColumns;
		UINT32 m_ImageMaxRows;
		FIELDTYPE m_FieldType;
		FLAGS m_Flags;

		#endif

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DATATIPPARMS::FLAGS)
#endif

}	// End namespace GRE

#endif	// INC_GRE_DATATIP_H
