/**
 * \file gre/layout.h
 * \brief GRE::LAYOUT definitions
 *
 * \if NODOC
 * $Id: layout.h_v 1.61 2014/12/09 16:30:48 mju Exp $
 *
 * $Log: layout.h_v $
 * Revision 1.61  2014/12/09 16:30:48  mju
 * Add useRenderProjection to getSizeMeters.
 *
 * Revision 1.60  2013/09/05 22:05:59  mju
 * Add setHardcopyDocDimensions.
 *
 * Revision 1.59  2013/08/30 21:34:18  mju
 * Use printsettings class.
 *
 * Revision 1.58  2013/04/03 21:23:12  mju
 * Remove legacy groupSettings methods.
 *
 * Revision 1.57  2013/03/22 21:40:30  mju
 * Make posnmode an enum in layout.
 *
 * Revision 1.56  2013/03/21 21:32:09  mju
 * Remove getFullName and setName.
 *
 * Revision 1.55  2013/03/20 19:52:18  mju
 * Make createGroupNoNotify public.
 *
 * Revision 1.54  2013/03/18 20:58:11  mju
 * Add members for grid size/order modes.
 *
 * Revision 1.53  2013/02/27 15:45:41  mju
 * Add updatedft to setDesignScale.
 *
 * Revision 1.50  2013/02/14 22:43:04  mju
 * Use container base class.
 *
 * Revision 1.48  2013/02/07 18:00:52  mju
 * Add v_save and rename other virtual methods with v_ prefix.
 *
 * Revision 1.46  2011/03/01 22:43:00  bcolombe
 * Add SavedLocations
 *
 * Revision 1.45  2009/09/23 21:54:31  mju
 * Add isSortLegendNatural.
 *
 * Revision 1.43  2007/11/26 18:43:41  mju
 * Add v_getActiveLayer, v_unhighlightAllElements.
 *
 * Revision 1.34  2006/05/05 13:48:11  mju
 * Support deferred creation.
 *
 * Revision 1.32  2005/11/28 18:53:42  mju
 * Add getMax/MinScaleVisibleAny.
 *
 * Revision 1.23  2005/06/10 20:36:06  vdronov
 * combined old and new 3d rendering
 *
 * Revision 1.1  2003/06/16 19:39:47  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LAYOUT_H
#define	INC_GRE_LAYOUT_H

#ifndef	INC_GRE_GROUP_H
	#include <gre/group.h>
#endif

#ifndef INC_MI32_PRINTSETTINGS_H
	#include <mi32/printsettings.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class PRINTENGINE;
#endif


namespace GRE {
//===================================================================================================================

class LAYOUT : public GRE::VIEWABLE, public SERIALIZABLE {

	public:

		//! Flags used in creation
		enum CREATEFLAGS {
			CREATEFLAG_None =					0x0000,
			CREATEFLAG_NoCallCreateCB =	0x0001,	//!< Don't call callback list when object created
			CREATEFLAG_Temporary =			0x0002,	//!< Create as 'temporary'
			CREATEFLAG_Hardcopy =			0x0010,	//!< Create layout in "hardcopy" mode
			CREATEFLAG_DeferCreate =		0x0020	//!< Defer creation until Create() called
			};

		enum POSNMODE {
			POSNMODE_Tiled =				0,		//!< Groups are tiled
			POSNMODE_Display =			1,		//!< Positioning on "display"
			POSNMODE_Page =				2		//!< Positioning on "page" 
			};

		//! Mode for determining grid size when tiling.
		enum GRIDSIZEMODE {
			GRIDSIZEMODE_Automatic =		0,
			GRIDSIZEMODE_Columns,
			GRIDSIZEMODE_Rows
			};

		//! Order of grid cell filling.
		enum GRIDFILLORDER {
			GRIDFILLORDER_RightThenDown =		0,
			GRIDFILLORDER_DownThenRight =		1
			};

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SML::CONTEXT *context							//!< SML context
			);

		//! Constructor.
		explicit LAYOUT (
			CREATEFLAGS flags = CREATEFLAG_None
			);

		//! Destructor.
		virtual ~LAYOUT ();

	#ifdef USE_MGUI
		//! Dialog to add single-layer group and layer of specified type.
		//! Should only be used for single-layer-group types (eg scalebar, text, legend).
		ERRVALUE AddGroupLayer (
			MDLGPARENT dlgparent,
			GRE::LAYER::TYPE layertype,
			GRE::LAYER **player = 0						//! Pointer to layer returned
			);
	#endif

	#ifdef USE_MGUI
		//! Dialog to prompt user for objects and add as multiple groups.
		ERRVALUE AddGroupMulti (
			MDLGPARENT dlgparent
			);
	#endif

	#ifdef USE_MGUI
		//! Dialog to prompt user for saved group and add to layout.
		int AddGroupSaved (
			MDLGPARENT dlgparent,
			bool fromtext
			);
	#endif

		//! Create layout if CREATEFLAG_DeferCreate was specified in constructor.
		void Create (
			CREATEFLAGS createflags = CREATEFLAG_None
			);

		//! Create new group in this layout
		ERRVALUE CreateGroup (
			GRE::GROUP **retgroup = 0,					//!< New group returned
			GRE::GROUP::CREATEFLAGS createflags = GRE::GROUP::CREATEFLAG_None,
			GRE::LISTPOS listpos = GRE::LISTPOS_After,
			GRE::GROUP *refgroup = 0						//!< Reference group for InsertBefore/InsertAfter
			);

		//! Create group without CreateChildrenBegin/End notification.
		ERRVALUE CreateGroupNoNotify (
			GRE::GROUP **retgroup = 0,						//!< New group returned
			GRE::GROUP::CREATEFLAGS createflags = GRE::GROUP::CREATEFLAG_None,
			GRE::LISTPOS listpos = GRE::LISTPOS_After,
			GRE::GROUP *refgroup = 0						//!< Reference group for InsertBefore/InsertAfter
			);

		//! Display "Print Setup" dialog.
		//! @return TRUE if settings changed, FALSE if not, < 0 if error
		int DlgPrintSetup (
			MDLGPARENT dlgparent,							//!< Parent for dialog
			PRINTENGINE *pPrintEngine = 0
			);

		//! Find group containing specified point
		GRE::GROUP* FindPointGroup (
			const DPOINT2D& point,
			GRE::FINDFLAGS findflags,
			const SPATREF::COORDREFSYS *pPointCRS = NULL
			) const;

		//! Get active group for this layout
		GRE::GROUP* GetActiveGroup () const;

		//! Get first group in list.
		GRE::GROUP* GetFirstGroup () const
			{ return (static_cast<GROUP*>(GetFirstChild())); }

		GRIDFILLORDER GetGridFillOrder () const
			{ return (m_GridFillOrder); }

		GRIDSIZEMODE GetGridSizeMode () const
			{ return (m_GridSizeMode); }

		UINT16 GetGridTileCount () const
			{ return (m_GridTileCount); }

		//! Get hardcopy DPI value.
		double GetHardcopyDPI () const
			{ return (m_HardcopyLayout.GetResolution().x); }

		//! Get "margin" rectangle in layout (page) coordinates
		DRECT2D GetHardcopyMarginRect () const;

		//! Get "page" rectangle in layout (page) coordinates
		DRECT2D GetHardcopyPageRect () const;

		//! Get "printable" rectangle in layout (page) coordinates
		DRECT2D GetHardcopyPrintableRect () const;

		//! Get hardcopy layout scale.
		double GetHardcopyScale () const
			{ return (m_HardcopyLayout.GetDesignScale()); }

		//! Get hardcopy layout values.
		const PRINTSETTINGS& GetHardcopyValues () const
			{ return (m_HardcopyLayout); }

		//! Get hardcopy zoom factor.
		double GetHardcopyZoom () const
			{ return (m_HardcopyLayout.GetZoom()); }

		//! Get last group in list.
		GRE::GROUP* GetLastGroup () const
			{ return (static_cast<GROUP*>(GetLastChild())); }

		//! Get unit for display of placement offset/size values.
		UINT8 GetPlacementUnitDisplay () const
			{ return (m_PlacementUnitDisplay); }

		//! Get group positioning mode.
		POSNMODE GetPosnMode () const
			{ return (m_PosnMode); }

		//! Get Saved Locations string
		const MISTRING& GetSavedLocations () const
			{ return (m_SavedLocations); }

		//! Invalidate all view buffers for this layout.
		void InvalidateAllViews ();

		//! Determine if layout is currently being loaded.
		bool IsBeingLoaded () const
			{ return (m_pSerializer != 0); }

		//! Determine if layout has been created.
		//! This is done automatically by constructor unless CREATEFLAG_DeferCreate was specified.
		bool IsCreated () const
			{ return (m_IsCreated); }

		//! Determine if layout is in "hardcopy" mode.
		bool IsHardcopy () const
			{ return (m_PosnMode == POSNMODE_Page); }

		//! Determine if legends should be sorted in "natural" order.
		bool IsSortLegendNatural () const
			{ return (m_SortLegendNatural); }

		//! Read from serialization
		virtual ERRVALUE SerialRead (
			SERIALIZER& serializer
			);

		//! Write values to serialization
		virtual ERRVALUE SerialWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			) const;

		//! Set active group for layout.
		void SetActiveGroup (
			GRE::GROUP *group
			) { SetActiveChild(group); }

		void SetGridFillOrder (
			GRIDFILLORDER GridFillOrder
			) { m_GridFillOrder = GridFillOrder; }

		void SetGridSize (
			GRIDSIZEMODE GridSizeMode,
			UINT16 GridTileCount
			) { m_GridSizeMode = GridSizeMode; m_GridTileCount = GridTileCount; }

		//! Set hardcopy document (layout) dimensions.
		void SetHardcopyDocDimensions (
			const PRINTMEDIA& DocDims,
			bool notify = true
			);

		//! Set hardcopy layout values.
		void SetHardcopyValues (
			const PRINTSETTINGS& values,
			bool notify = true
			);

		//! Set matte values.
		void SetMatte (
			const MATTE& matte,						//!< Matte values to set
			bool notify = true						//!< Set to invoke callbacks with "Changed" message
			);

		//! Set that group positions need to be recomputed.
		void SetNeedUpdatePositions (
			) { m_NeedUpdatePositions = 1; }

		//! Set unit for display of placement offset/size values.
		void SetPlacementUnitDisplay (
			UINT8 unit
			) { m_PlacementUnitDisplay = unit; }

		//! Set group positioning mode
		void SetPosnMode (
			POSNMODE PosnMode,
			bool notify = true
			);

		//! Set saved locations string
		void SetSavedLocations (
			const MISTRING& string
			) { m_SavedLocations = string; }

		//! Set hardcopy zoom to fit to page size.
		int SetZoomToFitPage ();

		//! Read from template (serialization)
		ERRVALUE TemplateRead (
			SERIALIZER& serializer
			);

		//! Write template to serialization
		ERRVALUE TemplateWrite (
			SERIALIZER& serializer
			);

		//! Recompute group positions.
		int UpdatePositions (
			bool ForceUpdate = true
			);

		//! Validate and correct group attachments.
		void ValidateGroupAttachments ();

		#ifndef GENERATING_DOXYGEN_OUTPUT
	protected:

		static const SML::CLASSMEMBER* GetSMLClassMembers ();

	private:

		typedef GRE::LAYOUT THISCLASS;

		static const SERIALIZERITEM* SerialGetItemDef (
			SERIALIZER& serializer
			);

		static ERRVALUE SerializerCB_Group (
			SERIALIZER& serializer,
			const SERIALIZERITEM*,
			void *vlayout,
			SERIALIZERITEM::ACTION action
			);

		static ERRVALUE TemplateCB_Group (
			SERIALIZER& serializer,
			const SERIALIZERITEM*,
			void *vlayout,
			SERIALIZERITEM::ACTION action
			);

		static const SERIALIZERITEM* TemplateGetItemDef ();

		PRINTSETTINGS m_HardcopyLayout;			// Used in map/poster layout mode
		double m_PosnScale;						// Scale used for attachment offsets in group positioning (eg. 24000)
		DRECT2D m_FullExtents;					// Extents of entire layout in layout coordinates
		POSNMODE m_PosnMode;						// Positioning mode
		GRIDSIZEMODE m_GridSizeMode;
		GRIDFILLORDER m_GridFillOrder;
		UINT16 m_GridTileCount;
		bool m_IsCreated;							// Has been created
		bool m_NeedUpdatePositions;			// Need to update positions of layout children
		bool m_SortLegendNatural;				// Sort legend in natural order
		SERIALIZER *m_pSerializer;				// Serializer used while loading
		mutable MISTRING m_activegroupname;	// Name of active group used when saving/loading layout
		MISTRING m_SavedLocations;
		UINT8 m_PlacementUnitDisplay;			// Unit for display of "placement" (size/offset) values

		// GRE::OBJECT overrides.
		virtual MICONSTRUCT* v_GetIcon () const;
		virtual const char* v_GetTypeName () const;
		virtual void v_SetModified (bool notify, bool ViewsNeedRedraw);

		// GRE::VIEWABLE overrides.
		virtual void v_Clear ();
		virtual ERRVALUE v_DrawAllViews (GRE::DRAWCONDITION condition);
		virtual GROUP* v_FindGroupGeoref (bool AllowLocalCRS);
		virtual double v_GetDesignScale () const;
		virtual const DRECT2D& v_GetExtents () const;
		virtual double v_GetMaxScaleVisibleAny () const;
		virtual double v_GetMinScaleVisibleAny () const;
		virtual ERRVALUE v_GetSizeMeters (DPOINT2D& sizemeters, bool bUseRenderProjection);
		virtual ERRVALUE v_Save (MDLGPARENT dlgparent, bool forceprompt);
		virtual void v_SetDesignScale (double DesignScale, bool notify, bool updateDft);
		virtual void v_UnhighlightAllElements (bool redraw);

		LAYOUT (const LAYOUT&);
		LAYOUT& operator= (const LAYOUT&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End LAYOUT

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(LAYOUT::CREATEFLAGS);
#endif

//===================================================================================================================

namespace UI {

//! Dialog to edit layout properties/settings.
ERRVALUE DlgLayoutSettings (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	LAYOUT *layout						//!< Layout to edit settings for
	);

}	// End namespace UI

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LAYOUT_H
