/**
 * \file gre/legendimagesample.h
 * \brief GRE::LEGEND_IMAGESAMPLE definitions.
 *
 * \if NODOC
 * $Id: legendimagesample.h_v 1.4 2014/11/19 21:01:51 mju Exp $
 *
 * $Log: legendimagesample.h_v $
 * Revision 1.4  2014/11/19 21:01:51  mju
 * Make addImage public, change other methods to allow more general image input.
 *
 * Revision 1.3  2007/07/02 22:33:12  mju
 * Add addImage.
 *
 * Revision 1.2  2007/06/08 17:46:33  mju
 * Add setThumbnails.
 *
 * Revision 1.1  2006/11/15 22:16:05  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LEGENDIMAGESAMPLE_H
#define	INC_GRE_LEGENDIMAGESAMPLE_H

#ifndef	INC_GRE_LEGEND_H
	#include <gre/legend.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		}
	}
namespace MGD {
	class DEVICE;
	}
#endif

namespace GRE {

class LAYER_IMAGE;

//===================================================================================================================

//! Image Sample legend subclass.
class LEGEND_IMAGESAMPLE : public GRE::LEGEND {
	public:

		//! Location used for sample area.
		enum LOCATION {
			LOCATION_TopLeft =		0,
			LOCATION_TopCenter =		1,
			LOCATION_TopRight =		2,
			LOCATION_CenterLeft =	3,
			LOCATION_Center =			4,
			LOCATION_CenterRight =	5,
			LOCATION_BottomLeft =	6,
			LOCATION_BottomCenter =	7,
			LOCATION_BottomRight =	8,
			};

		//! Construct legend for layer.
		explicit LEGEND_IMAGESAMPLE (
			LAYER_IMAGE *layer							//!< Associated image layer
			);

		//! Destructor.
		~LEGEND_IMAGESAMPLE ();

		//! Add scaled/cropped image to legend.
		//! @return TRUE if item added, FALSE if all cells NULL in specified area, < 0 if error.
		int AddImage (
			SPATMOD::IMAGE::STAGE *pStageInput,		//!< Input image stage already scaled and cropped
			const MISTRING &label,						//!< Label for legend item
			bool bUpdateItemSize = true				//!< Whether to update "item size" for legend, ignored if legend already has items
			);

		//! Add portion of image to legend.
		//! @return TRUE if added, FALSE if all cells null in specified area, < 0 if error.
		int AddSample (
			SPATMOD::IMAGE::STAGE *pStageInput,		//!< Input image stage, will be cropped to specified rectangle
			const LRECT2D &rect,							//!< Portion of image to use
			const MISTRING &label,						//!< Label for legend item
			bool bUpdateItemSize = true				//!< Whether to update "item size" for legend, ignored if legend already has items
			);											

		//! Add thumbnail to legend.
		//! @return TRUE if added, FALSE if all cells are null, < 0 if error.
		int AddThumbnail (
			SPATMOD::IMAGE::STAGE *pStageInput,		//!< Input image stage, will be reduced to specified size
			UINT32 TgtSizeX,								//!< X size of thumbnail in pixels, not including margins
			UINT32 TgtSizeY,								//!< Y size of thumbnail in pixels, not including margins
			const MISTRING &label,						//!< Label for legend item
			bool bUpdateItemSize = true				//!< Whether to update "item size" for legend, ignored if legend already has items
			);											

		//! Set item size.
		//! Usually the item size is set when adding the image.
		//! If specified parameters result in a change to the item size, all legend items will be deleted.
		void SetItemSize (
			INT32 SizeX,									//!< X total size in pixels, including margins
			INT32 SizeY										//!< Y total size in pixels, including margins
			) {
			if (m_ItemSize.x != SizeX || m_ItemSize.y != SizeY) {
				DeleteAllItems();
				m_ItemSize.x = SizeX;
				m_ItemSize.y = SizeY;
				}
			return;
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Individual item in legend.
		class ITEM : public GRE::LEGEND::ITEMBASE {
			public:
				//! Constructor.
				ITEM (
					LEGEND_IMAGESAMPLE &legend,
					MGD::DEVICE *& SampleMem,
					const MISTRING &label
					);

				//! Destructor.
				virtual ~ITEM ();

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MGD::DEVICE *m_pSampleMem;

				// Overrides.
				virtual void v_Draw (const DRAWCONTEXT &dc, const LRECT2D &samplerect) const;
			#ifdef USE_MGUI
				virtual void v_TreeViewCustomDraw (GRE::CTRL_TREEVIEW &TreeView, MGUI::CUSTOMDRAW_TREE &CustomDraw);
			#endif

				ITEM (const ITEM&);
				ITEM& operator= (const ITEM&);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_IMAGE::LEGEND_IMAGESAMPLE::ITEM

		LPOINT2D m_ItemSize;

		// Overrides.
		virtual void v_GetItemSizePixels (LPOINT2D &size) const;

		LEGEND_IMAGESAMPLE (const LEGEND_IMAGESAMPLE&);
		LEGEND_IMAGESAMPLE& operator= (const LEGEND_IMAGESAMPLE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_IMAGE::LEGEND_IMAGESAMPLE class.

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LEGENDIMAGESAMPLE_H
