/**
 * \file gre/lrmapgrd.h
 * \brief GRE::LAYER_MAPGRID class definitions.
 *
 * \if NODOC
 * $Id: lrmapgrd.h_v 1.48 2015/02/05 23:01:15 bcolombe Exp $
 *
 * $Log: lrmapgrd.h_v $
 * Revision 1.48  2015/02/05 23:01:15  bcolombe
 * USE_MGUI
 *
 * Revision 1.47  2014/12/08 16:29:36  mju
 * Check use_mgui for virtual fn.
 *
 * Revision 1.46  2014/06/20 21:23:24  mju
 * Add v_hasProperty.
 *
 * Revision 1.45  2013/12/27 22:24:26  mju
 * Don't use 3d when only need 2d.
 *
 * Revision 1.44  2013/10/11 19:45:11  mju
 * Add copySettings.
 *
 * Revision 1.43  2013/04/12 21:22:38  mju
 * For coordorient allow control of vertical reading and separate for grid/corners.
 *
 * Revision 1.42  2013/04/11 22:27:34  mju
 * Many dispparm changes, use enums, member naming.
 *
 * Revision 1.41  2013/04/08 20:18:44  mju
 * Add m_pdlgControlPanel.
 *
 * Revision 1.37  2011/05/10 13:47:46  mju
 * Add m_gridFlags to dispparm for mapgriddispflag values so don't overlap common flags.
 *
 * Revision 1.36  2010/10/18 13:10:52  mju
 * Add v_snapPoint/IsAllowed.
 *
 * Revision 1.35  2010/02/02 21:52:19  mju
 * Change tick to include location on both grid and map rectangles.
 *
 * Revision 1.34  2010/02/02 18:10:41  mju
 * Add tickFindEdgePoint.
 *
 * Revision 1.33  2009/04/21 17:24:20  mju
 * Add flag_ExcludeFromGroupExtents.
 *
 * Revision 1.32  2008/01/07 18:52:03  mju
 * Add dispflag_noFullCoordLowest.
 *
 * Revision 1.30  2007/10/03 17:34:52  mju
 * Add dispflag_matchGroupExtents.
 *
 * Revision 1.28  2007/05/02 13:31:17  mju
 * Add latlon fmt for DDMM.
 *
 * Revision 1.22  2005/04/21 19:48:28  mju
 * Add xXX and xXXN formatting.
 *
 * Revision 1.9  2003/03/10 16:09:52  mju
 * Save toggled tick coords.
 *
 * Revision 1.7  2003/03/07 23:20:25  mju
 * Add ability to toggle tick colors.
 * Add more UTM label formats.
 *
 * Revision 1.6  2002/11/04 21:01:03  mju
 * Add separate control for corner coords in 2D/3D.
 *
 * Revision 1.5  2002/10/15 22:01:42  mju
 * Add more text formatting options.
 * Add option to show corner labels.
 * Add field for border text offset.
 *
 * Revision 1.4  2002/09/23 17:20:53  mju
 * Add InitExtents to DISPPARM.
 *
 * Revision 1.3  2002/09/20 17:03:19  mju
 * Only declare v_ControlPanel if X or MFC.
 *
 * Revision 1.1  2002/09/10 14:24:51  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRMAPGRD_H
#define	INC_GRE_LRMAPGRD_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif


namespace GRE {

//===================================================================================================================

//! Map Grid layer.
class LAYER_MAPGRID : public GRE::LAYER {
	public:

		enum GRIDFLAGS {
			GRIDFLAG_None =						0,
			GRIDFLAG_SizeRelative =				0x00000001,	//!< Sizes relative to TickScale
			GRIDFLAG_ShowBorderTicksExt =		0x00000002,	//!< Show tick marks outside border
			GRIDFLAG_ShowBorderTicksInt =		0x00000004,	//!< Show tick marks inside border
			GRIDFLAG_NoFullCoordLowest =		0x00000008,	//!< Don't force full coordinates on lowest X/Y value
			GRIDFLAG_ShowGrid =					0x00000010,	//!< Show grid
			GRIDFLAG_ShowBorderTicks =			0x00000020,	//!< Show tick marks at border OLD,
			GRIDFLAG_ShowEdgeText =				0x00000040,	//!< Show edge coordinate text in 2D
			GRIDFLAG_ShowBorder =				0x00000080,	//!< Show border
			GRIDFLAG_ShowInteriorTicks =		0x00000100,	//!< Show interior tick marks
			GRIDFLAG_ShowEdgeText3D =			0x00000200,	//!< Show edge coordinate text in 3D
			GRIDFLAG_ShowBorderBottom3D =		0x00000400,	//!< Show border at Z = zmin in 3D
			GRIDFLAG_ShowBorderTop3D =			0x00000800,	//!< Show border at Z = zmax in 3D
			GRIDFLAG_BorderRelative =			0x00001000,	//!< Interval is relative to borders, not 0
			GRIDFLAG_ShowVerticalTicks =		0x00002000,	//!< Show ticks on vertical axes
			GRIDFLAG_LayoutScale =				0x00004000,	//!< Use layout mapscale
			GRIDFLAG_ShowVerticalScale =		0x00008000,	//!< Show scale on vertical axes
			GRIDFLAG_ShowMinSec0 =				0x00010000,	//!< Show minutes/seconds in lat-lon even if 0
			GRIDFLAG_ShowDegOnlyIfMin0 =		0x00020000,	//!< Show degrees in lat-lon only if minutes = 0
			GRIDFLAG_ShowCornerText =			0x00040000,	//!< Show coordinates at corners
			GRIDFLAG_ShowCornerText3D =		0x00080000,	//!< Show coordinates at corners in 3D (requires ShowCornerText also)
			GRIDFLAG_Antialias =					0x00100000,	//!< Antialias
			GRIDFLAG_HintThinLines =			0x00200000,	//!< Hint Thin Lines via Transparency
			GRIDFLAG_MatchGroupExtents =		0x00400000,	//!< Match group extents
			GRIDFLAG_ExcludeFromGroupExt =	0x00800000,	//!< Exclude from group extents
			GRIDFLAGS_ShowBorderTicksAny =	GRIDFLAG_ShowBorderTicks | GRIDFLAG_ShowBorderTicksExt | GRIDFLAG_ShowBorderTicksInt,
			GRIDFLAGS_MASK_NonStyle =			GRIDFLAG_MatchGroupExtents | GRIDFLAG_ExcludeFromGroupExt
			};

		enum LABELFLAGS {
			LABELFLAG_None =							0,				//!< Note, if no edge/corner flags are set will behave as though all are
			LABELFLAG_EdgeTop =						0x00000001,
			LABELFLAG_EdgeBottom =					0x00000002,
			LABELFLAG_EdgeLeft =						0x00000004,
			LABELFLAG_EdgeRight =					0x00000008,
			LABELFLAG_CornerTopEdgeLeft =			0x00000010,
			LABELFLAG_CornerTopEdgeRight =		0x00000020,
			LABELFLAG_CornerBottomEdgeLeft =		0x00000040,
			LABELFLAG_CornerBottomEdgeRight =	0x00000080,
			LABELFLAG_CornerLeftEdgeTop =			0x00000100,
			LABELFLAG_CornerLeftEdgeBottom =		0x00000200,
			LABELFLAG_CornerRightEdgeTop =		0x00000400,
			LABELFLAG_CornerRightEdgeBottom =	0x00000800,
			LABELFLAG_MASK_Edge =					0x0000000F,
			LABELFLAG_MASK_Corner =					0x00000FF0,
			};

		//! Coordinate label format for geographic CRS.
		enum COORDFMTGEOG {
			COORDFMTGEOG_MASK =			0x000F,
			COORDFMTGEOG_DDMMSS =		0x0000,
			COORDFMTGEOG_DDdddd =		0x0001,
			COORDFMTGEOG_DDMMmmm =		0x0002,
			COORDFMTGEOG_DDMMSSss =		0x0003,
			COORDFMTGEOG_DDMMSSs =		0x0004,
			COORDFMTGEOG_DDMM =			0x0005,
			COORDFMTGEOG_None =			0x000F,
			COORDFMTGEOG_NoSymbol =		0x0020,
			COORDFMTGEOG_AddSpace =		0x0040,
			COORDFMTGEOG_Leading0Deg =	0x0080,
			COORDFMTGEOG_Leading0MS =	0x0100,
			COORDFMTGEOG_DirFirst =		0x0200,
			COORDFMTGEOG_DirLast =		0x0400,
			};

		//! Coordinate label format for projected CRS.
		enum COORDFMTPROJ {
			COORDFMTPROJ_Space =				0,
			COORDFMTPROJ_Comma =				1,
			COORDFMTPROJ_Period =			2,
			COORDFMTPROJ_NoSpace =			3,
			COORDFMTPROJ_xXXxxxN =			4,
			COORDFMTPROJ_xXXxxx =			5,
			COORDFMTPROJ_xXXxxx_XX =		6,
			COORDFMTPROJ_xXXxxxN_XX =		7,
			COORDFMTPROJ_xXXxxxNp_XX =		8,
			COORDFMTPROJ_xXXN =				9,
			COORDFMTPROJ_xXX =				10,
			};

		//! Coordinate label orientation.
		enum COORDORIENT {
			COORDORIENT_Base_Horizontal =				0,
			COORDORIENT_Base_Parallel =				1,
			COORDORIENT_Base_Perpendicular =			2,
			COORDORIENT_Base_AlongTick =				3,
			COORDORIENT_Base_Vertical =				4,
			COORDORIENT_Base_MASK =					0x0F,
			COORDORIENT_VerticalReading_Down =	0x00,
			COORDORIENT_VerticalReading_Out =	0x10,
			COORDORIENT_VerticalReading_In =		0x20,
			COORDORIENT_VerticalReading_Up =		0x30,
			COORDORIENT_VerticalReading_MASK =	0x30,
			};

		//! Which vertical edges to show in 3D perspective.
		enum VERTICALEDGES {
			VERTICALEDGES_None =				0,
			VERTICALEDGES_Farthest1 =		1,
			VERTICALEDGES_Farthest2 =		2,
			VERTICALEDGES_Farthest3 =		3,
			VERTICALEDGES_All =				4,
			};

		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;							//!< Layer settings dialog, defined in <gre/uimapgrd.h>

				TEXTSTYLE m_CoordLabelStyle;
				double m_xinterval;					//!< Interval for non geographic CRS (always stored in meters)
				double m_yinterval;
				double m_zinterval;
				double m_xdegrees;					//!< Interval for geographic CRS (in degrees)
				double m_ydegrees;
				SPATREF::COORDREFSYS m_IntervalCRS;	//!< CRS of grid interval
				SPATREF::COORDREFSYS m_ExtentsCRS;		//!< CRS of grid extents
				DRECT3D m_MapExtents;				//!< Extents (sans ticks/text) in ExtentsProj
				UINT8 m_IntervalUnitsXY;			//!< X-Y interval display units (if not geographic)
				UINT8 m_IntervalUnitsZ;			//!< Z interval display units
				LINESTYLE m_StyleGrid;				//!< Style for grid lines
				LINESTYLE m_StyleBorder;			//!< Style for border/neatline
				LINESTYLE m_StyleBorderTickExt;	//!< Style for ticks outside border
				LINESTYLE m_StyleInteriorTick;	//!< Style for interior ticks
				COLOR m_BorderTickIntColor1;		//!< Primary color for border tick interior
				COLOR m_BorderTickIntColor2;		//!< Alternate color for border tick interior
				COLOR m_InteriorTickColor2;		//!< Alternate color for interior ticks
				double m_GridWidth;					//!< Thickness of grid lines
				double m_BorderWidth;				//!< Thickness of border
				double m_BorderTickExtWidth;		//!< Thickness of exterior border ticks
				double m_BorderTickExtLength;	//!< Outside length of border ticks
				double m_BorderTickIntWidth;		//!< Thickness of interior border ticks
				double m_BorderTickIntLength;	//!< Inside length of border ticks
				double m_InteriorTickWidth;		//!< Thickness of interior ticks
				double m_InteriorTickLength;		//!< Size of interior ticks
				double m_BorderTextOffset;		//!< Offset to border text from edge/tick endpoint
				double m_GraphicMapScale;			//!< Scale for graphics
				UINT8 m_GridWidthUnits;
				UINT8 m_BorderWidthUnits;
				UINT8 m_BorderTickExtWidthUnits;
				UINT8 m_BorderTickExtLengthUnits;
				UINT8 m_BorderTickIntWidthUnits;
				UINT8 m_BorderTickIntLengthUnits;
				UINT8 m_InteriorTickLengthUnits;
				UINT8 m_InteriorTickWidthUnits;
				UINT8 m_BorderTextOffsetUnits;
				UINT8 m_OldProjUnits;					//!< For old map grids used in conversion
				GRIDFLAGS m_GridFlags;					//!< GRIDFLAG values
				LABELFLAGS m_LabelFlags;				//!< Label override flags
				COORDFMTGEOG m_CoordFormatLatLon;	//!< Coordinate format for lat-lon
				COORDFMTPROJ m_CoordFormatProj;		//!< Coordinate format for projection other than lat/lon
				COORDORIENT m_CoordOrientEdge;		//!< Orientation for edge coordinates
				COORDORIENT m_CoordOrientCorner;		//!< Orientation for corner coordinates
				VERTICALEDGES m_VerticalEdges;
				MISTRING m_CoordUnitLabelXY;
				MISTRING m_CoordUnitLabelZ;
				DOUBLE_ARRAY<DPOINT2D> m_ToggledTickCoords;	//!< Coordinates of toggled ticks in 'extents' projection

				DISPPARM ();
				DISPPARM (const DISPPARM& rhs) { Copy(rhs); }
				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Free(); Copy(rhs); } return (*this); }

				MISTRING CoordToString (
					double val,
					bool IsNorthSouth,
					UINT32 flags
					) const;

				// Copy settings without changing object.
				// Does not validate.
				void CopySettingsFrom (
					const DISPPARM& other
					);

				const SPATREF::COORDREFSYS& GetExtentsCRS () const
					{ return (m_ExtentsCRS); }

				//! Initialize extents based on current projection and layer's group extents.
				//! The specified layer's group will be used to determine the extents,
				//! ignoring the layer itself.
				void InitExtents (
					const LAYER_MAPGRID *layer
					);

				void SetExtents (
					const DRECT2D& MapExtents
					) { m_MapExtents.Set(MapExtents,0,0); }

				void SetExtentsCRS (
					const SPATREF::COORDREFSYS& ExtentsCRS
					) { m_ExtentsCRS = ExtentsCRS; }

				void SetIntervalValues (
					const SPATREF::COORDREFSYS& IntervalCRS,	//!< Interval CRS
					double xinterval,									//!< X interval
					double yinterval,									//!< Y interval
					int IntervalUnit = -1							//!< Unit for interval values, if -1 will use from CRS, only applies if projected
					);

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				static int SerializerCB_TickToggledCoords (SERIALIZER& serializer, const SERIALIZERITEM *itemdef, void *cbdata, SERIALIZERITEM::ACTION action);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef LAYER::DISPPARM BASECLASS;
				typedef LAYER_MAPGRID::DISPPARM THISCLASS;
				void Copy (const DISPPARM&);
				void Free ();

				ERRVALUE v_Validate ();
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};

		//! Register this layer type
		static void RegisterType ();

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SML::CONTEXT *context
			);

		static const SML::CLASSMEMBER* GetSMLClassMembers ();

		//! Construct layer and insert into list
		explicit LAYER_MAPGRID (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_MAPGRID ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef LAYER BASECLASS;
		typedef LAYER_MAPGRID THISCLASS;

		class PRIVDRAWDATA;
		friend class PRIVDRAWDATA;

		struct TICK {
			enum LOCATION {
				LOCATION_Interior =		0,
				LOCATION_MinX =			1,
				LOCATION_MaxX =			2,
				LOCATION_MinY =			3,
				LOCATION_MaxY =			4,
				LOCATION_Other =			5
				};
			enum FLAGS {
				FLAG_None =			0x00,
				FLAG_Lowest =		0x01,
				FLAG_AltColor =	0x02,
				FLAG_Corner =		0x04
				};
			DPOINT2D m_pt;						//!< Coordinates in "extents" projection
			DPOINT2D m_pt2;					//!< Secondary point for determining orientation
			double m_label;					//!< X or Y map coordinate for tick point label
			LOCATION m_LocGridRect;
			LOCATION m_LocMapRect;
			FLAGS m_flags;

			TICK (
				const DPOINT2D& pt,
				const DPOINT2D& pt2,
				LOCATION LocGridRect,
				LOCATION LocMapRect,
				FLAGS flags = FLAG_None,
				double label = 0
				);
			};

		TRANS2D_MAPGEN m_TransMapToExt;
		DRECT2D m_GridRect;
		DPOINT2D m_GridInterval;
		SIMPLE_ARRAY<TICK> m_Ticks;
		int m_ActiveTick;
		void *m_LabelList;
		DISPPARM::DLG *m_pDlgControlPanel;

		static void SMLGET_MapGridInterval (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SMLARG*);
		static void SMLSET_MapGridInterval (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SMLARG*);

		void OnMenuToggleTickColor (MGUI::ID id, bool ToggleOn);
		ERRVALUE OnToggleTickColor ();
		void ReadDefaultsIni ();
		void SaveDefaultsIni ();
		ERRVALUE TickAddCornerPoints ();
		ERRVALUE TickAddEdgePoint (const DPOINT2D& label, const DPOINT2D& pt, const DPOINT2D& pt2, TICK::LOCATION type, TICK::FLAGS flags);
		ERRVALUE TickCalcPoints ();
		bool TickFindEdgePoint (const DPOINT2D& ptGridInside, const DPOINT2D& ptGridOutside, const DPOINT2D& ptMapInside, const DPOINT2D& ptMapOutside, TICK::LOCATION TickType, TICK::FLAGS TickFlags);
		ERRVALUE TickLoadToggledCoords ();
		ERRVALUE TickSaveToggledCoords ();

		// Overrides from GRE::LAYER
		virtual void v_ComputeMargins (double& tmargin, double& bmargin, double& lmargin, double& rmargin);
		#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
		#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_CopySettingsFrom (const LAYER *other);
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual bool v_HasProperty (PROPERTY property) const;
	#ifdef USE_MGUI
		virtual void v_MenuAddViewPopupItems (MGUI::MENU& menu, const DPOINT2D *point, VIEW *view);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);
		virtual void v_SetAntialiased (bool antialias);
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual void v_SetHintThinLines (bool hint);
		virtual int v_SnapPoint (DPOINT3D& ptObject, SNAPTO SnapTo, double MaxDistance, VIEW *view) const;
		virtual bool v_SnapPointIsAllowed (SNAPTO SnapTo) const;
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_MAPGRID (const LAYER_MAPGRID&);
		LAYER_MAPGRID& operator= (const LAYER_MAPGRID&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(LAYER_MAPGRID::GRIDFLAGS)
DEFINE_ENUM_OP_BITWISE(LAYER_MAPGRID::LABELFLAGS)
DEFINE_ENUM_OP_BITWISE(LAYER_MAPGRID::COORDFMTGEOG)
DEFINE_ENUM_OP_BITWISE(LAYER_MAPGRID::COORDFMTPROJ)
#endif

//===================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_LRMAPGRD_H
