/**
 * \file gre/rasterthememap.h
 * \brief GRE::RASTERTHEMEMAP definitions
 *
 * \if NODOC
 * $Id: rasterthememap.h_v 1.2 2015/06/12 21:27:54 mju Exp $
 *
 * $Log: rasterthememap.h_v $
 * Revision 1.2  2015/06/12 21:27:54  mju
 * Use single string to keep track of color generation settings for associated form.
 *
 * Revision 1.1  2015/06/11 21:42:45  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_RASTERTHEMEMAP_H
#define	INC_GRE_RASTERTHEMEMAP_H

#ifndef	INC_MI32_SERIALIZABLE_H
	#include <mi32/serializable.h>
#endif

#ifndef	INC_MI32_SERIALIZERITEM_H
	#include <mi32/serializeritem.h>
#endif

#ifndef	INC_MI32_COLORSPREAD_H
	#include <mi32/colorspread.h>
#endif

#ifndef	INC_MI32_COLORPAL_H
	#include <mi32/colorpal.h>
#endif

#ifndef	INC_MI32_COLORPALETTETYPE_H
	#include <mi32/colorpalettetype.h>
#endif

#ifndef	INC_MI32_COLORARRAY_H
	#include <mi32/colorarray.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

#ifndef	INC_MI32_MATH_H
	#include <mi32/math.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class STAT_UNIVARIATE;
#endif

namespace GRE {

//=====================================================================================================================

//! Raster theme mapping parameters.
class RASTERTHEMEMAP : public SERIALIZABLE {
	public:

		//! Distribution mode.
		enum DISTRIB {
			DISTRIB_UserDefined = 0,
			DISTRIB_EqualCount,
			DISTRIB_EqualInterval,
			};

		//! Get serializer item definition.
		static const SERIALIZERITEM* SerialGetItemDef ();

		//! Default constructor.
		RASTERTHEMEMAP ();

		//! Copy constructor.
		RASTERTHEMEMAP (
			const RASTERTHEMEMAP& rhs
			);

		//! Destructor.
		virtual ~RASTERTHEMEMAP ();

		//! Assignment operator.
		RASTERTHEMEMAP& operator= (
			const RASTERTHEMEMAP& rhs
			);

		//! Clear settings.
		void Clear ();

		//! Compute ranges using specified statistics.
		ERRVALUE ComputeRanges (
			const STAT_UNIVARIATE &ImageStats	//!< Image statistics
			);

		//! Get whether to generate colors for all classes.
		bool GetColorGenAllIntervals () const
			{ return (m_bColorGenAllIntervals); }

		//! Get maximum class for color generation.
		INT16 GetColorGenMaxInterval () const
			{ return (m_ColorGenMaxInterval); }

		//! Get minimum class for color generation.
		INT16 GetColorGenMinInterval () const
			{ return (m_ColorGenMinInterval); }

		//! Get color generation value string (for MGUI::FORM_COLORGENERATOR).
		const MISTRING& GetColorGenValueStr () const
			{ return (m_ColorGenValueStr); }

		//! Get distribution mode.
		DISTRIB GetDistribMode () const
			{ return (m_DistribMode); }

		//! Get boundaries of the ranges for the classes.
		const DOUBLE_ARRAY<double>& GetIntervalBoundaries () const
			{ return (m_IntervalBoundaries); }

		//! Get minimum value for specified interval.
		double GetIntervalMinValue (
			UINT32 interval
			) const { return (m_IntervalBoundaries[interval]); }

		//! Get Number of classes in the distribution.
		//! Note it will be one less than the number of class boundaries
		INT16 GetNumIntervals () const
			{ return (m_NumIntervals); }

		//! Get color palette.
		const COLOR_ARRAY& GetPalette () const
			{ return (m_Palette); }

		//! Get rounding mode.
		ROUNDMODE GetRoundMode () const
			{ return (m_RoundMode); }

		//! Get interval to round to.
		double GetRoundTo () const
			{ return (m_RoundTo); }

		//! Read default values from initialization file.
		void IniReadDft ();

		//! Write default values to initialization file.
		void IniWriteDft () const;

		//! Set all class boundaries.
		void SetIntervalBoundaries (
			const DOUBLE_ARRAY<double> &boundaries
			);

		//! Set color for specified class number.
		void SetIntervalColor (
			UINT32 IntervalNum,
			const COLOR &color
			);

		//! Set minimum value for specified class.
		void SetIntervalMinValue (
			UINT32 IntervalNum,
			double MinValue
			);

		//! Set whether color generation applies to all classes.
		void SetColorGenAllIntervals (
			bool bColorGenAllIntervals
			) { m_bColorGenAllIntervals = bColorGenAllIntervals; }

		//! Set range of classes for color generation.
		void SetColorGenIntervalRange (
			INT16 MinInterval,
			INT16 MaxInterval
			) { m_ColorGenMinInterval = MinInterval; m_ColorGenMaxInterval = MaxInterval; }

		//! Set color generator value string.
		void SetColorGenValueStr (
			const MISTRING &ValueStr		//!< Values from MGUI::FORM_COLORGENERATOR::GetValueStr
			) { m_ColorGenValueStr = ValueStr; }

		//! Set the distribution mode.
		void SetDistribMode (DISTRIB distribmode)
			{ m_DistribMode = distribmode; }

		//! Set number of class intervals.
		ERRVALUE SetNumIntervals (
			INT16 numintervals
			);

		//! Set palette.
		void SetPalette (
			const COLOR_ARRAY &palette
			);

		//! Set the Rounding mode.
		void SetRoundMode (ROUNDMODE roundmode)
			{ m_RoundMode = roundmode; }

		//! Set Interval to round to.
		void SetRoundTo (double roundto)
			{ m_RoundTo = roundto; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef RASTERTHEMEMAP THISCLASS;

		DOUBLE_ARRAY<double> m_IntervalBoundaries;	// Boundaries of the ranges for the classes
		COLOR_ARRAY m_Palette;
		MISTRING m_ColorGenValueStr;						// Color Generator value string
		double m_RoundTo;										// Interval to round to
		INT16 m_NumIntervals;								// Number of classes
		INT16 m_ColorGenMinInterval;						// Starting class for color generation
		INT16 m_ColorGenMaxInterval;						// Ending class for color generation
		DISTRIB m_DistribMode;								// Distribution mode, DISTRIB_...
		ROUNDMODE m_RoundMode;								// Rounding mode, ROUNDMODE_...
		bool m_bColorGenAllIntervals;						// Set if applies to all classes

		static int SerializerCB_IntervalBoundaries (SERIALIZER &serializer, const SERIALIZERITEM *, void *vdata, SERIALIZERITEM::ACTION action);

		ERRVALUE SerialReadIntervalBoundaries (SERIALIZER &serializer);
		ERRVALUE SerialWriteIntervalBoundaries (SERIALIZER &serializer) const;

		// Overrides from SERIALIZABLE.
		virtual ERRVALUE SerialRead (SERIALIZER& serializer);
		virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;
		#endif
	};

//=====================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_RASTERTHEMEMAP_H
