/**
 * \file gre/uiviewtooldefineregion.h
 * \brief GRE::UI::VIEWTOOL_DEFINEREGION definition
 *
 * \if NODOC
 * $Id: uiviewtooldefineregion.h_v 1.6 2014/08/25 17:29:06 mju Exp $
 *
 * $Log: uiviewtooldefineregion.h_v $
 * Revision 1.6  2014/08/25 17:29:06  mju
 * Add ctor using form_view so can control sidebar panel visibility.
 *
 * Revision 1.5  2014/05/12 19:13:36  mju
 * Add clearRegion, delegates for activate/deactive, option to hide layer on deactivate.
 * Add parm to setRegion to update tgt crs.
 *
 * Revision 1.4  2014/05/09 16:29:29  mju
 * Add ctor with name, icon, tool id.
 *
 * Revision 1.3  2014/04/21 15:10:59  mju
 * Add accessRegionLayer.
 *
 * Revision 1.2  2014/04/03 17:02:27  bcolombe
 * Add delegate when region changes
 * Add SetGroup to define group to create region in when viewable is layout
 *
 * Revision 1.1  2014/03/10 17:44:10  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UIVIEWTOOLDEFINEREGION_H
#define	INC_GRE_UIVIEWTOOLDEFINEREGION_H

#ifndef	INC_GRE_UIVIEWTOOL_H
#include <gre/uiviewtool.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
namespace GRE {
	class LAYER_REGION;
	}
namespace MGUI {
	class FORM_REGIONMANAGER;
	}
namespace SPATREF {
	class COORDREFSYS;
	}
#endif

namespace GRE {
namespace UI {

//===================================================================================================================

//! Tool allowing user to define region.
class VIEWTOOL_DEFINEREGION : public GRE::UI::VIEWTOOL {
	public:
		typedef fastdelegate::FastDelegate<void(const REGION2D&)> DELEGATE_ONCHANGEREGION;

		//! Construct for FORM_VIEW.
		//! Automatically attaches region manager from sidebar and sets up to show panel when needed.
		explicit VIEWTOOL_DEFINEREGION (
			GRE::UI::FORM_VIEW& FormView			//!< View form
			);

		//! Construct with default name, icon, tool id.
		explicit VIEWTOOL_DEFINEREGION (
			GRE::UI::VIEW_CANVAS *view				//!< View canvas to contain tool
			);

		//! Construct with specified name, icon, optional tool id.
		VIEWTOOL_DEFINEREGION (
			GRE::UI::VIEW_CANVAS *view,			//!< View canvas to contain tool
			const MISTRING& name,					//!< Tool name to display
			ICONID iconid,								//!< Icon id
			VIEWTOOLID toolid = VIEWTOOLID_Unspecified	//!< Tool id
			);

		//! Destructor.
		virtual ~VIEWTOOL_DEFINEREGION ();

		//! Access layer used to display region.
		//! @return Pointer to layer or NULL if not yet defined.
		LAYER_REGION* AccessRegionLayer ();

		//! Attach region manager form.
		void AttachRegionManagerForm (
			MGUI::FORM_REGIONMANAGER *pForm		//!< Form to attach
			);

		//! Clear region.
		//! Does not call delegate for region change.
		void ClearRegion ();

		//! Get current region.
		const REGION2D& GetRegion ();

		//! Set delegate to call when tool activated.
		void SetDelegateOnActivate (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when region changed.
		void SetDelegateOnChangeRegion (
			DELEGATE_ONCHANGEREGION delegate
			);

		//! Set delegate to call when tool deactivated.
		void SetDelegateOnDeactivate (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set group to create region layer in, will use viewable by default.
		//! Must use this if viewable is a layout.
		void SetGroup (
			GRE::GROUP *group
			);

		//! Set whether to automatically hide region layer when tool is deactivated.
		void SetHideRegionLayerOnDeactivate (
			bool bHide = true
			);

		//! Set region.
		//! Does not call delegate for region change.
		void SetRegion (
			const REGION2D& region,					//!< Region to set
			bool bUpdateTgtCRS = false				
			);

		//! Set target CRS for region.
		void SetTgtRegionCRS (
			const SPATREF::COORDREFSYS& crs
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		virtual bool v_Activate ();
		virtual void v_Deactivate ();
		virtual bool v_Initialize ();
		virtual void v_Suspend ();
		virtual void v_Resume ();

		#endif
	};

//===================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UIVIEWTOOLDEFINEREGION_H
