/**
 * \file mgui/comboedit.h
 * \brief MGUI::COMBOEDIT definitions
 *
 * \if NODOC
 * $Id: comboedit.h_v 1.1 2013/12/02 21:56:21 mju Exp $
 *
 * $Log: comboedit.h_v $
 * Revision 1.1  2013/12/02 21:56:21  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_COMBOEDIT_H
#define	INC_MGUI_COMBOEDIT_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MI32_RANGE_H
#include <mi32/range.h>
#endif

#ifndef	INC_MI32_LATLONFM_H
#include <mi32/latlonfm.h>
#endif

#ifndef	INC_MI32_UNITCONV_H
#include <mi32/unitconv.h>
#endif

namespace MGUI {

//===================================================================================================================

//! Combination control allowing selection from dropdown or direct editing.
class CTRL_COMBOEDIT : public CTRL_COMBOBOX {
	public:

		//! Flags for creation.
		enum FLAGS {
			FLAG_Default =			0,
			FLAG_AlignLeft =		0x01,	//!< Left-align value (default for text)
			FLAG_AlignRight =		0x02,	//!< Right-align value (default for numeric)
			FLAG_NoMin =			0x04,	//!< No minimum numeric value enforced
			FLAG_NoMax =			0x08,	//!< No maximum numeric value enforced
			FLAG_NoRange =			(FLAG_NoMin|FLAG_NoMax),	//!< No range enforcement
			FLAG_NoDefault =		0x10,	//!< No default specified in creation (numeric)
			FLAG_ReadOnly =		0x20,
			};

		//! Value formatting.
		enum FORMAT {
			FORMAT_Text,					//!< Text value
			FORMAT_Decimal,				//!< Decimal -123.456
			FORMAT_Exponential,			//!< Exponential -1.234E+10
			FORMAT_Latitude,				//!< Latitude [N/S] dd mm ss.ss
			FORMAT_Longitude,				//!< Longitude [E/W] ddd mm ss.ss
			FORMAT_DegMinSec,				//!< Angle ddd mm ss.ss
			FORMAT_ReciprocalScale		//!< 1:NNNN
			};

		//! Constructor.
		CTRL_COMBOEDIT ();

		//!< Destructor.
		virtual ~CTRL_COMBOEDIT ();

		//! Clear the value.
		void ClearValue (
			bool notify = true							//!< Call OnChangeValue() if control has already been created
			);

		//! Create control for text value.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			int maxlen,										//!< Maximum number of characters allowed in string, not including termination
			int width = 0,									//!< Width in 'typical' characters, minimum if can expand based on sizealign
			FLAGS flags = FLAG_Default,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Create control for numeric value.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			double dft,										//!< Default value
			double min,										//!< Minimum value
			double max,										//!< Maximum value
			int places = 0,								//!< Number of decimal places to show, -1 for automatic
			int width = 0,									//!< Width in "characters", 0 to automatically determine based on min/max/places
			FLAGS flags = FLAG_Default,				//!< Flags
			FORMAT format = FORMAT_Decimal,			//!< Format
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get current format setting.
		FORMAT GetFormat () const
			{ return (m_Format); }

		//! Get allowed numeric range.
		//! @return Allowed range.
		const DOUBLE_RANGE& GetRange (
			) const { return (m_range); }

		//! Get current numeric value.
		double GetValueNumeric ();

		//! Get current string value.
		const MISTRING& GetValueText ();

		//! Determine if has valid value.
		bool HasValue ();

		//! Read selection/value from INI file.
		//! Create must be done before using.
		void IniRead (
			INIHANDLE IniHandle,							//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,						//!< INI group to read from
			const char *IniField,						//!< INI field to read from
			bool notify = true							//!< Call OnChangeValue() if control has already been created
			);

		//! Write current selection/value to INI file.
		//! Create must be done before using.
		void IniWrite (
			INIHANDLE IniHandle,							//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,						//!< INI group to read from
			const char *IniField							//!< INI field to read from
			) const;

		//! Set delegate to call when user presses \<Enter\> in edit control.
		void SetDelegateOnActivate (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnActivate = delegate; }

		//! Set delegate to call when value is changed after validation.
		//! If the user modifies the text in the edit control will not be called until the control loses "focus".
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeValue = delegate; }

		//! Set delegate to call when user modifies the text in the edit control.
		//! The actual value does not get updated until the control loses focus or GetValue() is called.
		void SetDelegateOnUserEdit (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnUserEdit = delegate; }

		//! Set delegate to call to perform validation.
		void SetDelegateOnValidate (
			FastDelegate<bool(MISTRING&)> delegate
			) { m_DelegateOnValidate = delegate; }

		//! Set format for showing/entering values.
		//! Setting to Latitude, Longitude or DegMinSec will also adjust range.
		void SetFormat (
			FORMAT format,
			int places = -2,								//!< Decimal places, -1 for automatic, -2 to leave unchanged
			bool notify = true							//!< Notify if value changes as a result
			);

		//! Set allowed range.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetRange (
			const DOUBLE_RANGE& range,					//!< New range to set
			bool notify = true,							//!< Call OnChangeValue() if value adjusted to new range
			FLAGS flags = FLAG_Default					//!< FLAG_NoMin, FLAG_NoMax, FLAG_NoRange
			);

		//! Set allowed range.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetRange (
			double minimum,								//!< New minimum value
			double maximum,								//!< New maximum value
			bool notify = true,							//!< Call OnChangeValue() if value adjusted to new range
			FLAGS flags = FLAG_Default					//!< FLAG_NoMin, FLAG_NoMax, FLAG_NoRange
			) { return (SetRange(DOUBLE_RANGE(minimum,maximum),notify,flags)); }

		//! Set set of 'valid' or 'invalid' characters.
		//! This does not alter the current contents, if any.
		void SetValidChars (
			const MISTRING& string,						//!< Characters valid or invalid to enter
			bool invert = false							//!< Allow all but specified characters
			);

		//! Set current numeric value.
		void SetValue (
			double value,									//!< Numeric value to set
			bool notify = true
			);

		//! Set current string value with validation.
		//! OnValidate() will be called and if valid, the string will be updated.
		//! No comparison is made with the current string value.
		void SetValue (
			const MISTRING& string,						//!< New string to set
			bool notify = true							//!< Call OnChangeValue() if string actually updated
			);

		//! Set unit conversion for displayed value.
		void SetUnitConversion (
			const UNITCONV& unitconv
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef CTRL_COMBOBOX BASECLASS;
		typedef CTRL_COMBOEDIT THISCLASS;

		DELEGATE_VOID_NOPARMS m_DelegateOnActivate;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeValue;
		DELEGATE_VOID_NOPARMS m_DelegateOnUserEdit;
		FastDelegate<bool(MISTRING&)> m_DelegateOnValidate;
		MISTRING m_ValidChars;
		MISTRING m_valuestr;
		double m_valuenum;
		DOUBLE_RANGE m_range;
		UNITCONV m_unitconv;
		int m_MaxLength;
		FORMAT m_Format;
		FLAGS m_flags;
		DMSTYPE m_DegToStrFlags;
		INT16 m_places;
		INT16 m_dmsplaces;
		bool m_bUserEdited;
		bool m_bHasValue;
		bool m_bInternalModify;
		bool m_bValidInvert;

		static void CB_Activate (Widget, THISCLASS*, void*);
		static void CB_Focus (Widget, THISCLASS*, void*);
		static void CB_LosingFocus (Widget, THISCLASS*, void*);
		static void CB_ModifyVerify (Widget, THISCLASS*, void*);
		static void CB_ValueChanged (Widget, THISCLASS*, void*);

		bool CheckRange (double& value) const;
		bool CheckValidChars (MISTRING& string) const;
		void CreateCtrl (LAYOUT_PANE_BASE& ParentPane, int width, FLAGS flags, LAYOUT_SIZEALIGN sizealign);
		bool GetEditStr (MISTRING& string) const;
		void NotifyChangeValue ();
		void OnActivate ();
		void OnChangeSelection ();
		void OnUserEdit ();
		bool OnValidate (MISTRING& string) const;
		void SetCursorPos (int nCharIndex);
		void UpdateCtrl (const MISTRING& string);
		void UpdateText ();
		void UpdateValue (bool notify);

		virtual void v_OnSelection ();
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_COMBOEDIT::FLAGS)
#endif

//===================================================================================================================

//! Form containing label and COMBOEDIT control.
//! Access to the COMBOEDIT control for setting/getting values/options is via the GetCtrl methods.
class FORM_COMBOEDIT : public FORM_COMPOSITE {
	public:

		//! Default constructor.
		FORM_COMBOEDIT ();

		//! Destructor.
		virtual ~FORM_COMBOEDIT ();

		//! Create form for text value.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,						//!< Label string
			int maxlen,
			int width = 0,									//!< Width in 'typical' characters, 0 for default
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize,
			CTRL_COMBOEDIT::FLAGS flags = CTRL_COMBOEDIT::FLAG_Default,
			CTRL_LABEL::STYLE labelstyle = CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Create form for numeric value.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,						//!< Label string
			double dft,										//!< Default value
			double min,										//!< Minimum value
			double max,										//!< Maximum value
			int places = 0,								//!< Number of decimal places to show, -1 for automatic
			int width = 0,									//!< Width in "characters", 0 to automatically determine based on min/max/places
			CTRL_COMBOEDIT::FLAGS flags = CTRL_COMBOEDIT::FLAG_Default,
			CTRL_COMBOEDIT::FORMAT format = CTRL_COMBOEDIT::FORMAT_Decimal,			//!< Format
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize,
			CTRL_LABEL::STYLE labelstyle = CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Get CTRL_COMBOEDIT (non-const).
		CTRL_COMBOEDIT& GetCtrl ()
			{ return (m_comboedit); }

		//! Get CTRL_COMBOEDIT (const).
		const CTRL_COMBOEDIT& GetCtrl () const
			{ return (m_comboedit); }

		//! Get label control if any.
		//! Usually the label control is only retrieved for alignment purposes.
		CTRL_LABEL& GetLabel (
			) { return (m_label); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_COMBOEDIT m_comboedit;
		CTRL_LABEL m_label;

		FORM_COMBOEDIT (const FORM_COMBOEDIT&);
		FORM_COMBOEDIT& operator= (const FORM_COMBOEDIT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_COMBOEDIT_H
