/**
 * \file mgui/formpaneled.h
 * \brief MGUI::FORM_PANELED definition
 *
 * \if NODOC
 * $Id: formpaneled.h_v 1.5 2014/03/11 14:13:02 mju Exp $
 *
 * $Log: formpaneled.h_v $
 * Revision 1.5  2014/03/11 14:13:02  mju
 * Use begin/end delegates for panel visibility change notification.
 *
 * Revision 1.4  2014/01/21 13:57:54  mju
 * Add getPanelID.
 *
 * Revision 1.3  2013/12/13 20:48:49  mju
 * Add setDelegateOnSashMoved.
 *
 * Revision 1.2  2013/03/07 13:44:48  mju
 * Add getIndexFromID.
 *
 * Revision 1.1  2013/01/30 17:48:07  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMPANELED_H
#define	INC_MGUI_FORMPANELED_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MI32_ICONID_H
#include <mi32/iconid.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Form containing multiple "panels", any number of which may be visible at one time.
class FORM_PANELED : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum FORMSTYLE {
			FORMSTYLE_Default =						0,
			FORMSTYLE_VisibilityControlsFirst =	0x01,		//!< Place visibility controls before other panels (top or left)
			FORMSTYLE_VisibilityControlsLast =	0x02,		//!< Place visibility controls after other panels (bottom or right)
			};

		//! Panel styles.
		enum PANELSTYLE {
			PANELSTYLE_FixedSize =			0x00,		//!< Panel size (in form orientation direction) is fixed
			PANELSTYLE_Resizable =			0x01,		//!< Panel is resizable by user in form orientation direction
			PANELSTYLE_IgnoreNonVisible =	0x02,		//!< Ignore non-visible items in panel
			PANELSTYLE_DftHidden =			0x04,		//!< Default as hidden
			};

		//! Delegate for notifying of panel visibility change.
		//! First parameter is panel index. Second parameter is 'false' before change (begin) and 'true' after (end).
		typedef fastdelegate::FastDelegate<void(INT32,bool)> DELEGATE_CHANGEPANELVISIBLE;

		//! Constructor.
		FORM_PANELED ();

		//! Destructor.
		virtual ~FORM_PANELED ();

		//! Access pane for form.
		LAYOUT_PANE& AccessPaneForm ()
			{ return (GetPane()); }

		//! Access pane for toolbar.
		LAYOUT_PANE& AccessPaneToolBar ();

		//! Add panel to form with no icon for user control of visibility.
		//! Must be called after CreateBegin().
		//! @return Panel index, use GetPanelPane() to access pane for creating controls in panel.
		int AddPanel (
			PANELSTYLE panelstyle,							//!< Panel style
			LAYOUT_ORIENTATION orientation,				//!< Panel orientation
			const char *IniKey = 0,							//!< Key for saving settings, NULL if not saveable
			int ChildSpacing = 4,							//!< Spacing between panel children
			int ExtraBorder = 0								//!< Extra border around panel
			);

		//! Add panel to form with icon for user control of visibility.
		//! Must be called after CreateBegin().
		//! @return Panel index, use GetPanelPane() to access pane for creating controls in panel.
		int AddPanel (
			PANELSTYLE panelstyle,							//!< Panel style
			LAYOUT_ORIENTATION orientation,				//!< Panel orientation
			ICONID iconid,										//!< Icon id for visibility control button
			const MISTRING& label,							//!< Label for icon tooltip
			const char *IniKey = 0,							//!< Key for saving settings, NULL if not saveable
			int ChildSpacing = 4,							//!< Spacing between panel children
			int ExtraBorder = 0								//!< Extra border at edges of panel
			);

		//! Begin creating form.
		void CreateBegin (
			LAYOUT_PANE_BASE& PaneParent,					//!< Parent for form
			LAYOUT_ORIENTATION orientation,				//!< Form orientation
			FORMSTYLE formstyle = FORMSTYLE_Default,	//!< Form style
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_Expand,
			int ExtraBorder = 0								//!< Extra border at edges of form
			);

		//! End form creation.
		//! Must only be done after all panels are added.
		void CreateEnd ();

		//! Get zero-based panel index from ID.
		//! @return Panel index or -1 if none matching id found.
		int GetIndexFromID (
			MGUI::ID id											//!< Panel ID
			) const;

		//! Get panel ID from index.
		MGUI::ID GetPanelID (
			int panelindex										//!< Panel index
			) const;

		//! Get reference to pane for specified panel index.
		//! Do not do SetVisible on returned LAYOUT_PANE, visibility must be changed via SetPanelVisible().
		LAYOUT_PANE& GetPanelPane (
			int panelindex										//!< Panel index
			);

		//! Determine if panel is visible.
		bool IsPanelVisible (
			int panelindex										//!< Panel index
			) const;

		//! Determine if panel is visible by ID.
		bool IsPanelVisibleID (
			MGUI::ID id											//!< Panel ID
			) const;

		//! Set delegate to call before panel visibility about to be changed.
		//! Note that delegate is not called is panel visibility is changed programmatically.
		void SetDelegateOnChangePanelVisibleBegin (
			DELEGATE_CHANGEPANELVISIBLE delegate		//!< Delegate to call, parameter is panel index and 'false'
			);

		//! Set delegate to call after panel visibility has been changed.
		//! Note that delegate is not called is panel visibility is changed programmatically.
		void SetDelegateOnChangePanelVisibleEnd (
			DELEGATE_CHANGEPANELVISIBLE delegate		//!< Delegate to call, parameter is panel index and 'true'
			);

		//! Set delegate to call if user tries to hide all panels.
		//! If delegate is set, the last visible panel will not be hidden.  This allows the entire form to
		//! instead be hidden or a default panel to be set as visible.  Note that delegate is not called if
		//! panel is hidden programmatically.
		void SetDelegateOnHideAllPanels (
			DELEGATE_VOID_NOPARMS delegate				//!< Delegate
			);

		//! Set delegate to call when sash between panels is moved.
		void SetDelegateOnSashMoved (
			DELEGATE_VOID_NOPARMS delegate				//!< Delegate
			);

		//! Set panel ID.
		void SetPanelID (
			int panelindex,									//!< Panel index
			MGUI::ID id											//!< ID
			);

		//! Set whether panel is visible.
		void SetPanelVisible (
			int panelindex,									//!< Panel index
			bool visible										//!< Whether is visible
			);

		//! Set whether panel is visible by ID.
		void SetPanelVisibleID (
			MGUI::ID id,										//!< Panel ID
			bool visible										//!< Whether is visible
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_PANELED (const FORM_PANELED&);
		FORM_PANELED& operator= (const FORM_PANELED&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_PANELED::FORMSTYLE)
DEFINE_ENUM_OP_BITWISE(FORM_PANELED::PANELSTYLE)
#endif

//=====================================================================================================================

}

#endif	// INC_MGUI_FORMPANELED_H
