/**
 * \file mgui/formtogglebuttons.h
 * \brief MGUI::FORM_TOGGLEBUTTONS definition
 *
 * \if NODOC
 * $Id: formtogglebuttons.h_v 1.6 2015/06/05 15:12:44 mju Exp $
 *
 * $Log: formtogglebuttons.h_v $
 * Revision 1.6  2015/06/05 15:12:44  mju
 * Add setButtonLabel and style_ButtonsExpandable.
 *
 * Revision 1.5  2015/04/15 21:58:21  mju
 * Add style_HighlightOnMouseOver.
 *
 * Revision 1.4  2015/02/20 15:03:12  mju
 * Add style_FirstOwnRow.
 *
 * Revision 1.3  2014/12/12 17:34:02  mju
 * Add setSelectedAll.
 *
 * Revision 1.2  2014/07/23 21:03:25  mju
 * Change default for spacing so can be different for horizontal vs vertical.
 *
 * Revision 1.1  2014/07/23 20:32:33  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMTOGGLEBUTTONS_H
#define	INC_MGUI_FORMTOGGLEBUTTONS_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

namespace MGUI {

//===================================================================================================================

//! Form containing set of toggle buttons.
//! Buttons may use Radio or Check style
class FORM_TOGGLEBUTTONS : public MGUI::FORM_COMPOSITE {
	public:

		enum STYLE {
			STYLE_Check =			CTRL_TOGGLEBUTTON::STYLE_Check,			//!< Checkbox style
			STYLE_Radio =			CTRL_TOGGLEBUTTON::STYLE_Radio,			//!< Radiobutton style
			STYLE_HighlightOnMouseOver =	CTRL_TOGGLEBUTTON::STYLE_HighlightOnMouseOver,	//!< Only show highlight frame on mouseover, cannot combine with STYLE_NoFrame, will show shadow when "pressed".
			STYLE_NoMargin =		CTRL_TOGGLEBUTTON::STYLE_NoMargin,		//!< Don't include margins for icon or margin above/below for text
			STYLE_NoFrame =		CTRL_TOGGLEBUTTON::STYLE_NoFrame,		//!< Don't include frame for icon, cannot combine with STYLE_HighlightOnMouseOver
			STYLE_MinSpacing =	CTRL_TOGGLEBUTTON::STYLE_MinSpacing,	//!< Use minimal spacing between indicator and label (non-icon)
			STYLE_CheckAtLeastOne =			0x0100,								//!< Checkbox style with at least one item selected
			STYLE_RadioAllowNone =			0x0200,								//!< Radiobutton style, allow to have no item selected
			STYLE_NotifyAutoToggleOff =	0x0400,								//!< Notify when button is automatically toggled off due to exclusive/radio behavior
			STYLE_FirstOwnRow =				0x0800,								//!< Put first (usually exclusive) button in own row (only for vertical multi-column arrangement)
			STYLE_ButtonsExpandable =		0x1000,								//!< Buttons are expandable (use when changing labels)
			};

		//! Constructor.
		FORM_TOGGLEBUTTONS ();

		//! Destructor.
		virtual ~FORM_TOGGLEBUTTONS ();

		//! Add icon button with ICON.
		//! Button may be added before or after Create.
		void AddButton (
			ID id,											//!< ID to attach to button
			const ICON& icon,								//!< Icon
			const MISTRING& tooltip,					//!< ToolTip string
			bool bSelected = false,						//!< Whether initially selected
			bool bExclusive = false						//!< Button is "exclusive" (for checkbox style)
			);

		//! Add icon button with ICONID.
		//! Button may be added before or after Create.
		void AddButton (
			ID id,											//!< ID to attach to button
			ICONID iconid,									//!< Icon ID
			const MISTRING& tooltip,					//!< ToolTip string
			UINT16 iconsize = 0,							//!< Icon size, 0 for default
			bool bSelected = false,						//!< Whether initially selected
			bool bExclusive = false						//!< Button is "exclusive" (for checkbox style)
			);

		//! Add button with text label (no icon).
		//! Button may be added before or after Create.
		void AddButton (
			ID id,											//!< ID to attach to button
			const MISTRING& string,						//!< String for button
			bool bSelected = false,						//!< Whether initially selected
			bool bExclusive = false						//!< Button is "exclusive" (for checkbox style)
			);

		//! Create form with buttons in single column or row.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			LAYOUT_ORIENTATION orientation = LAYOUT_ORIENTATION_Vertical,		//!< Orientation
			STYLE style = STYLE_Check,					//!< Style
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize,
			int Spacing = -1,								//!< Spacing between buttons, -1 for default based on orientation
			int Margin = 0									//!< Margin around form
			);

		//! Create form with buttons in multiple rows or columns.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			LAYOUT_ORIENTATION orientation,			//!< Primary orientation
			int NumPerRowOrColumn,						//!< Number of buttons per row (for Horizontal) or column (for Vertical)
			STYLE style = STYLE_Check,					//!< Style
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize,
			int Spacing = -1,								//!< Spacing between buttons, -1 for default based on orientation
			int Margin = 0									//!< Margin around form
			);

		//! Get ICON for button.
		const ICON& GetIcon (
			ID id												//!< ID of button
			) const;

		//! Get ID of next selected button.
		//! @return ID or -1 if no more selected
		ID GetNextSelectedID (
			ID id = -1										//!< Previous ID, -1 to find first
			) const;

		//! Get ID of currently selected button.
		//! @return ID or -1 if don't have exactly one button selected.
		ID GetSelectedID () const;

		//! Determine if button is selected.
		bool IsSelectedID (
			ID id												//!< ID of button
			) const;

		//! Set button label.
		void SetButtonLabel (
			ID id,
			const MISTRING &label
			);

		//! Set delegate to all when button state is toggled.
		void SetDelegateOnToggle (
			DELEGATE_VOID_ID_BOOL delegate
			);

		//! Set whether individual button is enabled.
		//! Does not affect whether button is selected.
		void SetEnabledID (
			ID id,											//!< ID of button to enable/disable
			bool enabled = true							//!< Whether enabled or not
			);

		//! Set to have all buttons selected.
		//! Should only be used if form has no mutually-exclusive buttons.
		void SetSelectedAll (
			bool notify = false							//!< Whether to notify via delegate (for each button toggled on)
			);

		//! Set whether button is selected.
		//! If selected button is exclusive or using radio style, other buttons will be toggled off.
		//! In that case, notification for those buttons depends on whether "notify" is set and use of
		//! STYLE_NotifyAutoToggleOff in creation.
		//! This function does not force having a button toggled on regardless of STYLE setting.
		void SetSelectedID (
			ID id,											//!< ID of button to select/deselect
			bool selected = true,						//!< Whether selected or not
			bool notify = false							//!< Whether to notify via delegate if state changed
			);

		//! Set to have no buttons selected.
		void SetSelectedNone (
			bool notify = false							//!< Whether to notify via delegate (for each button toggled off)
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_TOGGLEBUTTONS (const FORM_TOGGLEBUTTONS&);
		FORM_TOGGLEBUTTONS& operator= (const FORM_TOGGLEBUTTONS&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_TOGGLEBUTTONS::STYLE)
#endif

//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMTOGGLEBUTTONS_H
