/**
 * \file mgui/treeregionmanager.h
 * \brief MGUI::CTRL_TREE_REGIONMANAGER definitions
 *
 * \if NODOC
 * $Id: treeregionmanager.h_v 1.6 2013/11/01 13:34:33 mju Exp $
 *
 * $Log: treeregionmanager.h_v $
 * Revision 1.6  2013/11/01 13:34:33  mju
 * Make observer a pointer so can delay construction until creation.
 *
 * Revision 1.5  2013/10/29 21:48:50  mju
 * Add parm to setUseCheckBoxes.
 *
 * Revision 1.4  2013/10/23 17:39:37  bcolombe
 * Make AddNode public to allow manually adding node to tree
 *
 * Revision 1.3  2013/10/21 19:57:59  mju
 * Don't nest treeview node/folder/object.
 *
 * Revision 1.2  2013/10/21 19:26:54  mju
 * Use regionsource.
 *
 * Revision 1.1  2013/10/21 18:48:48  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TREEREGIONMANAGER_H
#define	INC_MGUI_TREEREGIONMANAGER_H

#ifndef	INC_MGUI_TREE_H
#include <mgui/tree.h>
#endif

#ifndef	INC_MI32_REGIONMANAGER_H
#include <mi32/regionmanager.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Tree control to select managed regions.
//! When using this class do not alter the IDs of items in the tree. These IDs correspond to TREEVIEWNODE instances
//! in REGIONMANAGER.
class CTRL_TREE_REGIONMANAGER : public CTRL_TREE {
	public:

		//! List of "checked" source pointers.
		typedef MILIST<REGIONSOURCE*> CHECKEDLIST;

		//! Constructor.
		CTRL_TREE_REGIONMANAGER ();

		//! Destructor.
		virtual ~CTRL_TREE_REGIONMANAGER ();

		//! Add node to tree
		bool AddNode (ITEMHND hndParent, ITEMHND hndInsertAfter, const TREEVIEWNODE *pNode);

		//! Clear source checkboxes.
		void ClearCheckedSourceList (
			bool bNotify = true
			);

		//! Get currently active source, if any.
		//! The "active" source is indicated by inverted text in the tree.
		//! @return Null if no source is active.
		REGIONSOURCE* GetActiveSource () const
			{ return (GetSource(GetSelectedItem())); }

		//! Get list of "checked" sources.
		const CHECKEDLIST& GetCheckedSourceList () const
			{ return (m_CheckedList); }

		//! Get source corresponding to ITEMHND.
		//! @return Null if specified item handle does not refer to valid SOURCE.
		TREEVIEWNODE* GetNode (
			ITEMHND hndItem
			) const { return (static_cast<TREEVIEWNODE*>(reinterpret_cast<void*>(GetItemID(hndItem)))); }

		//! Get ITEMHND corresponding to TREEVIEW NODE.
		ITEMHND GetNodeHnd (
			const TREEVIEWNODE *pNode
			) const;

		//! Get source corresponding to ITEMHND.
		//! @return Null if specified item handle does not refer to valid SOURCE.
		REGIONSOURCE* GetSource (
			ITEMHND hndItem
			) const;

		void SetDelegateOnChangeCheckedList (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeCheckedList = delegate; }

		//! Set to use checkboxes for sources.
		void SetUseCheckBoxes (
			bool bUseCheckBoxes			//!< Whether checkboxes are shown
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		typedef CTRL_TREE BASECLASS;

		enum STATEIDX {
			STATEIDX_None,
			STATEIDX_Unchecked,
			STATEIDX_Checked,
			};

		class OBSERVER;

		TREEVIEW::OBSERVER *m_pObserver;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeCheckedList;
		CHECKEDLIST m_CheckedList;
		bool m_bUseCheckBoxes;

		static MGUI::ID NodeToID (const TREEVIEWNODE *pNode)
			{ return (reinterpret_cast<MGUI::ID>(static_cast<const void*>(pNode))); }

		void DeleteNodeItem (TREEVIEWNODE *pNode, ITEMHND hndItem);
		ITEMHND GetNodeHndPrevious (ITEMHND hndParent, const TREEVIEWNODE *pNode) const;
		void OnNodeAdd (TREEVIEWNODE *pNode);
		void OnNodeChangeName (TREEVIEWNODE *pNode);
		void OnNodeChangePosition (TREEVIEWNODE *pNode);
		void OnNodeRemove (TREEVIEWNODE *pNode);
		void OnObjectChange (TREEVIEWOBJECT *pObject);
		void UpdateStateImages (ITEMHND hndItem);

		virtual void v_OnCreateEnd ();
		virtual void v_OnLeftButtonDown (ITEMHND hndItem, HITRESULT HitResult, const LPOINT2D& point, KEYSTATE keystate);
		virtual void v_OnLeftButtonUp (ITEMHND hndItem, HITRESULT HitResult, const LPOINT2D& point, KEYSTATE keystate);
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TREEREGIONMANAGER_H
