/**
 * \file mi32/angleformatter.h
 * \brief ANGLEFORMATTER class definitions
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
**/

#ifndef	INC_MI32_ANGLEFORMATTER_H
#define	INC_MI32_ANGLEFORMATTER_H

#ifndef	INC_MI32_UOM_H
#include <mi32/uom.h>
#endif


#ifdef GEOMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

namespace SPATREF {
	class COORDAXIS;
	};


//! Universal angle value format converter.
//! Supports formatting/parsing, packing/unpacking and combining/separating.
class CLASSLIBEXPORT ANGLEFORMATTER {

	public:

		//! Direction symbol placement enumeration.
		enum DIRSYMPLACE {
			DIRSYMPLACE_None,				//!< Don't include direction symbol when formatting
			DIRSYMPLACE_Beginning,		//!< Place direction symbol at beginning
			DIRSYMPLACE_End				//!< Place direction symbol at end
			};

		//! Formatting flags.
		enum FORMATFLAGS {
			FORMAT_Default =		0x00,	//!< Default
			FORMAT_UnitSymbols =	0x01,	//!< Include unit symbols for components
			FORMAT_NoSpaces =		0x02,	//!< Don't include space between components
			FORMAT_StripZeros =	0x04,	//!< Strip trailing zeros after decimal point and trailing all-zero components
			FORMAT_MASK =			0x07
			};

		//! Mode for handling values out of range.
		enum RANGEMODE {
			RANGEMODE_Auto,				//!< Automatically determine mode from range as Wrap if >= full circle or Limit if < full circle
			RANGEMODE_Wrap,				//!< Wrap values outside range to be within range
			RANGEMODE_Clip					//!< Limit values outside range to maximum if above or minimum if below
			};

		//! Default constructor.
		ANGLEFORMATTER (
			);

		//! Copy constructor.
		ANGLEFORMATTER (
			const ANGLEFORMATTER& rhs
			);

		~ANGLEFORMATTER ();

		//! Assignment.
		ANGLEFORMATTER& operator= (
			const ANGLEFORMATTER& rhs
			);

		//! Combine components into angle value.
		//! @return Angle value in currently set unit.
		//! Does not depend on current settings for number of components.
		double Combine (
			double component1,			//!< First component value in currently set angle unit
			double component2,			//!< Second component value
			double component3 = 0		//!< Third component value if used
			) const;

		//! Format angle as string.
		MISTRING Format (
			double angle					//!< Angle value in currently set unit.
			) const;

		//! Get current unit setting.
		const UOM::UNIT& GetUnit (
			) const;

		//! Read setting from .ini file.
		//! @return true if entry in file, false if not
		//! Does not change axis or range settings, which if used should be specified before calling IniRead().
		bool IniRead (
			INIHANDLE handle,							//!< Ini Handle created by IniOpen()
			const char *group,						//!< Group to look for ini entry, App Name if =0
			const char *field							//!< Field to read string from
			);

		//! Write settings to .ini file.
		//! Does not save axis or range settings.
		void IniWrite (
			INIHANDLE handle,							//!< Ini Handle created by IniOpen()
			const char *group,						//!< Group to write ini entry, App Name if =0
			const char *field							//!< Field to write string to
			) const;

		//! Pack value into number.
		//! Uses current settings for number of components and component widths, with restriction
		//! that if component width is set less than minimum required for deterministic unpacking
		//! (typically 2 digits) then the minimum width will be used.
		//! @return Packed value.
		double Pack (
			double angle,					//!< Angle value in currently set unit.
			bool RoundToPrecision		//!< Round last component to current precision setting
			) const;

		//! Parse string to determine angle value.
		//! @return true if successful, false if no angle can be parsed
		bool Parse (
			const MISTRING& string,		//!< String to parse
			double& angle					//!< Returned angle value in currently set unit
			) const;

		//! Separate angle into components.
		//! Return component array must be large enough to hold all components (up to 3).
		void Separate (
			double angle,					//!< Angle value in currently set unit.
			double *components,			//!< Returned component values
			bool RoundToPrecision		//!< Round last component to current precision setting
			) const;

		//! Set coordinate axis.
		//! Automatically sets unit, range and component widths.
		void SetAxis (
			const SPATREF::COORDAXIS& axis
			);

		//! Set component widths for formatting, packing and unpacking.
		//! Widths only apply to integer portion of components, use SetPrecision
		//! to set number of decimal places for last component.
		void SetComponentWidths (
			int width1,						//!< Width of first component
			int width2 = 2,				//!< Width of second component (minimum 2, if used)
			int width3 = 2					//!< Width of third component (minimum 2, if used)
			);

		//! Set placement for direction symbol.
		void SetDirSymPlace (
			DIRSYMPLACE placement
			);

		//! Set formatting flags.
		void SetFormatFlags (
			FORMATFLAGS FormatFlags		//!< Formatting flags
			);

		//! Set number of components for formatting, packing and unpacking.
		//! Default number of components is maximum for unit (typically 3).
		void SetNumComponents (
			int numcomponents				//!< Number of components (1-3)
			);

		//! Set precision for last component when formatting.
		void SetPrecision (
			unsigned int precision		//!< Precision (0-15)
			);

		//! Set range and method of handling values outside of range.
		//! If range is less than a full circle then RANGEMODE_Limit will always be used
		//! for out of range values.  If range is more than a full circle and includes
		//! both negative and positive values then out of range values will retain
		//! their original sign if possible when RANGEMODE_Wrap or RANGEMODE_Auto is used.
		void SetRange (
			double minimum,				//! Minimum value in current unit
			double maximum,				//! Maximum value in current unit
			RANGEMODE rangemode = RANGEMODE_Auto
			);

		//! Set angular unit.
		//! Automatically converts existing range to new unit.
		//! Resets number of components to maximum and precision to default.
		void SetUnit (
			UINT8 AngleUnitId
			);

		//! Unpack angle from numerically packed value.
		//! Uses current settings for number of components and component widths, with restriction
		//! that if component width is set less than minimum required for deterministic unpacking
		//! (typically 2 digits) then the minimum width will be used.
		//! @return Angle value in currently set unit.
		double Unpack (
			double packed					//! Packed angle value
			) const;

	private:
		class PRIV;
		PRIV *m_pPriv;
	};

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_ANGLEFORMATTER_H
