/**
 * \file mi32/cameraintrinsics.h
 * \brief CAMERAINTRINSICS definitions
 *
 * \if NODOC
 * $Id: cameraintrinsics.h_v 1.3 2017/02/24 14:57:12 mju Exp $
 *
 * $Log: cameraintrinsics.h_v $
 * Revision 1.3  2017/02/24 14:57:12  mju
 * Rename some methods.
 *
 * Revision 1.2  2017/02/22 19:02:14  mju
 * Simplify setting focal length in MM and pixels.
 *
 * Revision 1.1  2016/01/11 15:25:41  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_CAMERAINTRINSICS_H
#define	INC_MI32_CAMERAINTRINSICS_H

#ifndef INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#include <string>

class MIEXIF;

//=====================================================================================================================

class CAMERAINTRINSICS {
	public:

		enum VALUE {
			VALUE_PrincipalPoint =		0x0000,
			VALUE_PixelSize =				0x0002,
			VALUE_FocalLengthPixels =	0x0004,
			VALUE_Skew =					0x0008,
			};

		CAMERAINTRINSICS ();
		CAMERAINTRINSICS (const CAMERAINTRINSICS &rhs);
		~CAMERAINTRINSICS ();

		CAMERAINTRINSICS& operator= (const CAMERAINTRINSICS &rhs);

		//! Clear all values.
		void Clear ();

		//! Get camera make.
		const std::string& GetCameraMake () const
			{ return (m_CameraMake); }

		//! Get camera model.
		const std::string& GetCameraModel () const
			{ return (m_CameraModel); }

		//! Get camera serial number.
		const std::string& GetCameraSerialNumber () const
			{ return (m_CameraSerialNumber); }

		//! Get focal length in millimeters.
		double GetFocalLengthMM () const
			{ return (m_FocalLengthMM); }

		//! Get focal length in pixels.
		double GetFocalLengthPixels () const
			{ return (m_FocalLengthPix); }

		//! Get image size (pixels).
		const LPOINT2D& GetImageSize () const
			{ return (m_ImageSize); }

		//! Get lens make.
		const std::string& GetLensMake () const
			{ return (m_LensMake); }

		//! Get lens model.
		const std::string& GetLensModel () const
			{ return (m_LensModel); }

		//! Get lens serial number.
		const std::string& GetLensSerialNumber () const
			{ return (m_LensSerialNumber); }

		//! Get number of distortion coefficients.
		UINT32 GetNumDistortionCoeff () const
			{ return (m_NumDistortionCoeff); }

		//! Get pixel size (pitch).
		//! @return Pixel size in millimeters.
		const DPOINT2D& GetPixelSize () const
			{ return (m_PixelSizeMM); }

		//! Get principal point.
		const DPOINT2D& GetPrincipalPoint () const
			{ return (m_PrincipalPoint); }

		//! Get skew.
		double GetSkew () const
			{ return (m_Skew); }

		//! Check if optional value present.
		bool HasValue (
			VALUE value
			) const { return ((m_ValidValues & value) != 0); }

		//! Initialize with values from Exif, reading previously saved values if any.
		//! @return true if has previously saved values, false if not.
		bool Initialize (
			const MIEXIF &exif,
			INT32 ImageSizeX,
			INT32 ImageSizeY
			);

		//! Initialize with specified image size, focal length and pixel size.
		void Initialize (
			INT32 ImageSizeX,
			INT32 ImageSizeY,
			double FocalLengthInitMM,
			double PixelSizeXmm,
			double PixelSizeYmm = 0
			);

		bool IsDefined () const;

		//! Load settings based on camera, lens, image size.
		bool Load ();

		//! Save settings for camera, lens, image size, focal length.
		void Save () const;

		//! Set camera make and model.
		void SetCameraMakeAndModel (
			const std::string &CameraMake,
			const std::string &CameraModel
			) { m_CameraMake = CameraMake; m_CameraModel = CameraModel; }

		//! Set camera serial number.
		void SetCameraSerialNumber (
			const std::string &CameraSerialNumber
			) { m_CameraSerialNumber = CameraSerialNumber; }

		//! Set focal length in pixels.
		void SetFocalLengthPixels (
			double FocalLengthPixels
			) { m_FocalLengthPix = FocalLengthPixels; m_ValidValues |= VALUE_FocalLengthPixels; }

		//! Set focal length in millimeters.
		//! Note, if pixel size is known this will update the focal length in pixels also.
		void SetFocalLengthMM (
			double FocalLengthMM
			);

		//! Set lens make and model.
		void SetLensMakeAndModel (
			const std::string &LensMake,
			const std::string &LensModel
			) { m_LensMake = LensMake; m_LensModel = LensModel; }

		//! Set lens serial number.
		void SetLensSerialNumber (
			const std::string &LensSerialNumber
			) { m_LensSerialNumber = LensSerialNumber; }

		//! Set sensor pixel size (pitch).
		void SetPixelSize (
			double PixelSizeXmm,
			double PixelSizeYmm = 0
			);

		//! Set principal point.
		void SetPrincipalPoint (
			const DPOINT2D &PrincipalPoint
			) { m_PrincipalPoint = PrincipalPoint; m_ValidValues |= VALUE_PrincipalPoint; }

		//! Set skew factor, typically from model reconstruction.
		void SetSkew (
			double skew
			) { m_Skew = skew; m_ValidValues |= VALUE_Skew; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LPOINT2D m_ImageSize;
		DPOINT2D m_PrincipalPoint;
		DPOINT2D m_PixelSizeMM;
		double m_FocalLengthMM;
		double m_FocalLengthPix;
		double m_Skew;
		double m_DistortionCoeff[15];
		std::string m_CameraMake;
		std::string m_CameraModel;
		std::string m_CameraSerialNumber;
		std::string m_LensMake;
		std::string m_LensModel;
		std::string m_LensSerialNumber;
		UINT32 m_ValidValues;
		UINT32 m_NumDistortionCoeff;
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_CAMERAINTRINSICS_H
