/*****************************************************************************
 * \file mi32/dbschema.h
 *
 * \if NODOC
 * Support for the International Hydrographic Organization (IHO) S-57
 * schema for database tables.
 * 
 * This is in response to Bugzilla ID 13849
 *  $Log: dbschema.h_v $
 *  Revision 1.17  2014/11/11 20:17:47  bcolombe
 *  Add SetupTable
 *
 *  Revision 1.16  2014/11/10 18:04:16  bcolombe
 *  Support context, attribute name
 *
 *  Revision 1.15  2013/12/19 22:56:20  bcolombe
 *  Support computed field
 *
 *  Revision 1.14  2013/12/17 18:48:38  bcolombe
 *  Some changes for conflicts
 *
 *  Revision 1.13  2013/11/11 22:51:37  bcolombe
 *  Add attachment type
 *
 *  Revision 1.12  2013/10/28 19:28:52  bcolombe
 *  Add conflict resolution
 *
 *  Revision 1.11  2013/10/07 18:09:21  bcolombe
 *  Add stringexpression
 *
 *  Revision 1.10  2013/09/03 22:05:33  bcolombe
 *  CreateTables, CreateStyle
 *
 *  Revision 1.9  2013/08/19 16:43:33  bcolombe
 *  Changes for relating tables
 *
 *  Revision 1.8  2012/11/26 18:10:49  bcolombe
 *  Added OBJECT MakeTable to make table with all defaults
 *
 *  Revision 1.7  2011/12/08 15:29:26  mju
 *  Fwd declare filepath.
 *
 *  Revision 1.6  2011/10/24 20:36:09  mju
 *  Remove redundant hdr.
 *
 *  Revision 1.5  2007/07/05 21:23:08  dwilliss
 *  keep track of unitid on enums
 *
 *  Revision 1.4  2007/07/02 17:41:36  dwilliss
 *  added attribute type for ListUnique
 *
 *  Revision 1.3  2007/06/22 19:25:55  dwilliss
 *  changed how we deal with object classes
 *
 *  Revision 1.2  2007/06/15 22:41:15  dwilliss
 *  *** empty log message ***
 *
 *  Revision 1.1  2007/06/15 21:48:14  dwilliss
 *  Initial revision
 *
 * \endif
 *
 ****************************************************************************/

#ifndef INC_MI32_DBSCHEMA_H
#define INC_MI32_DBSCHEMA_H

#ifndef INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef INC_MI32_ENUMSET_H
#include <mi32/enumset.h>
#endif

#ifndef INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef INC_RVC_DESCRIPTOR_H
#include <rvc/descriptor.h>
#endif

#ifndef INC_RVC_DBASE_H
#include <rvc/dbase.h>
#endif

#ifndef INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class FILEPATH;
class XMLDOC;
class XMLNODE;
#endif


namespace DBSCHEMA {
	class CATALOG;
	class OBJECT;

	enum CONFLICT {
		CONFLICT_None		= 0x00,
		CONFLICT_Replace	= 0x01,
		CONFLICT_Skip		= 0x02,
		CONFLICT_KeepBoth	= 0x04,
		CONFLICT_ThisTable	= 0x10,
		CONFLICT_AllTables	= 0x20,
		CONFLICT_Cancel		= 0x40,
		CONFLICT_MASK		= 0x0F
		};
	#ifndef GENERATING_DOXYGEN_OUTPUT
	DEFINE_ENUM_OP_BITWISE(CONFLICT)
	#endif

	// Will be read from the s57expectedinput.csv reference file
	struct ATTRIBUTE_ENUMVAL {
		int m_num;
		int m_unitid;	// used for the 3 field types that are actually units
		bool m_bObsolete;
		MISTRING m_text;
		MISTRING m_desc;
		};

	// Will mostly be read from the s57attributes.csv reference file
	// There is also a BaseTypes.xsd file which contains long descriptions
	// and ranges for numeric types that have them
	class ATTRIBUTE {
		public:
			enum TYPE {
				TYPE_Enum = 'E',
				TYPE_Float = 'F',
				TYPE_List = 'L',				//!< Allows duplicates
				TYPE_ListUnique = 'U',
				TYPE_Int = 'I',
				TYPE_CodedString = 'A',	// Text of a specific format (documented but not specified by any reference file -- bastards!)
				TYPE_FreeText = 'S',
				TYPE_StringExpression = 'X',
				TYPE_Computed = 'C'
				};

			// XXX Look up what these mean
			enum CLASS {
				CLASS_F = 'F',	
				CLASS_N = 'N',	
				CLASS_S = 'S',	
				CLASS_Dollar = '$',		// for whole chart?
				};

			ATTRIBUTE (
				const MISTRINGLIST& parts	// line from the csv file
				);

			ATTRIBUTE (
				);

			~ATTRIBUTE (
				);

			CLASS GetClass (
				) const { return m_Class; }

			const RVC::DESCRIPTOR& GetDescriptor (
				) const { return m_Descriptor; }

			const MILIST<ATTRIBUTE_ENUMVAL>& GetEnumValues (
				) const { return m_EnumValues; }

			int GetIDNum (
				) const { return m_num; }

			double GetMaxVal (
				) const { return m_MaxVal; }

			double GetMinVal (
				) const { return m_MinVal; }

			TYPE GetType (
				) const { return m_Type; }

			const MISTRING& GetFormat (
				) const { return m_format; }

			int GetMeasureID (
				) const { return m_MeasureNum; }

			const MISTRING& GetQueryStr (
				) const { return m_QueryStr; }

			int GetUnitID (
				) const { return m_UnitNum; }

			const MISTRING& GetUnitName (
				) const { return m_UnitName; }

		private:
			int m_num;
			RVC::DESCRIPTOR m_Descriptor;
			char m_acronym[8];
			MISTRING m_format;
			int m_MeasureNum;
			int m_UnitNum;
			CLASS m_Class;
			TYPE m_Type;	
			double m_MinVal;
			double m_MaxVal;
			MILIST<ATTRIBUTE_ENUMVAL> m_EnumValues;
			MISTRING m_UnitName;
			MISTRING m_QueryStr;
			friend class CATALOG;
		};


	// information about a class of objects.  Currently only a place to hold
	// the descriptor, but here for future expansion
	class OBJECT_CLASS {
		public:
			OBJECT_CLASS();
			~OBJECT_CLASS();

			bool IsRestrictedByElemType (
				) const { return m_bIsRestrictedByElemType; }

			const RVC::DESCRIPTOR& GetDescriptor (
				) const { return m_Descriptor; }

		private:
			RVC::DESCRIPTOR m_Descriptor;
			bool m_bIsRestrictedByElemType;

			friend class CATALOG;
		};

	class KEY {
		public:
			KEY (const MISTRING& PKey, const MISTRING& FKey) {m_PrimaryKey = PKey; m_ForeignKey = FKey;}
			
			~KEY () {}

			const MISTRING& GetPrimaryKey (
				) const { return m_PrimaryKey; }

			const MISTRING& GetForeignKey (
				) const { return m_ForeignKey; }

		private:
			MISTRING m_ForeignKey;
			MISTRING m_PrimaryKey;
		};

	class OBJECT {
		public:

		OBJECT (
			const CATALOG*
			);

		~OBJECT (
			);

		//! Return table attachment type
		const RVC::DBTABLE::ATTACHMENT& GetAttachment (
			) const { return m_Attachment; }

		//! Return the list of attributes in one of the attribute groups.
		//! The are up to 3 attribute groups, some of which may be empty
		const SIMPLE_ARRAY<const ATTRIBUTE*>& GetAttributeGroup (
			int num
			) const { return m_AttributeGroup[num]; }

		//! Get the catalog this object came from
		const CATALOG* GetCatalog (
			) const { return m_Catalog; }

		//! Get the object class
		const OBJECT_CLASS& GetClass (
			) const { return *m_Class; }

		//! Get the object descriptor
		const RVC::DESCRIPTOR& GetDescriptor (
			) const { return m_Descriptor; }

		int GetIDNum (
			) const { return m_num; }

		//! Determine if an object is allowd for a specific element type
		bool IsForElementType (
			ELEMTYPE ElemType
			) const { return m_ElemTypes.Contains(ElemType); }

		//! Get fieldinfo from object
		ERRVALUE SetupTable (
			RVC::DBASE_ELEMENT& dbase,
			RVC::DBTABLE::MAKEPARMS& MakeParms,
			FastDelegate<UOM::IDNUM(const MISTRING&)> delegate = 0,
			bool bUseDCNOTEField = false
			) const;

		//! Make default table from object
		ERRVALUE MakeTable (
			RVC::DBASE_ELEMENT& dbase,
			RVC::DBTABLE& dbtable,
			UINT32& ConflictRequest
			) const;

		ERRVALUE MakeTable (
			RVC::DBASE_ELEMENT& dbase,
			RVC::DBTABLE& dbtable
			) const;

		private:
			char m_acronym[15];
			int m_num;
			const OBJECT_CLASS* m_Class;
			RVC::DESCRIPTOR m_Descriptor;
			RVC::DBTABLE::ATTACHMENT m_Attachment;
			ENUMSET<ELEMTYPE, ELEMTYPE_NumTypes> m_ElemTypes;
			SIMPLE_ARRAY<const ATTRIBUTE*> m_AttributeGroup[3];
			MILIST<KEY> m_Keys;
			const CATALOG* m_Catalog;

			friend class CATALOG;
		};

	// Container for S57 stuff
	class CATALOG {
		public:
			enum CONTEXT {
				CONTEXT_Import,
				CONTEXT_Table
				};

			CATALOG();
			~CATALOG();

			ERRVALUE CreateTables (
				RVC::DBASE* dbasepoint = NULL,
				RVC::DBASE* dbaseline = NULL,
				RVC::DBASE* dbasepoly = NULL
				) const;

			ERRVALUE CreateTables (
				UINT32& ConflictRequest,
				RVC::DBASE* dbasepoint = NULL,
				RVC::DBASE* dbaseline = NULL,
				RVC::DBASE* dbasepoly = NULL
				) const;

			ERRVALUE CreateStyle (
				const RVC::OBJITEM& ParentObjItem
				) const;

			const ATTRIBUTE* FindAttribute (
				const char* acronym
				) const;

			const ATTRIBUTE* FindAttribute (
				int num
				) const;

			const OBJECT* FindObject (
				const char* acronym
				) const;

			const OBJECT* FindObject (
				int num
				) const;

			const RVC::DESCRIPTOR& GetDescriptor (
				) const { return m_Descriptor; }

			const SIMPLE_ARRAY<OBJECT_CLASS*>& GetObjectClasses (
				) const { return m_ObjectClasses; }

			const SIMPLE_ARRAY<OBJECT*>& GetObjects (
				) const { return m_ObjectTypes; }

			ERRVALUE GetXML (
				XMLDOC& doc
				);

			ERRVALUE Read (
				const XMLDOC& doc,
				CONTEXT context = CONTEXT_Table
				);

			ERRVALUE Read (
				const FILEPATH& filepath,
				CONTEXT context = CONTEXT_Table
				);

		private:
			SIMPLE_ARRAY<OBJECT_CLASS*> m_ObjectClasses;
			SIMPLE_ARRAY<ATTRIBUTE*> m_AttributeTypes;
			SIMPLE_ARRAY<OBJECT*> m_ObjectTypes;
			RVC::DESCRIPTOR m_Descriptor;
			CONTEXT m_context;

			ATTRIBUTE* FindAttribute (
				const MISTRING& code					// This is a string with the acronym and code combined
				);

			OBJECT* FindObject (
				const MISTRING& code
				);

			bool GetContext (XMLNODE* node);

			void SetAttributes (
				SIMPLE_ARRAY<const ATTRIBUTE*>& AttributeGroup,
				const MISTRING&mistr
				);

		};


} // end DBSCHEMA namespace

#endif  // INC_MI32_DBSCHEMA_H
