/**
 * \file mi32/elemnumset.h
 * \brief ELEMNUMSET definitions
 *
 * \if NODOC
 * $Id: elemnumset.h_v 1.3 2015/12/08 21:48:44 mju Exp $
 *
 * $Log: elemnumset.h_v $
 * Revision 1.3  2015/12/08 21:48:44  mju
 * Add checkDuplicate.
 *
 * Revision 1.2  2015/05/14 15:35:45  mju
 * In getElemNum use cast so returns correct value for -1.
 *
 * Revision 1.1  2015/05/07 14:51:34  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_ELEMNUMSET_H
#define	INC_MI32_ELEMNUMSET_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

//=====================================================================================================================

//! Set of unique element numbers.
//! Optimized to not allocate memory for storage of single number.
//! If used with SIMPLE_ARRAY must call Init/Free for proper initialization and destruction.
class ELEMNUMSET {
	public:

		//! Default constructor.
		ELEMNUMSET ()
			{ Init(); }

		//! Copy constructor.
		ELEMNUMSET (
			const ELEMNUMSET &rhs
			) { Init(); Copy(rhs); }

		//! Destructor.
		~ELEMNUMSET ()
			{ Free(); }

		//! Assignment.
		ELEMNUMSET& operator= (
			const ELEMNUMSET &rhs
			) { Copy(rhs); return (*this); }

		//! Add element number to set.
		//! If set already contains the specified element number it will NOT be added again.
		ERRVALUE AddElemNum (
			ELEMNUM ElemNum,			//!< Element number to add, must be >= 0
			bool bCheckDuplicate		//!< Check for duplicates
			);

		//! Copy set contents.
		ERRVALUE Copy (
			const ELEMNUMSET &rhs
			);

		//! Delete specified element number from set.
		void DeleteElemNum (
			ELEMNUM ElemNum			//!< Element number to delete
			);

		//! Free contents of set.
		void Free ();

		//! Get element number at specified index.
		//! @return Element number of -1 if index out of range.
		ELEMNUM GetElemNum (
			UINT32 idx					//!< Index in set
			) const { return ((idx < m_NumItems) ? static_cast<ELEMNUM>(m_ElemNums[idx]) : -1); }

		//! Get number of element numbers in set.
		UINT32 GetNumItems () const
			{ return (m_NumItems); }

		//! Initialize set for case where constructor is not called.
		void Init ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 *m_ElemNums;							// Array of element numbers
		UINT32 m_NumItems;							// Number of entries in the erray
		UINT32 m_MaxItems;							// Maximum items in array or element number if NumItems < 2
		#endif
	};

#endif	// INC_MI32_ELEMNUMSET_H
