/**
 * \file mi32/linerasterizer.h
 * \brief LINERASTERIZER class definitions
 *
 * \if NODOC
 * $Id: linerasterizer.h_v 1.1 2014/07/18 16:12:08 mju Exp $
 *
 * $Log: linerasterizer.h_v $
 * Revision 1.1  2014/07/18 16:12:08  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LINERASTERIZER_H
#define	INC_MI32_LINERASTERIZER_H

#ifndef	INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class POLYLINE;
#endif

//=====================================================================================================================

//! High-accuracy line rasterizer.
//! Provides methods to split lines into segments at integer grid steps.
//! Result of split is passed to delegate.
class LINERASTERIZER {
	public:

		//! Directions to split.
		enum SPLIT {
			SPLIT_X =		1,		//!< Split at integer X values
			SPLIT_Y =		2,		//!< Split at integer Y values
			SPLIT_XY =		3		//!< Split at both X and Y integer values
			};

		//! Delegate for pair of 2D points.
		typedef fastdelegate::FastDelegate<ERRVALUE(const DPOINT2D&, const DPOINT2D&)> DELEGATE_ONPAIR2D;

		//! Delegate for pair of 3D points.
		typedef fastdelegate::FastDelegate<ERRVALUE(const DPOINT3D&, const DPOINT3D&)> DELEGATE_ONPAIR3D;

		//! Constructor.
		LINERASTERIZER ();

		//! Destructor.
		~LINERASTERIZER ();

		//! Set delegate for 2D split.
		void SetDelegateOnSplit2D (
			DELEGATE_ONPAIR2D delegate
			) { m_OnSplit2D = delegate; }

		//! Set delegate for 3D split.
		void SetDelegateOnSplit3D (
			DELEGATE_ONPAIR3D delegate
			) { m_OnSplit3D = delegate; }

		//! Split polyline in 2D at integer steps in specified direction.
		ERRVALUE SplitLine2D (
			const POLYLINE& polyline,		//!< Line to split
			SPLIT split = SPLIT_Y			//!< Direction(s) of split
			);

		//! Split polyline in 3D at integer steps in specified direction.
		ERRVALUE SplitLine3D (
			const POLYLINE& polyline,		//!< Line to split
			SPLIT split = SPLIT_Y			//!< Direction(s) of split
			);

		//! Split segment in 2D at integer steps in specified direction.
		ERRVALUE SplitSegment2D (
			const DPOINT2D& ptStart,		//!< Starting point
			const DPOINT2D& ptEnd,			//!< Ending point
			SPLIT split = SPLIT_Y			//!< Direction(s) of split
			);

		//! Split segment in 3D at integer steps in specified direction.
		ERRVALUE SplitSegment3D (
			const DPOINT3D& ptStart,		//!< Starting point
			const DPOINT3D& ptEnd,			//!< Ending point
			SPLIT split = SPLIT_Y			//!< Direction(s) of split
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef LINERASTERIZER THISCLASS;

		DELEGATE_ONPAIR2D m_OnSplit2D;
		DELEGATE_ONPAIR3D m_OnSplit3D;
		DELEGATE_ONPAIR2D m_Resplit2DX;
		DELEGATE_ONPAIR3D m_Resplit3DX;

		static ERRVALUE SplitSegment2DX (
			const DPOINT2D& ptStart,
			const DPOINT2D& ptEnd,
			const DELEGATE_ONPAIR2D& OnSplit
			);

		static ERRVALUE SplitSegment2DY (
			const DPOINT2D& ptStart,
			const DPOINT2D& ptEnd,
			const DELEGATE_ONPAIR2D& OnSplit
			);

		static ERRVALUE SplitSegment3DX (
			const DPOINT3D& ptStart,
			const DPOINT3D& ptEnd,
			const DELEGATE_ONPAIR3D& OnSplit
			);

		static ERRVALUE SplitSegment3DY (
			const DPOINT3D& ptStart,
			const DPOINT3D& ptEnd,
			const DELEGATE_ONPAIR3D& OnSplit
			);

		ERRVALUE Resplit2DX (
			const DPOINT2D& ptStart,
			const DPOINT2D& ptEnd
			) const;

		ERRVALUE Resplit3DX (
			const DPOINT3D& ptStart,
			const DPOINT3D& ptEnd
			) const;

		#endif

	};

//=====================================================================================================================

#endif	// INC_MI32_LINERASTERIZER_H
