/**
 * \file mi32/mediasize.h
 * \brief MEDIASIZE definitions
 *
 * \if NODOC
 * $Id: mediasize.h_v 1.5 2013/08/21 20:58:36 mju Exp $
 *
 * $Log: mediasize.h_v $
 * Revision 1.5  2013/08/21 20:58:36  mju
 * Add setHeight/Width.
 *
 * Revision 1.4  2013/08/20 16:02:21  mju
 * Use double.
 *
 * Revision 1.3  2013/08/20 15:05:19  mju
 * Add get/setLandscape.
 *
 * Revision 1.2  2013/08/15 15:46:59  mju
 * Add getDisplay.
 *
 * Revision 1.1  2013/05/14 16:55:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MEDIASIZE_H
#define	INC_MEDIASIZE_H

#ifndef	INC_MI32_SCRNUNIT_H
#include <mi32/scrnunit.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

//=====================================================================================================================

//! Container for media/paper/page size.
class MEDIASIZE {
	public:

		//! Standard size codes.
		enum CODE {
			CODE_Custom =		0,
			CODE_ISO_A0 =		100,	//!< 841 x 1189
			CODE_ISO_A1 =		101,	//!< 594 x 841
			CODE_ISO_A2 =		102,	//!< 420 x 594
			CODE_ISO_A3 =		103,	//!< 297 x 420
			CODE_ISO_A4 =		104,	//!< 210 x 297
			CODE_ISO_A5 =		105,	//!< 148 x 210
			CODE_ISO_A6 =		106,	//!< 105 x 148
			CODE_ISO_A7 =		107,	//!< 74 x 105
			CODE_ISO_A8 =		108,	//!< 52 x 74
			CODE_ISO_A9 =		109,	//!< 37 x 52
			CODE_ISO_A10 =		110,	//!< 26 x 37
			CODE_ISO_B0 =		200,	//!< 1000 x 1414
			CODE_ISO_B1 =		201,	//!< 707 x 1000
			CODE_ISO_B2 =		202,	//!< 500 x 707
			CODE_ISO_B3 =		203,	//!< 353 x 500
			CODE_ISO_B4 =		204,	//!< 250 x 353
			CODE_ISO_B5 =		205,	//!< 176 x 250
			CODE_ISO_B6 =		206,	//!< 125 x 176
			CODE_ISO_B7 =		207,	//!< 88 x 125
			CODE_ISO_B8 =		208,	//!< 62 x 88
			CODE_ISO_B9 =		209,	//!< 44 x 62
			CODE_ISO_B10 =		210,	//!< 31 x 44
			CODE_ISO_C0 =		300,	//!< 917 x 1297
			CODE_ISO_C1 =		301,	//!< 648 x 917
			CODE_ISO_C2 =		302,	//!< 458 x 648
			CODE_ISO_C3 =		303,	//!< 324 x 458
			CODE_ISO_C4 =		304,	//!< 229 x 324
			CODE_ISO_C5 =		305,	//!< 162 x 229
			CODE_ISO_C6 =		306,	//!< 114 x 162
			CODE_ISO_C7 =		307,	//!< 81 x 114
			CODE_ISO_C8 =		308,	//!< 57 x 81
			CODE_ISO_C9 =		309,	//!< 40 x 57
			CODE_ISO_C10 =		310,	//!< 28 x 40
			CODE_ANSI_A =		400,	//!< 8.5 x 11 in, Letter
			CODE_ANSI_B =		401,	//!< 11 x 17 in, Ledger/Tabloid
			CODE_ANSI_C =		402,	//!< 17 x 22 in
			CODE_ANSI_D =		403,	//!< 22 x 34 in
			CODE_ANSI_E =		404,	//!< 34 x 44 in
			// Other North American codes
			CODE_NA_Legal =	491,	//!< 8.5 x 14 in
			CODE_NA_JrLegal =	492,	//!< 8 x 5 in
			// Architecture sizes
			CODE_Arch_A =		500,	//!< 9 x 12 in
			CODE_Arch_B =		501,	//!< 12 x 18 in
			CODE_Arch_C =		502,	//!< 18 x 24 in
			CODE_Arch_D =		503,	//!< 24 x 36 in
			CODE_Arch_E =		504,	//!< 36 x 48 in
			CODE_Arch_E1 =		511,	//!< 30 x 42 in
			CODE_Arch_E2 =		512,	//!< 26 x 38 in
			CODE_Arch_E3 =		513,	//!< 27 x 39 in
			// Japanese sizes, common in Japan, China, Taiwan
			CODE_JIS_B0 =		600,	//!< 1030 x 1456
			CODE_JIS_B1 =		601,	//!< 728 x 1030
			CODE_JIS_B2 =		602,	//!< 515 x 728
			CODE_JIS_B3 =		603,	//!< 364 x 515
			CODE_JIS_B4 =		604,	//!< 257 x 364
			CODE_JIS_B5 =		605,	//!< 182 x 257
			CODE_JIS_B6 =		606,	//!< 128 x 182
			CODE_JIS_B7 =		607,	//!< 91 x 128
			CODE_JIS_B8 =		608,	//!< 64 x 91
			CODE_JIS_B9 =		609,	//!< 45 x 64
			CODE_JIS_B10 =		610,	//!< 32 x 45
			CODE_JIS_B11 =		611,	//!< 22 x 32
			CODE_JIS_B12 =		612,	//!< 16 x 22
			};

		//! Get name for CODE.
		static MISTRING GetCodeName (
			CODE code,									//!< Code
			bool bIncludeDimensions = false		//!< Whether to include dimensions
			);

		//! Default constructor.
		MEDIASIZE ();

		//! Construct from code.
		MEDIASIZE (
			CODE code
			) { SetCode(code); }

		//! Assign from code.
		MEDIASIZE& operator= (
			CODE code
			) { SetCode(code); return (*this); }

		//! Compare for equality.
		bool operator== (
			const MEDIASIZE& rhs
			) const { return (IsEqual(rhs)); }

		//! Compare for inequality.
		bool operator!= (
			const MEDIASIZE& rhs
			) const { return (!IsEqual(rhs)); }

		//! Get size code.
		CODE GetCode () const
			{ return (m_code); }

		//! Get display string.
		MISTRING GetDisplay (
			bool bIncludeDimsIfNotCustom = false		//!< Whether to include dimensions if not custom size
			) const;

		//! Get height in stored units.
		double GetHeight () const
			{ return (m_height); }

		//! Get height in specified units.
		double GetHeight (
			SCREENUNIT units							//!< Desired units
			) const { return (ConvertScreenUnits(m_height,m_units,units)); }

		//! Get units.
		SCREENUNIT GetUnits () const
			{ return (m_units); }

		//! Get width in stored units.
		double GetWidth () const
			{ return (m_width); }

		//! Get width in specified units.
		double GetWidth (
			SCREENUNIT units							//!< Desired units
			) const { return (ConvertScreenUnits(m_width,m_units,units)); }

		//! Read entry from .ini file.
		//! @return true if entry in file, false if not
		bool IniRead (
			INIHANDLE handle,							//!< Ini Handle created by IniOpen()
			const char *group,						//!< Group to look for ini entry, App Name if =0
			const char *field							//!< Field to read from
			);

		//! Write entry to .ini file.
		void IniWrite (
			INIHANDLE handle,							//!< Ini Handle created by IniOpen()
			const char *group,						//!< Group to write ini entry, App Name if =0
			const char *field							//!< Field to write entry to
			) const;

		//! Determine if has "landscape" orientation.
		bool IsLandscape () const
			{ return (m_width > m_height); }

		//! Set custom size.
		void SetCustom (
			double width,								//!< Width in specified units
			double height,								//!< Height in specified units
			SCREENUNIT units,							//!< Units
			bool bFindMatchingCode					//!< Whether to find matching CODE if any
			);

		//! Set height.
		void SetHeight (
			double height,								//!< Height in same units as current
			bool bFindMatchingCode = true			//!< Whether to find matching CODE if any
			);

		//! Set width.
		void SetWidth (
			double width,								//!< Width in same units as current
			bool bFindMatchingCode = true			//!< Whether to find matching CODE if any
			);

		//! Set whether has "landscape" orientation.
		void SetLandscape (
			bool bLandscape							//!< Whether landscape or not
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		MEDIASIZE (CODE code, double width, double height, SCREENUNIT units = SCREENUNIT_Millimeters);

	private:
		double m_width;
		double m_height;
		CODE m_code;
		SCREENUNIT m_units;

		bool IsEqual (const MEDIASIZE& rhs) const;
		bool SetCode (CODE code);
		#endif
	};

//=====================================================================================================================

#endif	// INC_MEDIASIZE_H
