/**
 * \file mi32/miexif.h
 * \brief MIEXIF definitions
 *
 * \if NODOC
 * $Id: miexif.h_v 1.8 2017/03/01 16:58:03 mju Exp $
 *
 * $Log: miexif.h_v $
 * Revision 1.8  2017/03/01 16:58:03  mju
 * Add tags for software, datetime.
 *
 * Revision 1.7  2017/02/24 17:31:24  mju
 * Add focal plane resolution tags.
 *
 * Revision 1.6  2015/11/16 15:38:51  mju
 * Add useHex parm to tagToKey.
 *
 * Revision 1.5  2015/08/21 20:46:12  mju
 * Redesign to use map of maps so handles all group keys.
 * Add write capability (for jpeg).
 *
 * Revision 1.4  2015/08/20 21:12:17  mju
 * Add generateString and getStringMap.
 *
 * Revision 1.3  2015/08/12 14:18:34  mju
 * Add tags for black level.
 *
 * Revision 1.2  2015/08/10 20:04:20  mju
 * Add getGPSLatLon.
 *
 * Revision 1.1  2015/08/10 19:53:43  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MIEXIF_H
#define	INC_MI32_MIEXIF_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#include <map>
#include <string>

class MISTRING;
class FILEPATH;

//=====================================================================================================================

//! Container for EXIF values, typically read from metadata.
class MIEXIF {
	public:

		enum GROUP {
			GROUP_Image =			0x010000,
			GROUP_Photo =			0x020000,
			GROUP_Thumbnail =		0x040000,
			GROUP_Iop =				0x080000,
			GROUP_GPSInfo =		0x100000,
			GROUP_Primary =		GROUP_Image | GROUP_Photo,
			GROUP_MASK =			0x1F0000
			};

		enum TAG {
			TAG_Make =								271,
			TAG_Model =								272,
			TAG_Software =							305,
			TAG_DateTime =							306,
			TAG_SerialNumber =					50735,
			TAG_BodySerialNumber =				42033,
			TAG_LensMake =							42035,
			TAG_LensModel =						42036,
			TAG_LensSerialNumber =				42037,
			TAG_ExposureTime =					33434,
			TAG_FNumber =							33437,
			TAG_FocalLength =						37386,
			TAG_ShutterSpeedValue =				37377,	//!< APEX units, not reciprocal of ExposureTime
			TAG_ApertureValue =					37378,	//!< APEX units, not same as F-Number
			TAG_SubjectDistance =				37382,
			TAG_ISOSpeedRatings =				34855,
			TAG_ISOSpeed =							34867,
			TAG_FocalPlaneXResolution =		41486,
			TAG_FocalPlaneYResolution =		41487,
			TAG_FocalPlaneResolutionUnit =	41488,
			TAG_BlackLevelRepeatDim =			50713,
			TAG_BlackLevel =						50714,
			TAG_GPSVersionID =					GROUP_GPSInfo|1,
			TAG_GPSLatitudeRef =					GROUP_GPSInfo|2,
			TAG_GPSLatitude =						GROUP_GPSInfo|3,
			TAG_GPSLongitudeRef =				GROUP_GPSInfo|4,
			TAG_GPSLongitude =					GROUP_GPSInfo|5,
			TAG_GPSAltitudeRef =					GROUP_GPSInfo|6,
			TAG_GPSAltitude =						GROUP_GPSInfo|7,
			TAG_GPSDOP =							GROUP_GPSInfo|11,
			};

		typedef std::map<std::string,std::string> STRMAP;

		//! Generate string for item at iterator.
		static MISTRING GenerateString (
			GROUP group,
			const STRMAP::const_iterator &itStrMap
			);

		//! Constructor.
		MIEXIF ();

		//! Destructor.
		~MIEXIF ();

		//! Add value.
		void AddValue (
			const char *tagstr,					//!< Exif tag string, eg "Image.FNumber" or "Exif.Image.Model"
			const char *value,					//!< Value string
			bool bReplaceExisting = true		//!< Whether to replace existing value
			);

		//! Copy GPSInfo values from another MIEXIF instance.
		void CopyGpsInfoFrom (
			const MIEXIF &exif
			);

		// Copy specified value from another MIEXIF instance
		void CopyValueFrom (
			const MIEXIF &src,
			const std::string &groupkey,
			const std::string &valuekey
			);

		//! Create with default values.
		void Create ();

		//! Get single numeric value.
		//! @return true if found, false if not
		bool GetDouble (
			double &value,						//!< Value returned
			TAG tag,								//!< Tag
			GROUP group = GROUP_Primary	//!< Group to search
			) const;

		//! Get multiple numeric values.
		//! @return true if found, false if not
		bool GetDouble (
			DOUBLE_ARRAY<double> &values,	//!< Value array returned
			TAG tag,								//!< Tag
			GROUP group = GROUP_Primary	//!< Group to search
			) const;

		//! Get GPS Latitude (Y) and Longitude (X).
		//! @return true if found, false if not
		bool GetGPSLatLon (
			DPOINT2D &pt						//!< Point returned
			) const;

		//! Get single numeric value.
		//! @return true if found, false if not
		bool GetInteger (
			int &value,							//!< Value returned
			TAG tag,								//!< Tag
			GROUP group = GROUP_Primary	//!< Group to search
			) const;

		//! Get multiple numeric values.
		//! @return true if found, false if not
		bool GetInteger (
			SIMPLE_ARRAY<int> &values,		//!< Value array returned
			TAG tag,								//!< Tag
			GROUP group = GROUP_Primary	//!< Group to search
			) const;

		//! Get single rational value.
		//! @return true if found, false if not
		bool GetRational (
			int &numerator,					//!< Numerator returned
			int &denominator,					//!< Denominator returned
			TAG tag,								//!< Tag
			GROUP group = GROUP_Primary	//!< Group to search
			) const;

		//! Get value as ASCII character string.
		//! @return Pointer to string or NULL if not found.
		const char* GetString (
			TAG tag,								//!< Tag
			GROUP group = GROUP_Primary	//!< Group to search
			) const;

		//! Get string map.
		const STRMAP* GetStringMap (
			GROUP group
			) const;

		//! Get string map.
		const STRMAP* GetStringMap (
			const std::string &groupkey	//!< Exif group key ("Image", "Photo", etc)
			) const;

		//! Get value.
		const char* GetValue (
			const std::string &groupkey,
			const std::string &valuekey
			) const;

		//! Set contents from Exif metadata string.
		ERRVALUE SetFromExif (
			const MISTRING &metadata		//!< Full metadata string with all Exif entries separated by newlines
			);

		//! Set contents from TIFF metadata string.
		ERRVALUE SetFromTIFF (
			const MISTRING &metadata		//!< Full metadata string with all entries separated by newlines
			);

		//! Write values to specified image file.
		ERRVALUE Write (
			const FILEPATH &filepath		//!< Existing image filepath to add exif values to
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef std::map<std::string,STRMAP> GROUPMAP;
		GROUPMAP m_GroupMap;

		static const char* GroupToKey (GROUP group);
		static const char* TagToKey (TAG tag, GROUP group, bool bHex = false);

		void AddValueLow (const std::string &groupkey, const std::string &valuekey, const char *value, bool bReplaceExisting = true);
		void Clear ();

		MIEXIF (const MIEXIF &rhs);
		MIEXIF& operator= (const MIEXIF &rhs);
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_MIEXIF_H
