/**
 * \file mi32/mitablet.h
 * \brief MicroImages Tablet Interface
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef	INC_MI32_MITABLET_H
#define	INC_MI32_MITABLET_H

//! MicroImages Tablet Interface
namespace MITABLET {

//! Information about specific tablet device.
class DEVICEINFO {
	public:

		enum CAPABILITIES {
			CAPABILITY_IntegratedDisplay =	0x01,	//!< Display and tablet share same surface
			CAPABILITY_Touch =					0x02,	//!< Cursor must be in physical contact with device to report position
			CAPABILITY_CanReportProximity =	0x04,	//!< Device can generate events when cursor enters or leaves detection range
			CAPABILITY_CursorHasUniqueId =	0x08,	//!< Device can uniquely identify active cursor in hardware
			};

		class RANGE {
			public:
				enum UNIT {
					UNIT_None =				0,
					UNIT_Inches =			1,
					UNIT_Centimeters =	2,
					UNIT_Circle =			3
					};

				double GetResolution () const { return (m_Resolution); }
				INT32 GetMaximum () const { return (m_Maximum); }
				INT32 GetMinimum () const { return (m_Minimum); }
				UNIT GetUnit () const { return (m_Unit); }
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				double m_Resolution;
				INT32 m_Minimum;
				INT32 m_Maximum;
				UNIT m_Unit;
				#endif
			};

		CAPABILITIES GetCapabilities () const { return (m_Capabilities); }

		//! Get information about specified cursor.
		const CURSORINFO& GetCursorInfo (
			INT32 CursorNum
			) const;

		const MISTRING& GetName () const { return (m_Name); }

		INT32 GetNumCursors () const { return (m_NumCursors); }

		const RANGE& GetRangeOrientationAzimuth () { return (m_RangeOrientationAzimuth); }
		const RANGE& GetRangeOrientationElevation () { return (m_RangeOrientationElevation); }
		const RANGE& GetRangeOrientationTwist () { return (m_RangeOrientationTwist); }

		const RANGE& GetRangePressureNormal () { return (m_RangePressureNormal); }
		const RANGE& GetRangePressureTangent () { return (m_RangePressureTangent); }

		const RANGE& GetRangeRotationPitch () { return (m_RangeRotationPitch); }
		const RANGE& GetRangeRotationRoll () { return (m_RangeRotationRoll); }
		const RANGE& GetRangeRotationYaw () { return (m_RangeRotationYaw); }

		const RANGE& GetRangeX () { return (m_RangeX); }
		const RANGE& GetRangeY () { return (m_RangeY); }
		const RANGE& GetRangeZ () { return (m_RangeZ); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_Name;
		CAPABILITIES m_Capabilities
		RANGE m_RangeX;
		RANGE m_RangeY;
		RANGE m_RangeZ;
		RANGE m_RangeOrientationAzimuth;
		RANGE m_RangeOrientationElevation;
		RANGE m_RangeOrientationTwist;
		RANGE m_RangePressureNormal;
		RANGE m_RangePressureTangent;
		RANGE m_RangeRotationPitch;
		RANGE m_RangeRotationRoll;
		RANGE m_RangeRotationYaw;
		#endif
	};

class CURSORINFO {
	public:

		const MISTRING& GetName () const { return (m_Name); }

		UINT8 GetNumButtons () const { return (m_NumButtons); }

		bool IsActive () const { return (m_IsActive); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_Name;
		bool m_IsActive;
		UINT8 m_NumButtons;
		#endif
	};

//! Tablet System interface, contains static methods to manage tablet system.
class SYSTEM {
	public:

		//! Tablet mode
		enum MODE {
			MODE_System,				//!< System mode, tablet controls mouse cursor on one or more screens
			MODE_Digitizer				//!< Digitizer mode, tablet separate from mouse cursor
			};

		//! Get information about specific device.
		static const DEVICEINFO& GetDeviceInfo
			INT32 DeviceNum = 0		//!< Device number, 0 for first
			);

		//! Get next tablet event from queue.
		//! @return true if have event, false if none in queue.
		static bool GetEvent (
			EVENT& event				//!< Event returned
			);

		//! Get tablet mode.
		//! By default, tablet will be used in "system" mode.
		static MODE GetMode ();

		//! Get number of tablet devices installed.
		//! Usually returns 0 or 1.
		static INT32 GetNumDevices ();

		//! Initialize tablet interface.
		//! @return true if tablet available, false if not.
		static bool Initialize ();

		//! Determine if has tablet available.
		static bool HasTablet ();

		//! Set tablet mode and optional device.
		//! By default, tablet will be used in "system" mode.
		static void SetMode (
			MODE mode,					//!< New tablet mode
			INT32 DeviceNum = 0		//!< Device number, 0 for first
			);

	};

class EVENT {
	public:
		enum BUTTONCHANGE {
			BUTTONCHANGE_None,
			BUTTONCHANGE_Press,
			BUTTONCHANGE_Release
			};

		struct ORIENTATION {
			double m_Azimuth;			//!< Rotation in degrees about Z axis
			double m_Elevation;		//!< Angle in degrees relative to X-Y plane, upward is positive, downward is negative
			double m_Twist;			//!< Rotation of cursor abot its own major axis
			};

		struct ROTATION {
			double m_Pitch;			//!< Pitch in degree
			double m_Roll;				//!< Roll in degree
			double m_Yaw;				//!< Yaw in degree
			};

		BUTTONCHANGE GetButtonChange (
			UINT8 button
			) const;

		//! Get bitmask for button changes.
		UINT32 GetButtonChangeMask () const;

		//! Get bitmask for current button state.
		UINT32 GetButtonStateMask () const;

		//! Get cursor number.
		INT32 GetCursorNum () const { return (m_CursorNum); }

		//! Get orientation values.
		const ORIENTATION& GetOrientation () const { return (m_Orientation); }

		//! Get position.
		const LPOINT3D& GetPosition () const { return (m_Position); }

		//! Get pressure in direction "normal" digitizer plane.
		UINT32 GetPressureNormal () const { return (m_PressureNormal); }

		//! Get pressure in direction "tangent" digitizer plane.
		UINT32 GetPressureTangent () const { return (m_PressureTangent); }

		//! Determine if cursor (eg, stylus) is in "inverted" orientation.
		//! Typically inverted orientation corresponds to "erase" action.
		bool IsInverted () const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_CursorNum;
		LPOINT3D m_Position;
		ORIENTATION m_Orientation;
		UINT32 m_PressureNormal;
		UINT32 m_PressureTangent;
		#endif
	};

}	// End namespace MITABLET

#endif	// INC_MI32_MITABLET_H
