/**
 * \file mi32/rect.h
 * \brief Definitions for 2D and 3D rectangle structures
 *
 * \if NODOC
 * $Id: rect.h_v 1.71 2016/02/03 22:32:32 mju Exp $
 *
 * $Log: rect.h_v $
 * Revision 1.71  2016/02/03 22:32:32  mju
 * Remove checksize as clang complains and sizes don't change anyway.
 *
 * Revision 1.70  2015/02/23 15:28:13  mju
 * Remove getRectRoundNoLimit.
 *
 * Revision 1.69  2015/02/23 15:24:30  mju
 * Add getRectRoundExact to replace noLimit.
 *
 * Revision 1.68  2014/12/29 17:14:08  mju
 * Add doxygen exclusions for deprecated methods.
 *
 * Revision 1.67  2013/12/31 16:30:02  mju
 * Remove unused deprecated methods.
 *
 * Revision 1.66  2013/12/30 21:32:24  mju
 * No implicit rect3d ctor from 2d.
 *
 * Revision 1.65  2013/12/30 14:34:10  mju
 * Move wrect2d to own hdr.
 * Add specific 2D/3D methods for drect and deprecate nonspecific ones.
 *
 * Revision 1.64  2013/12/27 22:22:30  mju
 * Start disallowing implicit 3d ctor from 2d.
 * Add dimension-specific methods.
 *
 * Revision 1.63  2013/12/24 16:37:15  mju
 * Add contains for rect3d using point/rect2d to avoid z comparisons.
 *
 * Revision 1.59  2010/11/09 17:35:23  mju
 * Add drect2d::set using any opposite corners.
 *
 * Revision 1.54  2009/05/15 12:59:48  mju
 * Add alignToMultiple.
 *
 * Revision 1.49  2007/05/03 21:28:20  mju
 * Add getRectRoundLimited/NoLimit/Positive, deprecate getRect.
 *
 * Revision 1.45  2004/11/16 16:53:28  dwilliss
 * Added a GetRectTruncate to DRECT2D
 *
 * Revision 1.41  2003/11/12 13:20:00  mju
 * Add drect3d.getMinimum/Maximum.
 *
 * Revision 1.8  1999/12/02 23:23:43  mju
 * Add ClipTo method.
 *
 * Revision 1.4  1999/05/12  16:50:02  mju
 * Add DRECT3D ctor that takes DRECT2D and z min/max values.
 *
 * Revision 1.1  1999/05/05  17:30:09  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_RECT_H
#define	INC_MI32_RECT_H

//*****************************************************************************
//		The structures defined here must not change in size as they are stored
//		in many existing data files, as well as passed to DLLs.  The members
//		must remain public and their names may not be changed for any reason.
//*****************************************************************************

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifdef ALLOW_NAMESPACES
	#include <limits>
	#include <cmath>
#else
	#include <limits.h>
	#include <float.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT

#ifdef GEOMDLL
	#define GEOMLIBEXPORT MI_DLLEXPORT
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define GEOMLIBEXPORT MI_DLLIMPORT
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

GEOMLIBEXPORT void LRect2DGetCorners (const LRECT2D &rect, LPOINT2D *pts, bool ClosePoly, bool OrientCCW);
GEOMLIBEXPORT bool DRect2DComputeEdgePoint (const DRECT2D &rect, const DPOINT2D &OutsidePt, DPOINT2D &EdgePt);
GEOMLIBEXPORT void DRect2DGetCorners (const DRECT2D &rect, DPOINT2D *pts, bool ClosePoly, bool OrientCCW);
GEOMLIBEXPORT void DRect2DGetCorners (const DRECT2D &rect, DPOINT3D *pts, bool ClosePoly, bool OrientCCW);
GEOMLIBEXPORT double DRect2DGetDistance (const DRECT2D &trect, const DRECT2D &rect);
GEOMLIBEXPORT double DRect2DGetMaxDistance (const DRECT2D &trect, const DRECT2D &rect);
GEOMLIBEXPORT double DRect2DGetMaxDistanceSquared (const DRECT2D &trect, const DRECT2D &rect);
GEOMLIBEXPORT bool DRect2DIsEquivalentTo (const DRECT2D &trect, const DRECT2D &rect, double thresold);

#endif // GENERATING_DOXYGEN_OUTPUT

//-------------------------------------------------------------------------------------------------------------------

//! 2D rectangle using 32-bit integer coordinates.
struct LRECT2D {

	INT32 xinit;							//!< Minimum X coordinate (inclusive)
	INT32 yinit;							//!< Minimum Y coordinate (inclusive)
	INT32 xlast;							//!< Maximum X coordinate (inclusive)
	INT32 ylast;							//!< Maximum Y coordinate (inclusive)

	//! Default constructor, initializes to invalid (empty) rectangle.
	LRECT2D (
		): xinit(INT32_MAX), yinit(INT32_MAX), xlast(INT32_MIN), ylast(INT32_MIN) { }

	//! Construct rectangle of specified size.
	LRECT2D (
		INT32 xsize,
		INT32 ysize
		) : xinit(0), yinit(0), xlast(xsize-1), ylast(ysize-1) { }

	//! Construct from LPOINT2D.
	LRECT2D (
		const LPOINT2D pt
		) : xinit(pt.x), yinit(pt.y), xlast(pt.x), ylast(pt.y) { }

	//! Construction from X/Y ranges.
	LRECT2D (
		INT32 xi,
		INT32 yi,
		INT32 xl,
		INT32 yl
		) : xinit(xi), yinit(yi), xlast(xl), ylast(yl) { }

	//! Check if contains specified point.
	bool Contains (
		INT32 x,
		INT32 y
		) const { return (x >= xinit && x <= xlast && y >= yinit && y <= ylast); }

	//! Check if contains specified point.
	bool Contains (
		const LPOINT2D &pt
		) const { return (pt.x >= xinit && pt.x <= xlast && pt.y >= yinit && pt.y <= ylast); }

	//! Check if contains specified rectangle.
	bool Contains (
		const LRECT2D &rhs
		) const { return (rhs.xinit >= xinit && rhs.xlast <= xlast && rhs.yinit >= yinit && rhs.ylast <= ylast); }

	//! Expand rectangle by specified amount.
	void Expand (
		const INT32 value
		) { xinit -= value; yinit -= value; xlast += value; ylast += value; }
	
	//! Expand rectangle by specified amounts.
	void Expand (
		const INT32 dx,
		const INT32 dy
		) { xinit -= dx; yinit -= dy; xlast += dx; ylast += dy; }

	//! Extend rectangle to encompass both rectangles.
	void Extend (
		const LRECT2D &rect
		) {
		if (rect.xinit < xinit) xinit = rect.xinit;
		if (rect.yinit < yinit) yinit = rect.yinit;
		if (rect.xlast > xlast) xlast = rect.xlast;
		if (rect.ylast > ylast) ylast = rect.ylast;
		}

	//! Extend rectangle to include specified LPOINT2D point.
	void Extend (
		const LPOINT2D &pt
		) {
		if (pt.x < xinit) xinit = pt.x;
		if (pt.x > xlast) xlast = pt.x;
		if (pt.y < yinit) yinit = pt.y;
		if (pt.y > ylast) ylast = pt.y;
		}

	//! Extend rectangle to include specified X,Y point.
	void Extend (
		const INT32 dx,
		const INT32 dy
		) {
		if (dx < xinit) xinit = dx;
		if (dy < yinit) yinit = dy;
		if (dx > xlast) xlast = dx;
		if (dy > ylast) ylast = dy;
		}

	//! Compute area of rectangle (Could be negative if invalid rectangle).
	INT32 GetArea (
		) const { return (GetWidth() * GetHeight()); }

	//! Return bottom left corner.
	void GetBottomLeft (
		LPOINT2D &ret
		) const { ret.x = xinit; ret.y = yinit; }

	//! Get bottom right corner.
	void GetBottomRight (
		LPOINT2D &ret
		) const { ret.x = xlast; ret.y = yinit; }

	//! Return center point of rectangle.
	DPOINT2D GetCenter (
		) const { return (DPOINT2D((xlast + xinit)/2.0, (ylast + yinit)/2.0)); }

	//! Get center point of rectangle.
	void GetCenter (
		DPOINT2D &ret
		) const { ret.x = (xinit + xlast) / 2.0; ret.y = (yinit + ylast) / 2.0; }

	//! Get corners as array.
	void GetCorners (
		LPOINT2D *corners,					//!< Array of points to fill in
		bool ClosePoly = false,				//!< Return "closed" polygon, corners[4] = corners[0]
		bool OrientCCW = false				//!< Orient points in counterclockwise order for right-hand cartesian
		) const { LRect2DGetCorners(*this, corners, ClosePoly, OrientCCW); }

	//! Compute number of non zero dimensions (0, 1, or 2).
	int GetDimension (
		) const {
		int count(0);
		if (xinit < xlast) count ++;
		if (yinit < ylast) count ++;
		return (count);
		}

	//! Return height (Y size) of rectangle.
	UINT32 GetHeight (
		) const { return (ylast - yinit + 1); }

	//! Get maximum X, maximum Y values.
	LPOINT2D GetMaxXMaxY (
		) const { return (LPOINT2D(xlast,ylast)); }

	//! Get maximum X, minimum Y values.
	LPOINT2D GetMaxXMinY (
		) const { return (LPOINT2D(xlast,yinit)); }

	//! Get minimum X, maximum Y values.
	LPOINT2D GetMinXMaxY (
		) const { return (LPOINT2D(xinit,ylast)); }

	//! Get minimum X, minimum Y values.
	LPOINT2D GetMinXMinY (
		) const { return (LPOINT2D(xinit,yinit)); }

	//! Get size of rectangle.
	void GetSize (
		LPOINT2D &ret
		) const { ret.x = xlast - xinit; ret.y = ylast - yinit; }

	//! Return top left point of rectangle.
	void GetTopLeft (
		LPOINT2D &ret
		) const { ret.x = xinit; ret.y = ylast; }

	//! Return top right point of rectangle.
	void GetTopRight (
		LPOINT2D &ret
		) const { ret.x = xlast; ret.y = ylast; }

	//! Return width (X size) of rectangle.
	UINT32 GetWidth (
		) const { return (xlast - xinit + 1); }

	//! Read rectangle from INI file.
	bool IniRead (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName
		) { return (_iniRead(IniHandle,IniGroup,IniName,INITYPE_INT32,this,4) == 4); }

	//! Read rectangle from INI file.
	void IniWrite (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName
		) const { _iniWrite(IniHandle,IniGroup,IniName,INITYPE_INT32,this,4); }

	//! Intersect two DRECT2D's, may result in an invalid rectangle.
	void Intersect (
		const LRECT2D &rect
		) {
		if (rect.IsValid()) {
			if (rect.xinit > xinit) xinit = rect.xinit;
			if (rect.yinit > yinit) yinit = rect.yinit;
			if (rect.xlast < xlast) xlast = rect.xlast;
			if (rect.ylast < ylast) ylast = rect.ylast;
			}
		else {
			*this = rect;
			}
		return;
		}

	//! Check if rectangle is non-empty (has positive area).
	bool IsNonEmpty (
		) const { return (xinit < xlast && yinit < ylast); }

	//! Check if rectangle is valid.
	bool IsValid (
		) const { return (xinit <= xlast && yinit <= ylast); }

	//! Limit point (LPOINT2D) to extents specified by rectangle.
	void Limit (
		LPOINT2D &point
		) const {
		if (point.x < xinit) point.x = xinit; else if (point.x > xlast) point.x = xlast;
		if (point.y < yinit) point.y = yinit; else if (point.y > ylast) point.y = ylast;
		}

	//! Limit point (DPOINT2D) to extents specified by rectangle.
	void Limit (
		DPOINT2D &point
		) const {
		if (point.x < xinit) point.x = xinit; else if (point.x > xlast) point.x = xlast;
		if (point.y < yinit) point.y = yinit; else if (point.y > ylast) point.y = ylast;
		}

	//! Check if rectangle overlaps.
	bool Overlaps (
		const LRECT2D &rect
		) const { return (!(rect.xlast < xinit || rect.xinit > xlast || rect.ylast < yinit || rect.yinit > ylast)); }
	
	//! Set given (x1,y1), (x2,y2)
	void Set (
		INT32 x1,
		INT32 y1,
		INT32 x2,
		INT32 y2
		) { xinit = x1; yinit = y1; xlast = x2; ylast = y2; }

	//! Sets the initial values to the maximum possible and the final values to the minimum possible, thus making the rectangle "invalid".
	//! Setting a rectangle to invalid means that no point will be in it.  It also makes it easy 
	//! to compute the min/max of a bunch of points.  You just iterate through all the points 
	//! and Extend() the rectangle.  Normally, you'd have to handle the first point specially.
	void SetInvalid (
		) { xinit = yinit = INT32_MAX; xlast = ylast = INT32_MIN; }

	//! Perform byte-order swapping.
	void SwapBytes (
		) { ::SwapBytes(xinit); ::SwapBytes(xlast); ::SwapBytes(yinit); ::SwapBytes(ylast); }

	};

inline bool operator== (
	const LRECT2D &lhs,
	const LRECT2D &rhs
	) {
	return (lhs.xinit == rhs.xinit && lhs.xlast == rhs.xlast && lhs.yinit == rhs.yinit && lhs.ylast == rhs.ylast);
	}

inline bool operator!= (
	const LRECT2D &lhs,
	const LRECT2D &rhs
	) {
	return (!operator==(lhs,rhs));
	}

//-------------------------------------------------------------------------------------------------------------------

//! 3D box using INT32 coordinates.
struct LRECT3D : public LRECT2D {

	INT32 zinit;
	INT32 zlast;

	//! Initialize to nonexistent area.
	LRECT3D (
		) : LRECT2D(), zinit(INT32_MAX), zlast(INT32_MIN) { }

	//! Construct from LPOINT3D.
	LRECT3D (
		const LPOINT3D &pt
		) : LRECT2D(pt), zinit(pt.z), zlast(pt.z) { }

	//! Construct from two LPOINT3D's
	//! A regular ctor from LPOINT3D and then a call to ::Extend(LPOINT3D&) would use
	//! 9 assignments and 6 comparisons, this uses 3 compares and 6 assignments
	LRECT3D (
		const LPOINT3D &pt1,
		const LPOINT3D &pt2
		) {
		if (pt1.x < pt2.x) {
			xinit = pt1.x;
			xlast = pt2.x;
			}
		else {
			xinit = pt2.x;
			xlast = pt1.x;
			}
		if (pt1.y < pt2.y) {
			yinit = pt1.y;
			ylast = pt2.y;
			}
		else {
			yinit = pt2.y;
			ylast = pt1.y;
			}
		if (pt1.z < pt2.z) {
			zinit = pt1.z;
			zlast = pt2.z;
			}
		else {
			zinit = pt2.z;
			zlast = pt1.z;
			}
		}

	//! Construct from LRECT2D.
	explicit LRECT3D (
		const LRECT2D &rect
		) : LRECT2D(rect), zinit(0), zlast(0) { }

	//! Construct from LRECT2D and Z min/max.
	LRECT3D (
		const LRECT2D &rect,
		INT32 zmin,
		INT32 zmax
		) : LRECT2D(rect), zinit(zmin), zlast(zmax) { }

	//! Assignment from LPOINT3D
	LRECT3D& operator= (
		const LPOINT3D &pt
		) {
		xinit = xlast = pt.x;
		yinit = ylast = pt.y;
		zinit = zlast = pt.z;
		return (*this);
		}

	//! Check if point is inside.
	bool Contains (
		const LPOINT2D &pt
		) const { return (LRECT2D::Contains(pt)); }
	
	//! Check if point is inside.
	bool Contains (
		const LPOINT3D &pt
		) const { return (LRECT2D::Contains(pt) && pt.z >= zinit && pt.z <= zlast); }
	
	//! Check if rectangle is inside.
	bool Contains (
		const LRECT2D &rhs
		) const { return (LRECT2D::Contains(rhs)); }

	//! Check if rectangle is inside.
	bool Contains (
		const LRECT3D &rhs
		) const { return (LRECT2D::Contains(rhs) && rhs.zinit >= zinit && rhs.zlast <= zlast); }

	//! Expand rectangle by specified amount.
	void Expand (
		const INT32 value
		) {
		LRECT2D::Expand(value);
		zinit -= value; zlast += value;
		}

	//! Expand rectangle by specified amounts.
	void Expand (
		const INT32 dx,
		const INT32 dy
		) { LRECT2D::Expand(dx, dy); }

	//! Extend rectangle to encompass both rectangles.
	void Extend (
		const INT32 dx,
		const INT32 dy
		) { LRECT2D::Extend(dx, dy); }

	//! Extend rectangle to encompass both rectangles.
	void Extend (
		const INT32 dx,
		const INT32 dy,
		const INT32 dz
		) {
		LRECT2D::Extend(dx, dy);
		if (dz < zinit) zinit = dz;
		if (dz > zlast) zlast = dz;
		return;
		}

	//! Union rectangles together, resulting rect is extents of both
	void Extend (
		const LRECT3D &rect
		) {
		LRECT2D::Extend(rect);
		if (rect.zinit < zinit) zinit = rect.zinit;
		if (rect.zlast > zlast) zlast = rect.zlast;
		return;
		}

	//! Extend to contain specified point.
	void Extend (
		const LPOINT2D &pt
		) { LRECT2D::Extend(pt); }

	//! Extend to contain specified point.
	void Extend (
		const LPOINT3D &pt
		) {
		LRECT2D::Extend(pt);
		if (pt.z < zinit) zinit = pt.z;
		if (pt.z > zlast) zlast = pt.z;
		}

	//! Return center of the box.
	DPOINT3D GetCenter (
		) const { return (DPOINT3D((xinit+xlast)/2.0,(yinit+ylast)/2.0,(zinit+zlast)/2.0)); }

	//! Return center of the box.
	void GetCenter (
		DPOINT3D &ret
		) const {
		ret.x = (xinit + xlast) / 2.0;
		ret.y = (yinit + ylast) / 2.0;
		ret.z = (zinit + zlast) / 2.0;
		}

	//! Return depth of the box (z distance).
	INT32 GetDepth (
		) const { return (zlast - zinit); }

	//! Compute number of non zero dimensions (0, 1, 2, or 3).
	int GetDimension (
		) const {
		int count(LRECT2D::GetDimension());
		if (zinit < zlast) count ++;
		return (count);
		}

	//! Get maximum coordinate.
	void GetMaximum (
		LPOINT3D &maximum
		) const { maximum.x = xlast; maximum.y = ylast; maximum.z = zlast; }

	//! Get minimum coordinate.
	void GetMinimum (
		LPOINT3D &minimum
		) const { minimum.x = xinit; minimum.y = yinit; minimum.z = zinit; }

	//! Get size of box.
	void GetSize (
		LPOINT3D &ret
		) const {
		ret.x = xlast - xinit;
		ret.y = ylast - yinit;
		ret.z = zlast - zinit;
		}

	//! Compute volume of box.
	INT32 GetVolume (
		) const { return (LRECT2D::GetArea() * (zlast - zinit)); }

	//! Read rectangle from INI file.
	bool IniRead (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName
		) { return (_iniRead(IniHandle,IniGroup,IniName,INITYPE_INT32,this,6) == 6); }

	//! Read rectangle from INI file.
	void IniWrite (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName
		) const { _iniWrite(IniHandle,IniGroup,IniName,INITYPE_INT32,this,6); }

	//! Intersect two LRECT3D's, may result in an invalid rectangle.
	void Intersect (
		const LRECT3D &rect
		) {
		if (rect.IsValid()) {
			LRECT2D::Intersect(rect);
			if (rect.zinit > zinit) zinit = rect.zinit;
			if (rect.zlast < zlast) zlast = rect.zlast;
			}
		else {
			*this = rect;
			}
		return;
		}
	
	//! Check if rectangle is valid.
	bool IsValid (
		) const { return (LRECT2D::IsValid() && zinit <= zlast); }

	//! Check if rectangle overlaps.
	bool Overlaps (
		const LRECT3D &rhs
		) const { return (LRECT2D::Overlaps(rhs) && !(rhs.zlast < zinit || rhs.zinit > zlast)); }

	//! Set given (x1,y1), (x2,y2)
	void Set (
		INT32 x1,
		INT32 y1,
		INT32 x2,
		INT32 y2
		) { xinit = x1; yinit = y1; xlast = x2; ylast = y2; zinit = zlast = 0; }

	//! Set given (xmin,ymin,zmin), (xmax,ymax,zmax)
	void Set (
		INT32 xmin,
		INT32 ymin,
		INT32 zmin,
		INT32 xmax,
		INT32 ymax,
		INT32 zmax
		) { xinit = xmin; yinit = ymin; zinit = zmin; xlast = xmax; ylast = ymax; zlast = zmax; }

	//! Sets the initial values to the maximum possible and the final values to the minimum possible, thus making the rectangle "invalid".
	//! Setting a rectangle to invalid means that no point will be in it.  It also makes it easy 
	//! to compute the min/max of a bunch of points.  You just iterate through all the points 
	//! and Extend() the rectangle.  Normally, you'd have to handle the first point specially.
	void SetInvalid (
		) { LRECT2D::SetInvalid(); zinit = INT32_MAX; zlast = INT32_MIN; }

	//! Perform byte-order swapping.
	void SwapBytes (
		) { ::SwapBytes(xinit); ::SwapBytes(xlast); ::SwapBytes(yinit); ::SwapBytes(ylast); ::SwapBytes(zinit); ::SwapBytes(zlast); }

	};

//-------------------------------------------------------------------------------------------------------------------

//! 2D rectangle using 'double' precision coordinates.
struct DRECT2D {
	
	double xinit;							//!< Minimum X coordinate (inclusive)
	double yinit;							//!< Minimum Y coordinate (inclusive)
	double xlast;							//!< Maximum X coordinate (inclusive)
	double ylast;							//!< Maximum Y coordinate (inclusive)

	//! Default constructor, initialize to "invalid" area.
	DRECT2D (
		) : xinit(DBL_MAX), yinit(DBL_MAX), xlast(-DBL_MAX), ylast(-DBL_MAX) { }

	//! Construct from FPOINT2D (or FPOINT3D).
	DRECT2D (
		const FPOINT2D &pt
		) : xinit(pt.x), yinit(pt.y), xlast(pt.x), ylast(pt.y) { }

	//! Construct from DPOINT2D (or DPOINT3D).
	DRECT2D (
		const DPOINT2D &pt
		) : xinit(pt.x), yinit(pt.y), xlast(pt.x), ylast(pt.y) { }

	//! Construct from two DPOINT2D's (or DPOINT3D's).
	//! A regular ctor from DPOINT2D and then a call to ::Extend(DPOINT2D&) would use
	//! 6 assignments and 4 comparisons, this uses 2 compares and 4 assignments
	DRECT2D (
		const DPOINT2D &pt1,
		const DPOINT2D &pt2
		) { SetCorners(pt1,pt2); }

	DRECT2D (
		double xi,
		double yi,
		double xl,
		double yl
		) : xinit(xi), yinit(yi), xlast(xl), ylast(yl) { }

	//! Implicit conversion from LRECT2D.
	DRECT2D (
		const LRECT2D &rect
		) : xinit(rect.xinit), yinit(rect.yinit), xlast(rect.xlast), ylast(rect.ylast) { }

	//! Assignment from DPOINT2D
	DRECT2D& operator= (
		const DPOINT2D &pt
		) { xinit = xlast = pt.x; yinit = ylast = pt.y; return (*this); }

	//! Assignment from LRECT2D.
	DRECT2D& operator= (
		const LRECT2D &rhs
		) { xinit = rhs.xinit; yinit = rhs.yinit; xlast = rhs.xlast; ylast = rhs.ylast; return (*this); }

	//! Align rectangle to integer multiple of specified size.
	void AlignToMultiple (
		double AlignSize,
		double RoundTolerance = .000001
		) {
		xinit = floor(xinit / AlignSize + RoundTolerance) * AlignSize;
		yinit = floor(yinit / AlignSize + RoundTolerance) * AlignSize;
		xlast = ceil(xlast / AlignSize - RoundTolerance) * AlignSize;
		ylast = ceil(ylast / AlignSize - RoundTolerance) * AlignSize;
		}

	//! Calculate length of diagonal in 2D.
	double CalcDiagonalLength2D () const
		{ double dx = xlast - xinit; double dy = ylast - yinit; return (sqrt(dx*dx+dy*dy)); }

	//! Clip this rectangle to specified rectangle.
	void ClipTo (
		const DRECT2D &rect
		) {
		if (xinit < rect.xinit) xinit = rect.xinit;
		if (yinit < rect.yinit) yinit = rect.yinit;
		if (xlast > rect.xlast) xlast = rect.xlast;
		if (ylast > rect.ylast) ylast = rect.ylast;
		}

	//! Given the center point of the DRECT2D and the 'TestPt' which is outside the box,
	//! compute the point on the edge of the box which falls between the center point and
	//! the 'TestPt'.
	//! @return 'true' if the point is found, false if the 'TestPt' is inside the box or the box is invalid
	bool ComputeEdgePoint (
		const DPOINT2D &OutsidePt,			//!< Point outside of 'this' 
		DPOINT2D &EdgePt						//!< Point on rectangle edge RETURNED if 'true' returned, center of the box if 'false' is returned
		) const { return (DRect2DComputeEdgePoint(*this, OutsidePt, EdgePt)); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if contains specified point.
	DEPRECATED_MSG("Use Contains2D") bool Contains (
		double x,
		double y
		) const { return (Contains2D(x,y)); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if contains specified point.
	DEPRECATED_MSG("Use Contains2D") bool Contains (
		const DPOINT2D &pt
		) const { return (Contains2D(pt.x,pt.y)); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if contains specified point.
	DEPRECATED_MSG("Use Contains2D") bool Contains (
		const DRECT2D &rect
		) const { return (Contains2D(rect)); }
	#endif

	//! Check if contains specified point.
	bool Contains2D (
		double x,
		double y
		) const { return (x >= xinit && x <= xlast && y >= yinit && y <= ylast); }

	//! Check if contains specified point.
	bool Contains2D (
		const DPOINT2D &pt
		) const { return (Contains2D(pt.x,pt.y)); }

	//! Check if contains specified rectangle.
	bool Contains2D (
		const DRECT2D &rect
		) const { return (rect.xinit >= xinit && rect.xlast <= xlast && rect.yinit >= yinit && rect.ylast <= ylast); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Expand rectangle by specified amount in 2 dimensions.
	DEPRECATED_MSG("Use Expand2D") void Expand (
		const double value
		) { Expand2D(value,value); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Expand rectangle by specified amounts.
	DEPRECATED_MSG("Use Expand2D") void Expand (
		const double x,
		const double y
		) { Expand2D(x,y); }
	#endif

	//! Expand rectangle by specified amount in 2 dimensions.
	void Expand2D (
		const double value
		) { Expand2D(value,value); }

	//! Expand rectangle by specified amounts.
	void Expand2D (
		const double x,
		const double y
		) { xinit -= x; yinit -= y; xlast += x; ylast += y; }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Extend rectangle to include specified X,Y point.
	DEPRECATED_MSG("Use Extend2D") void Extend (
		const double x,
		const double y
		) { Extend2D(x,y); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Extend rectangle to encompass both rectangles.
	DEPRECATED_MSG("Use Extend2D") void Extend (
		const DRECT2D &rect
		) { Extend2D(rect); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Extend rectangle to include specified DPOINT2D point.
	DEPRECATED_MSG("Use Extend2D") void Extend (
		const DPOINT2D &pt
		) { Extend2D(pt); }
	#endif

	//! Extend rectangle to include specified X,Y point.
	void Extend2D (
		const double x,
		const double y
		) {
		if (x < xinit) xinit = x;
		if (y < yinit) yinit = y;
		if (x > xlast) xlast = x;
		if (y > ylast) ylast = y;
		}

	//! Extend rectangle to encompass both rectangles.
	void Extend2D (
		const DRECT2D &rect
		) {
		if (rect.xinit < xinit) xinit = rect.xinit;
		if (rect.yinit < yinit) yinit = rect.yinit;
		if (rect.xlast > xlast) xlast = rect.xlast;
		if (rect.ylast > ylast) ylast = rect.ylast;
		}

	//! Extend rectangle to include specified DPOINT2D point.
	void Extend2D (
		const DPOINT2D &pt
		) { Extend2D(pt.x,pt.y); }

	//! Compute area of rectangle.
	double GetArea () const
		{ return ((xlast - xinit) * (ylast - yinit)); }

	//! Return bottom left point of rectangle.
	void GetBottomLeft (
		DPOINT2D &ret
		) const { ret.x = xinit; ret.y = yinit; }

	//! Return bottom right point of rectangle.
	void GetBottomRight (
		DPOINT2D &ret
		) const { ret.x = xlast; ret.y = yinit; }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Return center point of rectangle.
	DEPRECATED_MSG("Use GetCenter2D") DPOINT2D GetCenter () const
		{ return (GetCenter2D()); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Return center point of rectangle.
	DEPRECATED_MSG("Use GetCenter2D") void GetCenter (
		DPOINT2D &ret
		) const { ret.x = (xinit + xlast) / 2.0; ret.y = (yinit + ylast) / 2.0; }
	#endif

	//! Return center point of rectangle.
	DPOINT2D GetCenter2D () const
		{ return (DPOINT2D((xlast + xinit)/2.0, (ylast + yinit)/2.0)); }

	//! Return center point of rectangle.
	void GetCenter2D (
		DPOINT2D &ret
		) const { ret.x = (xinit + xlast) / 2.0; ret.y = (yinit + ylast) / 2.0; }

	//! Get corners as array.
	void GetCorners (
		DPOINT2D *corners,				  	//!< Array of points to fill in
		bool ClosePoly = false,				//!< Return "closed" polygon, corners[4] = corners[0]
		bool OrientCCW = false				//!< Orient points in counterclockwise order for right-hand cartesian
		) const { DRect2DGetCorners(*this, corners, ClosePoly, OrientCCW); }

	//! Get corners as array.
	void GetCorners (
		DPOINT3D *corners,				  	//!< Array of points to fill in
		bool ClosePoly = false,				//!< Return "closed" polygon, corners[4] = corners[0]
		bool OrientCCW = false				//!< Orient points in counterclockwise order for right-hand cartesian
		) const { DRect2DGetCorners(*this, corners, ClosePoly, OrientCCW); }

	//! Compute number of non zero dimensions (0, 1, or 2).
	int GetDimension (
		) const {
		int count(0);
		if (xinit < xlast) count ++;
		if (yinit < ylast) count ++;
		return (count);
		}

	//! Compute the distance between two disjoint rectangles
	//! @return Distance between two disjoint rectangles, 0.0 if they overlap.
	double GetDistance (
		const DRECT2D &rect
		) const { return (DRect2DGetDistance(*this, rect)); }

	//! Return height of rectangle.
	double GetHeight () const
		{ return (ylast - yinit); }

	//! Compute the maximum distance between two rectangles
	//! Uses Extend() and then computes the diagonal distance
	//! @return Max distance between two rectangles
	double GetMaxDistance (
		const DRECT2D &rect
		) const { return (DRect2DGetMaxDistance(*this, rect)); }

	//! Compute the maximum distance between two rectangles
	//! Uses Extend() and then computes the diagonal distance
	//! @return Max distance between two rectangles
	double GetMaxDistanceSquared (
		const DRECT2D &rect
		) const { return (DRect2DGetMaxDistanceSquared(*this, rect)); }

	//! Get rectangle as LRECT2D with rounding.
	void GetRectRoundExact (
		LRECT2D &rect
		) const {
		rect.xinit = static_cast<INT32>(FAST_ROUND_EXACT(xinit));
  		rect.yinit = static_cast<INT32>(FAST_ROUND_EXACT(yinit));
		rect.xlast = static_cast<INT32>(FAST_ROUND_EXACT(xlast));
		rect.ylast = static_cast<INT32>(FAST_ROUND_EXACT(ylast));
		}

	//! Get rectangle as LRECT2D with rounding, subject to range restrictions of FAST_ROUND_LIMITED.
	void GetRectRoundLimited (
		LRECT2D &rect
		) const {
		rect.xinit = static_cast<INT32>(FAST_ROUND_LIMITED(xinit));
  		rect.yinit = static_cast<INT32>(FAST_ROUND_LIMITED(yinit));
		rect.xlast = static_cast<INT32>(FAST_ROUND_LIMITED(xlast));
		rect.ylast = static_cast<INT32>(FAST_ROUND_LIMITED(ylast));
		}

	//! Get rectangle as LRECT2D with rounding, valid for all positive coordinates only.
	void GetRectRoundPositive (
		LRECT2D &rect
		) const {
		rect.xinit = static_cast<INT32>(FAST_ROUND_POSITIVE(xinit));
  		rect.yinit = static_cast<INT32>(FAST_ROUND_POSITIVE(yinit));
		rect.xlast = static_cast<INT32>(FAST_ROUND_POSITIVE(xlast));
		rect.ylast = static_cast<INT32>(FAST_ROUND_POSITIVE(ylast));
		}

	//! Get rectangle as an LRECT2D (with truncating towards 0)
	void GetRectTruncate (
		LRECT2D &rect
		) const {
		rect.xinit = static_cast<INT32>(FAST_TRUNCATE(xinit));
  		rect.yinit = static_cast<INT32>(FAST_TRUNCATE(yinit));
		rect.xlast = static_cast<INT32>(FAST_TRUNCATE(xlast));
		rect.ylast = static_cast<INT32>(FAST_TRUNCATE(ylast));
		}

	//! Get size of rectangle.
	void GetSize (
		DPOINT2D &ret
		) const { ret.x = xlast - xinit; ret.y = ylast - yinit; }

	//! Return top left point of rectangle.
	void GetTopLeft (
		DPOINT2D &ret
		) const { ret.x = xinit; ret.y = ylast; }

	//! Return top right point of rectangle.
	void GetTopRight (
		DPOINT2D &ret
		) const { ret.x = xlast; ret.y = ylast; }

	//! Return width of rectangle.
	double GetWidth () const
		{ return (xlast - xinit); }

	//! Read rectangle from INI file.
	bool IniRead (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName
		) { return (_iniRead(IniHandle,IniGroup,IniName,INITYPE_DRECT2D,this,1) == 1); }

	//! Read rectangle from INI file with default.
	bool IniRead (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName,
		const DRECT2D &dft
		) { if (IniRead(IniHandle,IniGroup,IniName)) return (true); else { *this = dft; return (false); } }

	//! Read rectangle from INI file.
	void IniWrite (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName
		) const { _iniWrite(IniHandle,IniGroup,IniName,INITYPE_DRECT2D,this,1); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Intersect two DRECT2D's, may result in an invalid rectangle.
	DEPRECATED_MSG("Use Intersect2D") void Intersect (
		const DRECT2D &rect
		) { Intersect2D(rect); }
	#endif

	//! Intersect two DRECT2D's, may result in an invalid rectangle.
	void Intersect2D (
		const DRECT2D &rect
		) {
		if (rect.IsValid2D()) {
			if (rect.xinit > xinit) xinit = rect.xinit;
			if (rect.yinit > yinit) yinit = rect.yinit;
			if (rect.xlast < xlast) xlast = rect.xlast;
			if (rect.ylast < ylast) ylast = rect.ylast;
			}
		else {
			*this = rect;
			}
		}

	//! Determine if two rectangles have non-empty intersection.
	bool Intersects2D (
		const DRECT2D &rect
		) const { return (rect.xlast > xinit && rect.xinit < xlast && rect.ylast > yinit && rect.yinit < ylast); }

	//! Determine if the two rectangles are equivalent 
	//! This method uses "Fuzzy" edge comparisons
	//! Specify a different threshold to allow greater distances
	//! @return 'True' if they are, 'False' if not
	bool IsEquivalentTo (
		const DRECT2D &rect,
		double threshold = 0.0			//!< Default threshold is max absolute value of rectangle coordinate * 1.0E-13
		) const { return (DRect2DIsEquivalentTo(*this, rect, threshold)); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if rectangle is non-empty (has positive area).
	DEPRECATED_MSG("Use IsNonEmpty2D") bool IsNonEmpty () const
		{ return (IsNonEmpty2D()); }
	#endif

	//! Check if rectangle is non-empty (has positive area).
	bool IsNonEmpty2D () const
		{ return (xinit < xlast && yinit < ylast); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if rectangle is valid.
	DEPRECATED_MSG("Use IsValid2D") bool IsValid () const
		{ return (IsValid2D()); }
	#endif

	//! Check if rectangle is valid.
	bool IsValid2D () const
		{ return (xinit <= xlast && yinit <= ylast); }

	//! Limit point to extents specified by rectangle.
	void Limit (
		DPOINT2D &point
		) const {
		if (point.x < xinit) point.x = xinit; else if (point.x > xlast) point.x = xlast;
		if (point.y < yinit) point.y = yinit; else if (point.y > ylast) point.y = ylast;
		}

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if rectangle overlaps.
	DEPRECATED_MSG("Use Overlaps2D") bool Overlaps (
		const DRECT2D &rect
		) const { return (Overlaps2D(rect)); }
	#endif

	//! Check if rectangle overlaps.
	bool Overlaps2D (
		const DRECT2D &rect
		) const { return (!(rect.xlast < xinit || rect.xinit > xlast || rect.ylast < yinit || rect.yinit > ylast)); }

	//! Set given (x1,y1), (x2,y2) with no ordering test.
	void Set (
		double x1,
		double y1,
		double x2,
		double y2
		) { xinit = x1; yinit = y1; xlast = x2; ylast = y2; }

	//! Set 2D values.
	void Set2D (
		const DRECT2D &rect2d
		) { operator=(rect2d); }

	//! Set from any two opposite corners.
	void SetCorners (
		const DPOINT2D pt1,
		const DPOINT2D pt2
		) {
		if (pt1.x < pt2.x) { xinit = pt1.x; xlast = pt2.x; }
		else { xinit = pt2.x; xlast = pt1.x; }
		if (pt1.y < pt2.y) { yinit = pt1.y; ylast = pt2.y; }
		else { yinit = pt2.y; ylast = pt1.y; }
		}

	//! Sets the initial values to the maximum possible and the final values to the minimum possible, thus making the rectangle "invalid".
	//! Setting a rectangle to invalid means that no point will be in it.  It also makes it easy 
	//! to compute the min/max of a bunch of points.  You just iterate through all the points 
	//! and Extend() the rectangle.  Normally, you'd have to handle the first point specially.
	void SetInvalid (
		) { xinit = yinit = DBL_MAX; xlast = ylast = -DBL_MAX; }

	//! Perform byte-order swapping.
	void SwapBytes (
		) { ::SwapBytes(xinit); ::SwapBytes(xlast); ::SwapBytes(yinit); ::SwapBytes(ylast); }

	};

inline bool operator== (
	const DRECT2D &lhs,
	const DRECT2D &rhs
	) {
	return (lhs.xinit == rhs.xinit && lhs.xlast == rhs.xlast && lhs.yinit == rhs.yinit && lhs.ylast == rhs.ylast);
	}

inline bool operator!= (
	const DRECT2D &lhs,
	const DRECT2D &rhs
	) {
	return (!operator==(lhs,rhs));
	}

inline void SwapBytes (
	DRECT2D &rect
	) {
	rect.SwapBytes();
	}

//-------------------------------------------------------------------------------------------------------------------

//! 3D rectangle using 'double' precision coordinates.
struct DRECT3D : public DRECT2D {
	
	double zinit;
	double zlast;

	//! Initialize to nonexistent area.
	DRECT3D (
		) : DRECT2D(), zinit(DBL_MAX), zlast(-DBL_MAX) { }

	//! Construct from FPOINT3D.
	DRECT3D (
		const FPOINT3D &pt
		) : DRECT2D(pt), zinit(pt.z), zlast(pt.z) { }

	//! Construct from DPOINT3D.
	DRECT3D (
		const DPOINT3D &pt
		) : DRECT2D(pt), zinit(pt.z), zlast(pt.z) { }

	//! Construct from two DPOINT3D's
	//! A regular ctor from DPOINT3D and then a call to ::Extend(DPOINT3D&) would use
	//! 9 assignments and 6 comparisons, this uses 3 compares and 6 assignments
	DRECT3D (
		const DPOINT3D &pt1,
		const DPOINT3D &pt2
		) {
		if (pt1.x < pt2.x) {
			xinit = pt1.x;
			xlast = pt2.x;
			}
		else {
			xinit = pt2.x;
			xlast = pt1.x;
			}
		if (pt1.y < pt2.y) {
			yinit = pt1.y;
			ylast = pt2.y;
			}
		else {
			yinit = pt2.y;
			ylast = pt1.y;
			}
		if (pt1.z < pt2.z) {
			zinit = pt1.z;
			zlast = pt2.z;
			}
		else {
			zinit = pt2.z;
			zlast = pt1.z;
			}
		}

	//! Construct from DRECT2D.
	explicit DRECT3D (
		const DRECT2D &rect
		) : DRECT2D(rect), zinit(0.0), zlast(0.0) { }

	//! Construct from DRECT2D and Z min/max.
	DRECT3D (
		const DRECT2D &rect,
		double zmin,
		double zmax
		) : DRECT2D(rect), zinit(zmin), zlast(zmax) { }

	//! Assignment from DPOINT3D
	DRECT3D& operator= (
		const DPOINT3D &pt
		) {
		xinit = xlast = pt.x;
		yinit = ylast = pt.y;
		zinit = zlast = pt.z;
		return (*this);
		}

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if point is inside.
	DEPRECATED_MSG("Use Contains2D") bool Contains (
		const DPOINT2D &pt
		) const { return (Contains2D(pt)); }
	#endif
	
	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if point is inside.
	DEPRECATED_MSG("Use Contains3D") bool Contains (
		const DPOINT3D &pt
		) const { return (Contains3D(pt)); }
	#endif
	
	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if rectangle is inside.
	DEPRECATED_MSG("Use Contains2D") bool Contains (
		const DRECT2D &rhs
		) const { return (Contains2D(rhs)); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if rectangle is inside.
	DEPRECATED_MSG("Use Contains3D") bool Contains (
		const DRECT3D &rect3d
		) const { return (Contains3D(rect3d)); }
	#endif

	//! Check if 3D point is inside.
	bool Contains3D (
		const DPOINT3D &pt
		) const { return (Contains2D(pt) && pt.z >= zinit && pt.z <= zlast); }
	
	//! Check if 3D rectangle is inside.
	bool Contains3D (
		const DRECT3D &rect3d
		) const { return (Contains2D(rect3d) && rect3d.zinit >= zinit && rect3d.zlast <= zlast); }

	//! Expand rectangle by specified amount.
	void Expand3D (
		const double value
		) { Expand2D(value); zinit -= value; zlast += value; }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Extend rectangle to include specified X,Y point.
	DEPRECATED_MSG("Use Extend2D") void Extend (
		const double x,
		const double y
		) { Extend2D(x, y); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Extend rectangle to include specified X,Y,Z point.
	DEPRECATED_MSG("Use Extend3D") void Extend (
		const double x,
		const double y,
		const double z
		) { Extend3D(x,y,z); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Extend to contain specified point.
	DEPRECATED_MSG("Use Extend2D") void Extend (
		const DPOINT2D &pt
		) { Extend2D(pt); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Extend to contain specified point.
	DEPRECATED_MSG("Use Extend3D") void Extend (
		const DPOINT3D &pt
		) { Extend3D(pt); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Union rectangles together, resulting rect is extents of both
	DEPRECATED_MSG("Use Extend2D") void Extend (
		const DRECT2D &rect
		) { Extend2D(rect); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Union rectangles together, resulting rect is extents of both
	DEPRECATED_MSG("Use Extend3D") void Extend (
		const DRECT3D &rect
		) { Extend3D(rect); }
	#endif

	//! Extend rectangle to include specified X,Y,Z point.
	void Extend3D (
		const double x,
		const double y,
		const double z
		) { Extend2D(x, y); if (z < zinit) zinit = z; if (z > zlast) zlast = z; }

	//! Extend to contain specified 3D point.
	void Extend3D (
		const DPOINT3D &pt
		) { Extend2D(pt); if (pt.z < zinit) zinit = pt.z; if (pt.z > zlast) zlast = pt.z; }

	//! Union rectangles together, resulting rect is extents of both
	void Extend3D (
		const DRECT3D &rect
		) { Extend2D(rect); if (rect.zinit < zinit) zinit = rect.zinit; if (rect.zlast > zlast) zlast = rect.zlast; }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Return center of the box.
	DEPRECATED_MSG("Use GetCenter3D") DPOINT3D GetCenter (
		) const { return (GetCenter3D()); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Return center of the box.
	DEPRECATED_MSG("Use GetCenter3D") void GetCenter (
		DPOINT3D &ret
		) const { GetCenter3D(ret); }
	#endif

	//! Return center of the box.
	DPOINT3D GetCenter3D () const
		{ return (DPOINT3D((xinit+xlast)/2.0,(yinit+ylast)/2.0,(zinit+zlast)/2.0)); }

	//! Return center of the box.
	void GetCenter3D (
		DPOINT3D &ret
		) const { ret.x = (xinit + xlast) / 2.0; ret.y = (yinit + ylast) / 2.0; ret.z = (zinit + zlast) / 2.0; }

	//! Return depth of the box (z distance).
	double GetDepth (
		) const { return (zlast - zinit); }

	//! Compute number of non zero dimensions (0, 1, 2, or 3).
	int GetDimension (
		) const {
		int count(DRECT2D::GetDimension());
		if (zinit < zlast) count ++;
		return (count);
		}

	//! Get maximum coordinate.
	void GetMaximum (
		DPOINT3D &maximum
		) const { maximum.x = xlast; maximum.y = ylast; maximum.z = zlast; }

	//! Get minimum coordinate.
	void GetMinimum (
		DPOINT3D &minimum
		) const { minimum.x = xinit; minimum.y = yinit; minimum.z = zinit; }

	//! Get size of box.
	void GetSize (
		DPOINT3D &ret
		) const {
		ret.x = xlast - xinit;
		ret.y = ylast - yinit;
		ret.z = zlast - zinit;
		}

	//! Compute volume of box.
	double GetVolume (
		) const { return (DRECT2D::GetArea() * (zlast - zinit)); }

	//! Read rectangle from INI file.
	bool IniRead (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName
		) { return (_iniRead(IniHandle,IniGroup,IniName,INITYPE_Double,this,6) == 6); }

	//! Read rectangle from INI file.
	void IniWrite (
		INIHANDLE IniHandle,
		const char *IniGroup,
		const char *IniName
		) const { _iniWrite(IniHandle,IniGroup,IniName,INITYPE_Double,this,6); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Intersect with DRECT2D, leaves Z values unchanged.
	DEPRECATED_MSG("Use Intersect2D") void Intersect (
		const DRECT2D &rect2d
		) { Intersect2D(rect2d); }
	#endif
	
	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Intersect two DRECT3D's, may result in an invalid rectangle.
	DEPRECATED_MSG("Use Intersect3D") void Intersect (
		const DRECT3D &rect
		) { Intersect3D(rect); }
	#endif

	//! Intersect two DRECT3D's, may result in an invalid rectangle.
	void Intersect3D (
		const DRECT3D &rect
		) {
		if (rect.IsValid3D()) {
			Intersect2D(rect);
			if (rect.zinit > zinit) zinit = rect.zinit;
			if (rect.zlast < zlast) zlast = rect.zlast;
			}
		else {
			*this = rect;
			}
		return;
		}
	
	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if rectangle is valid.
	DEPRECATED_MSG("Use IsValid3D") bool IsValid () const
		{ return (IsValid3D()); }
	#endif

	//! Check if rectangle is valid.
	bool IsValid3D () const
		{ return (IsValid2D() && zinit <= zlast); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if overlaps RECT2D.
	DEPRECATED_MSG("Use Overlaps2D") bool Overlaps (
		const DRECT2D &rect2d
		) const { return (Overlaps2D(rect2d)); }
	#endif

	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Check if rectangle overlaps.
	DEPRECATED_MSG("Use Overlaps3D") bool Overlaps (
		const DRECT3D &rhs
		) const { return (Overlaps3D(rhs)); }
	#endif

	//! Check if rectangle overlaps.
	bool Overlaps3D (
		const DRECT3D &rhs
		) const { return (Overlaps2D(rhs) && !(rhs.zlast < zinit || rhs.zinit > zlast)); }

	//! Set given (x1,y1), (x2,y2)
	void Set (
		double x1,
		double y1,
		double x2,
		double y2
		) { xinit = x1; yinit = y1; xlast = x2; ylast = y2; zinit = zlast = 0.0; }

	//! Set given (xmin,ymin,zmin), (xmax,ymax,zmax)
	void Set (
		double xmin,
		double ymin,
		double zmin,
		double xmax,
		double ymax,
		double zmax
		) { xinit = xmin; yinit = ymin; zinit = zmin; xlast = xmax; ylast = ymax; zlast = zmax; }

	void Set (
		const DRECT2D &rect2d,
		double zmin,
		double zmax
		) { DRECT2D::operator=(rect2d); zinit = zmin; zlast = zmax; }

	//! Sets the initial values to the maximum possible and the final values to the minimum possible, thus making the rectangle "invalid".
	//!
	//! Setting a rectangle to invalid means that no point will be in it.  It also makes it easy 
	//! to compute the min/max of a bunch of points.  You just iterate through all the points 
	//! and Extend() the rectangle.  Normally, you'd have to handle the first point specially.
	void SetInvalid (
		) { DRECT2D::SetInvalid(); zinit = DBL_MAX; zlast = -DBL_MAX; }

	//! Perform byte-order swapping.
	void SwapBytes (
		) { ::SwapBytes(xinit); ::SwapBytes(xlast); ::SwapBytes(yinit); ::SwapBytes(ylast); ::SwapBytes(zinit); ::SwapBytes(zlast); }

	};

inline bool operator== (
	const DRECT3D &lhs,
	const DRECT3D &rhs
	) {
	return (lhs.xinit == rhs.xinit && lhs.xlast == rhs.xlast && lhs.yinit == rhs.yinit && lhs.ylast == rhs.ylast && lhs.zinit == rhs.zinit && lhs.zlast == rhs.zlast);
	}

inline bool operator!= (
	const DRECT3D &lhs,
	const DRECT3D &rhs
	) {
	return (!operator==(lhs,rhs));
	}

inline void SwapBytes (
	DRECT3D &rect
	) {
	rect.SwapBytes();
	}

//-------------------------------------------------------------------------------------------------------------------

#endif	// INC_MI32_RECT_H
