/**
 * \file mi32/regionsource.h
 * \brief REGIONSOURCE definitions
 *
 * \if NODOC
 * $Id: regionsource.h_v 1.5 2013/10/28 15:06:02 mju Exp $
 *
 * $Log: regionsource.h_v $
 * Revision 1.5  2013/10/28 15:06:02  mju
 * Add regionsource_delegated.
 *
 * Revision 1.4  2013/10/23 21:38:37  mju
 * Eliminate treeviewfolder.
 *
 * Revision 1.3  2013/10/22 20:34:23  mju
 * Add regionsource_unowned.
 *
 * Revision 1.2  2013/10/21 19:57:41  mju
 * Use treeviewnode/folder/object.
 *
 * Revision 1.1  2013/10/21 19:26:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_REGIONSOURCE_H
#define	INC_MI32_REGIONSOURCE_H

#ifndef	INC_MI32_TREEVIEW_H
#include <mi32/treeview.h>
#endif

#ifndef	INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif

//=====================================================================================================================

//! Base class for region source.
class REGIONSOURCE : public TREEVIEWOBJECT {
	public:

		//! Destructor.
		virtual ~REGIONSOURCE ();

		//! Calculate region if necessary.
		//! Must done when need to access the region itself, prior to calling GetRegion.
		//! Should only be done when need actual region, as may take significant time.
		//! If an error occurs the source will be set as invalid and all observers will be notified.
		ERRVALUE CalcRegion ();

		//! Get extents if available, will return invalid extents if not available.
		const DRECT2D& GetExtents () const;

		//! Get RVC::OBJITEM used to reference source in manager.
		void GetObjItemManaged (
			RVC::OBJITEM& objitem		//!< Location suitable for use in REGIONMANAGER::FindSource
			) const;

		//! Get region.
		//! @return Region, will be empty if not valid or CalcRegion has not been called after changed.
		const REGION2D& GetRegion () const;

	protected:

		//! Construct as empty region.
		REGIONSOURCE (TREEVIEWNODE& parent, const MISTRING& name, TREEVIEWNODE *pNodeInsertBefore = 0);

		//! Construct with region.
		REGIONSOURCE (TREEVIEWNODE& parent, const REGION2D& region, const MISTRING& name, TREEVIEWNODE *pNodeInsertBefore = 0);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		mutable REGION2D m_region;

		virtual void v_NotifyChanged (bool bValid);
		#endif

		//! Calculate region for source.
		virtual ERRVALUE v_CalcRegion (
			REGION2D& region
			);

		//! Get maximal extents for region if available.
		//! Override if can efficiently determine maximum possible extents without computing the region.
		//! Default implementation returns NULL.
		virtual const DRECT2D* v_GetExtents () const;

		//! Get pointer to region.
		//! Should override if can directly get the region without having to compute,
		//! otherwise override v_CalcRegion.
		virtual const REGION2D* v_GetRegion () const;
	};

//=====================================================================================================================

//! Region source where actual region is maintained elsewhere.
class REGIONSOURCE_UNOWNED : public REGIONSOURCE {
	public:

		//! Construct with region pointer.
		REGIONSOURCE_UNOWNED (
			TREEVIEWNODE& parent,						//!< Parent node in TREEVIEW
			const REGION2D *pRegion,					//!< Pointer to region, must remain valid
			const MISTRING& name,						//!< Name to show in tree view
			TREEVIEWNODE *pNodeInsertBefore = 0		//!< Node to insert before
			);

		//! Destructor.
		virtual ~REGIONSOURCE_UNOWNED ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const REGION2D *m_pRegion;

		virtual const REGION2D* v_GetRegion () const;
		#endif
	};

//=====================================================================================================================

//! Region source with delegate to calculate.
class REGIONSOURCE_DELEGATED : public REGIONSOURCE {
	public:

		typedef fastdelegate::FastDelegate<ERRVALUE(REGION2D&)> DELEGATE_CALC;

		//! Construct with region pointer.
		REGIONSOURCE_DELEGATED (
			TREEVIEWNODE& parent,						//!< Parent node in TREEVIEW
			DELEGATE_CALC delegate,						//!< Delegate to calculate region
			const MISTRING& name,						//!< Name to show in tree view
			TREEVIEWNODE *pNodeInsertBefore = 0		//!< Node to insert before
			);

		//! Destructor.
		virtual ~REGIONSOURCE_DELEGATED ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DELEGATE_CALC m_delegate;

		virtual ERRVALUE v_CalcRegion (REGION2D& region);
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_REGIONSOURCE_H
