/**
 * \file mi32/rendermedia.h
 * \brief RENDERMEDIA definitions
 *
 * \if NODOC
 * $Id: rendermedia.h_v 1.6 2013/08/21 21:36:41 mju Exp $
 *
 * $Log: rendermedia.h_v $
 * Revision 1.6  2013/08/21 21:36:41  mju
 * Add setHeight/Width.
 *
 * Revision 1.5  2013/08/21 20:58:05  mju
 * Add methods to set individual values.
 *
 * Revision 1.4  2013/08/20 16:53:28  mju
 * Add inlines for getInteriorHeight/Width.
 *
 * Revision 1.3  2013/08/20 16:27:33  mju
 * Add changeOrient and simplify.
 *
 * Revision 1.2  2013/08/15 16:50:26  mju
 * Add iniRead/Write, hasSameDimensions, isEqual.
 *
 * Revision 1.1  2013/05/15 15:51:26  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_RENDERMEDIA_H
#define	INC_MI32_RENDERMEDIA_H

#ifndef	INC_MI32_MEDIASIZE_H
#include <mi32/mediasize.h>
#endif

#ifndef	INC_MI32_ORIENT8_H
#include <mi32/orient8.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

//=====================================================================================================================

//! Container for rendering media size, margins and resolution.
class RENDERMEDIA {
	public:

		//! Default constructor.
		RENDERMEDIA ();

		//! Compare for equality.
		bool operator== (
			const RENDERMEDIA& rhs
			) const { return (IsEqual(rhs)); }

		//! Compare for inequality.
		bool operator!= (
			const RENDERMEDIA& rhs
			) const { return (!IsEqual(rhs)); }

		//! Calculate pixel size in specified or stored units.
		DPOINT2D CalcPixelSize ( 
			SCREENUNIT units = SCREENUNIT_COUNT		//!< Units, default to use same as stored
			) const;

		//! Change orientation.
		void ChangeOrient (
			ORIENT8 orient									//!< Orientation change relative to current
			);

		//! Get bottom margin in stored units.
		double GetMarginBottom () const
			{ return (m_MarginBottom); }

		//! Get left margin in stored units.
		double GetMarginLeft () const
			{ return (m_MarginLeft); }

		//! Get right margin in stored units.
		double GetMarginRight () const
			{ return (m_MarginRight); }

		//! Get top margin in stored units.
		double GetMarginTop () const
			{ return (m_MarginTop); }

		//! Get total media size.
		const MEDIASIZE& GetSize () const
			{ return (m_Size); }

		//! Get height of interior area between margins in stored units.
		double GetInteriorHeight () const
			{ return (m_Size.GetHeight() - m_MarginTop - m_MarginBottom); }

		//! Get width of interior area between margins in stored units.
		double GetInteriorWidth () const
			{ return (m_Size.GetWidth() - m_MarginLeft - m_MarginRight); }

		//! Get resolution in pixels per stored unit.
		const DPOINT2D& GetResolution () const
			{ return (m_Resolution); }

		//! Get stored units.
		SCREENUNIT GetUnits () const
			{ return (m_Size.GetUnits()); }

		//! Determine if has same dimensions as other RENDERMEDIA instance.
		bool HasSameDimensions (
			const RENDERMEDIA& other
			) const;

		//! Read entry from .ini file.
		//! @return true if entry in file, false if not
		bool IniRead (
			INIHANDLE handle,								//!< Ini Handle created by IniOpen()
			const char *group,							//!< Group to look for ini entry, App Name if =0
			const char *field								//!< Field to read from
			);

		//! Write entry to .ini file.
		void IniWrite (
			INIHANDLE handle,								//!< Ini Handle created by IniOpen()
			const char *group,							//!< Group to write ini entry, App Name if =0
			const char *field								//!< Field to write entry to
			) const;

		//! Determine if equal.
		bool IsEqual (
			const RENDERMEDIA& rhs
			) const;

		//! Set all dimension values.
		ERRVALUE SetDimensions (
			const MEDIASIZE& Size,						//!< Media size
			double MarginTop,								//!< Top margin in media size units
			double MarginBottom,							//!< Bottom margin in media size units
			double MarginLeft,							//!< Left margin in media size units
			double MarginRight							//!< Right margin in media size units
			);

		//! Set media height.
		void SetHeight (
			double height
			) { m_Size.SetHeight(height); }

		//! Set media width.
		void SetWidth (
			double width
			) { m_Size.SetWidth(width); }

		//! Set bottom margin.
		void SetMarginBottom (
			double MarginBottom							//!< Bottom margin in media size units
			) { m_MarginBottom = MarginBottom; }

		//! Set left margin.
		void SetMarginLeft (
			double MarginLeft								//!< Left margin in media size units
			) { m_MarginLeft = MarginLeft; }

		//! Set right margin.
		void SetMarginRight (
			double MarginRight							//!< Right margin in media size units
			) { m_MarginRight = MarginRight; }

		//! Set top margin.
		void SetMarginTop (
			double MarginTop								//!< Top margin in media size units
			) { m_MarginTop = MarginTop; }

		//! Set resolution.
		void SetResolution (
			double resolution,							//!< Resolution in pixels/unit
			SCREENUNIT units = SCREENUNIT_COUNT		//!< Units, default to use same as stored
			);

		//! Set resolution.
		void SetResolution (
			const DPOINT2D& resolution,				//!< Resolution in pixels/unit
			SCREENUNIT units = SCREENUNIT_COUNT		//!< Units, default to use same as stored
			);

		//! Set media size.
		void SetSize (
			const MEDIASIZE& size
			) { m_Size = size; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MEDIASIZE m_Size;
		double m_MarginTop;
		double m_MarginBottom;
		double m_MarginLeft;
		double m_MarginRight;
		DPOINT2D m_Resolution;
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_RENDERMEDIA_H
