/**
 * \file sharedarray.h <mi32/sharedarray.h>
 * \brief Definitions for SHARED_ARRAY template
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef	INC_MI32_DOUBLEAR_H
#define	INC_MI32_DOUBLEAR_H

#ifndef	INC_MI32_MIDBLARY_H
#include <mi32/midblary.h>		//! For MIDOUBLEARRAY defn
#endif

#ifndef	INC_MI32_MEMBUF_H
#include <mi32/membuf.h>		//! For SwapBytes() defn
#endif

#ifndef	INC_MEMORY_H
#include <memory.h>				//! For memset/memcpy
#define	INC_MEMORY_H
#endif

//! Double array template.
//!
//! This template is designed to contain an array of items which can be allocated
//! using MmAlloc/MmRealloc.  The primary purpose of this template is to replace
//! allocated arrays where direct access to both array elements and the ability
//! to access a pointer to the first element (as for a C array) is required.
//!
//! For maximum efficiency, only minimal validation of access to array elements is performed.
//! No initialization of array items is performed by the template.
//!
//! This class is also designed to handle the 4 byte off alignment issue in regards to systems
//! that have a 4 byte allocation granularity.
//!
//! Best if used as SHARED_ARRAY<double>, SHARED_ARRAY<DPOINT2D>, SHARED_ARRAY<DPOINT3D>, 
//! SHARED_ARRAY<DRECT2D>, and SHARED_ARRAY<DRECT3D>

template <class _CT> class SHARED_ARRAY {

	public:

		//! Default constructor.
		SHARED_ARRAY (
			):
			m_data(0),
			m_numitems(0),
			m_spare(0)
			{ }

		//! Copy constructor
		SHARED_ARRAY (
			const SHARED_ARRAY<_CT>& rhs
			) :
			m_items(rhs.m_items),
			m_numitems(rhs.m_numitems),
			m_spare(0)
			{
			m_data = reinterpret_cast<_CT*>(m_items.GetPointer());
			}

		~SHARED_ARRAY (
			) {}

		//! Assignment.
		SHARED_ARRAY<_CT>& operator= (
			const SHARED_ARRAY<_CT>& rhs
			) {
			if (this != &rhs) {
				m_items = rhs.m_items;
				m_numitems = rhs.m_numitems;
				m_data = reinterpret_cast<_CT*>(m_items.GetPointer());
				}
			return (*this);
			}

		//! Equality operator.
		bool operator== (
			const SHARED_ARRAY<_CT>& rhs
			) const {
			if (m_numitems != rhs.m_numitems) return (false);
			if (m_numitems == 0) return (true);
			return (memcmp(m_data,m_data,m_numitems*sizeof(_CT)) == 0);
			}

		//! Inequality operator.
		bool operator!= (
			const SHARED_ARRAY<_CT>& rhs
			) const {
			return (!operator==(rhs));
			}

		//! Cast to const array of template type.
		operator const _CT*(
			) const {
			return (m_data);
			}

		//! Cast to array of template type.
		operator _CT*(
			) {
			return (m_data);
			}

		//! Read (const) access to array element.
		//!
		//! @note Does not verify that index is within valid range.
		template <typename _IDXTYPE> const _CT& operator[] (
			_IDXTYPE index
			) const {
			return (m_data[index]);
			}

		//! Write (non-const) access to array element.
		//!
		//! @note Does not verify that index is within valid range.
		template <typename _IDXTYPE> _CT& operator[] (
			_IDXTYPE index
			) {
			return (m_data[index]);
			}

		//! Append item to end of array, extending if necessary.
		ERRVALUE Append (
			const _CT& item
			) {
			if (m_numitems >= GetMaxItems()) {
				int err;
				if ((err = m_items.Reserve(MAX(16,(2*m_numitems*sizeof(_CT) + sizeof(double) - 1) / sizeof(double)), true)) < 0) return (err);
				m_data = reinterpret_cast<_CT*>(m_items.GetPointer());
				}
			m_data[m_numitems++] = item;
			return (0);
			}
			
		//! Assign from C array, reserving space if needed.
		//! Resizes the array to the specified number of items.
		ERRVALUE Assign (
			const _CT *items,						//!< Items to copy to array
			int numitems							//!< Number of items to copy
			) {
			ERRVALUE err;
			if ((err = Reserve(numitems,true)) < 0) return (err);
			memcpy(m_data,items,numitems*sizeof(_CT));
			m_numitems = numitems;
			return (0);
			}

		//! Attach a buffer to the SHARED_ARRAY.
		//! After passing a buffer to Attach(), the SHARED_ARRAY "<b>owns</b>" the
		//! buffer and will free it in its destructor.  
		//! @param items Reference to a pointer to the templated data type.
		//! @param numitems The number of items being attached
		//!
		//! The reason it's a reference is that after recording the pointer,
		//! Attach() will set <i>your</i> pointer to 0.  This will prevent you
		//! from accidently freeing a pointer you no longer own.  If you want
		//! it back, call the Detach() method
		void Attach (
			MIDOUBLEARRAY& items,
			int numitems
			) {
			m_items.Attach(items);
			m_numitems = numitems;
			m_data = reinterpret_cast<_CT*>(m_items.GetPointer());
			return;
			}
			
		//! Clear the array, does not free resources.
		void Clear (
			) { m_numitems = 0; }

		//! Clear items in the array
		void ClearItems (
			int start = 0,
			int end = -1						//!< Default -1, use number of items
			) {
			if (end < start) end = m_numitems - 1;
			memset(m_data + start, 0, (end - start + 1) * sizeof(_CT));
			return;
			}

		//! Copy the contents of another SHARED_ARRAY.
		//! Same as assignment, but returns an error code instead of throwing an exception.
		ERRVALUE Copy (
			const SHARED_ARRAY<_CT>& rhs
			) {
			if (this != &rhs) {
				return (SetItems(0, rhs, rhs.m_numitems));
				}
			return (0);
			}

		//! Delete item in the array.
		//! Use with care as this is not efficient for large arrays.
		void DeleteItem (
			int index
			) {
			if (index >= 0 && index < m_numitems) {
				--m_numitems;
				if (index < m_numitems) {
					memmove(m_data+index,m_data+index+1,(m_numitems-index)*sizeof(_CT));
					}
				}
			return;
			}
			
		//! Detach the buffer from the SHARED_ARRAY.
		//!
		//! This turns ownership of the buffer over to the caller, who is then
		//! responsibile for seeing that it gets disposed of.
		//! After calling Detach(), the SHARED_ARRAY will behave as if it had
		//! just been constructed.  In other words, it will be a NULL pointer
		//! pointing to 0 items.
		//! This means that if you want to know how many items are in the
		//! array returned, you'd better find out @{before} you call Detach()!
		void Detach (
			MIDOUBLEARRAY& items
			) {
			items.Attach(m_items);
			m_numitems = 0;
			m_data = 0;
			return;
			}
			
		// See above comment on Attach() for Detach() method bad idea.

		//! Free the array.
		void Free (
			) {
			m_items.Free();
			m_numitems = 0;
			m_data = 0;
			return;
			}

		//! Get items from array.
		void GetItems (
			_CT *items,								//!< Buffer to hold items
			int numitems,							//!< Number of items to copy
			int firstitem = 0						//!< First array item to retrieve
			) const {
			memcpy(items,m_data+firstitem,MIN(m_numitems-firstitem, numitems)*sizeof(_CT));
			return;
			}

		//! Get maximum number of items in array.
		int GetMaxItems (
			) const {
			return (m_items.GetNumReserved() * sizeof(double) / sizeof(_CT));
			}

		//! Get number of items in array.
		int GetNumItems (
			) const { return (m_numitems); }

		//! Get size of array required to hold current number of items in bytes.
		int GetSizeInBytes (
			) const { return (m_numitems*sizeof(_CT)); }

		//! Determine if array is empty.
		bool IsEmpty (
			) const { return (m_numitems == 0); }

		//! Remove item at specified index, maintaining order.
		//! Note that this is inefficient for large arrays.
		void RemoveItem (
			INT32 item
			) {
			if (item < m_numitems && item >= 0) {
				--m_numitems;
				if (item < m_numitems) { memmove(&m_data[item], &m_data[item+1], (m_numitems - item) * sizeof(_CT)); }
				}
			return;
			}

		//! Reserve space in array for up to specified number of items.
		ERRVALUE Reserve (
			int newmaxitems,						//!< New maximum number of items
			bool clear = false					//!< Clear the array by resetting number of items to 0
			) {
			if (clear) m_numitems = 0;
			if (newmaxitems > GetMaxItems()) {
				ERRVALUE err;
				if ((err = m_items.Reserve((newmaxitems * sizeof(_CT) + sizeof(double) - 1) / sizeof(double), true)) < 0) return (err);
				m_data = reinterpret_cast<_CT*>(m_items.GetPointer());
				}
			return (0);
			}

		//! Reserve space in array for up to specified number of items, throw exception on failure.
		void ReserveExc (
			int newmaxitems,						//!< New maximum number of items
			bool clear = false					//!< Clear the array by resetting number of items to 0
			) {
			if (clear) m_numitems = 0;
			if (newmaxitems > GetMaxItems()) {
				m_items.ReserveExc((newmaxitems * sizeof(_CT) + sizeof(double) - 1) / sizeof(double), true);
				m_data = reinterpret_cast<_CT*>(m_items.GetPointer());
				}
			return;
			}

		//! Resize array to specified number of items.
		//! If the new size is larger than the previous size and the array is
		//! not cleared the new array entries will have indeterminant values.
		ERRVALUE Resize (
			int numitems,							//!< New number of items desired
			bool keepold = true,					//!< Keep old array entries
			bool clear = false					//!< Clear new entries to zeros, will clear all if keepold is false
			) {
			int err;
			if ((err = Reserve(numitems,!keepold)) < 0) return (err);
			if (clear && numitems > m_numitems) memset(m_data+m_numitems,0,(numitems-m_numitems)*sizeof(_CT));
			m_numitems = numitems;
			return (0);
			}

		//! Resize array to specified number of items, throw exception on failure.
		//! If the new size is larger than the previous size and the array is
		//! not cleared the new array entries will have indeterminant values.
		void ResizeExc (
			int numitems,							//!< New number of items desired
			bool keepold = true,					//!< Keep old array entries
			bool clear = false					//!< Clear new entries to zeros, will clear all if keepold is false
			) {
			ReserveExc(numitems,!keepold);
			if (clear && numitems > m_numitems) memset(m_data+m_numitems,0,(numitems-m_numitems)*sizeof(_CT));
			m_numitems = numitems;
			return;
			}

		//! Reverse order of items in array.
		void Reverse (
			) {
			for (int i = 0; (i < m_numitems/2); ++i) {
				_CT temp(m_data[i]);
				m_data[i] = m_data[m_numitems-i-1];
				m_data[m_numitems-i-1] = temp;
				}
			return;
			}

		//! Set array items from C array, expanding if needed.
		//! Will not reduce the number of items in the array.
		ERRVALUE SetItems (
			int firstitem,							//!< First array item to replace
			const _CT *items,						//!< Items to copy to array
			int numitems							//!< Number of items to copy
			) {
			ERRVALUE err;
			if ((err = Reserve(firstitem+numitems,true)) < 0) return (err);
			memcpy(m_data+firstitem,items,numitems*sizeof(_CT));
			if (m_numitems < firstitem + numitems) m_numitems = firstitem + numitems;
			return (0);
			}

		//! Swap two SHARED_ARRAYs.
		void Swap (
			SHARED_ARRAY<_CT>& rhs
			) {
			m_items.Swap(rhs.m_items);
			_CT *t_data = m_data;
			m_data = rhs.m_data;
			rhs.m_data = t_data;
			int t_numitems = m_numitems;
			m_numitems = rhs.m_numitems;
			rhs.m_numitems = t_numitems;
			return;
			}
			
		//! Swap bytes in a SHARED_ARRAY, assume that there is an overload to swap a _CT
		void SwapBytes (
			) { for (int i = 0;(i < m_numitems);++i) ::SwapBytes(m_data[i]); }
			
		//! Transfer data ownership from 'rhs' to 'this'.
		void TransferOwnerFrom (
			SHARED_ARRAY<_CT>& rhs
			) {
			MIDOUBLEARRAY temp;
			int numitems = rhs.GetNumItems();
			rhs.Detach(temp);
			Attach(temp, numitems);
			return;
			}
			
		//! Transfer data ownership from 'rhs' to 'this'.
		template <class _T2> void TransferOwnerFromExt (
			SHARED_ARRAY<_T2>& rhs
			) {
			MIDOUBLEARRAY temp;
			UINT32 numbytes = rhs.GetNumItems() * sizeof(_T2);
			rhs.Detach(temp);
			int numitems = static_cast<int>(numbytes / sizeof(_CT));
			Attach(temp, numitems);
			return;
			}
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MIDOUBLEARRAY m_items;					//!< The accessed array
		_CT* m_data;								//!< Pointer to the data for fast access
		int m_numitems;							//!< Number of valid items
		int m_spare;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif	//!< INC_MI32_DOUBLEAR_H
