/**
 * \file mi32/treeview.h
 * \brief TREEVIEW definitions
 *
 * \if NODOC
 * $Id: treeview.h_v 1.5 2013/10/24 16:00:50 mju Exp $
 *
 * $Log: treeview.h_v $
 * Revision 1.5  2013/10/24 16:00:50  mju
 * Add control to node to set whether treeview notification enabled.
 *
 * Revision 1.4  2013/10/23 21:38:42  mju
 * Eliminate treeviewfolder.
 *
 * Revision 1.3  2013/10/22 20:34:18  mju
 * Use node type for insertAfter.
 *
 * Revision 1.2  2013/10/21 19:57:19  mju
 * Don't nest node/folder/object.
 *
 * Revision 1.1  2013/10/18 21:26:47  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_TREEVIEW_H
#define	INC_MI32_TREEVIEW_H

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class TREEVIEWOBJECT;
class TREEVIEW;
#endif

//=====================================================================================================================

//! Base for node in treeview.
//! Can contain other nodes.
class TREEVIEWNODE {
	public:

		enum TYPE {
			TYPE_Root,					//!< Root node
			TYPE_Node,					//!< Descendent node (not derived from TREEVIEWOBJECT)
			TYPE_Object					//!< Object (derived from TREEVIEWOBJECT)
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Internal constructor for root node.
		TREEVIEWNODE (TREEVIEW& treeview, const MISTRING& name);
		#endif

		//! Construct as TYPE_Node with specified parent and name.
		TREEVIEWNODE (
			TREEVIEWNODE& parent,						//!< Parent node
			const MISTRING& name,						//!< Node name
			TREEVIEWNODE *pNodeInsertBefore = 0		//!< Node to insert before, NULL to place at end of list
			);

		//! Destructor.
		virtual ~TREEVIEWNODE ();

		//! Attach or change position in hierarchy.
		//! If notification is enabled, will notify of Add or ChangePosition depending on wheter was already attached.
		void Attach (
			TREEVIEWNODE& parent,					//!< Parent node
			TREEVIEWNODE *pNodeInsertBefore		//!< Node to insert before, NULL to place at end of list
			);

		//! Find object with specified ID.
		const TREEVIEWOBJECT* FindObject (UINT32 UniqueID) const;

		//! Get first child.
		TREEVIEWNODE* GetFirstChild () const
			{ return (m_pFirstChild); }

		//! Get parent node.
		TREEVIEWNODE* GetParent () const;

		//! Get last child.
		TREEVIEWNODE* GetLastChild () const
			{ return (m_pLastChild); }

		//! Get name for display.
		const MISTRING& GetName () const
			{ return (m_name); }

		//! Get next node in list.
		TREEVIEWNODE* GetNext () const
			{ return (m_pNext); }

		//! Get previous node in list.
		TREEVIEWNODE* GetPrevious () const
			{ return (m_pPrevious); }

		//! Get root node.
		const TREEVIEWNODE* GetRoot () const;

		//! Get TREEVIEW if in hierarchy.
		const TREEVIEW* GetTreeView (
			bool bCheckEnableNotify = true		//!< Whether to check for enabling notificiation
			) const;

		//! Get node type.
		TYPE GetType () const
			{ return (m_type); }

		//! Determine if treeview notification is enabled for node and all ancestors.
		bool IsEnableNotify () const;

		//! Determine if is root node.
		bool IsRoot () const
			{ return (m_type == TYPE_Root); }

		//! Set node name.
		void SetName (
			const MISTRING& name
			);

	protected:

		//! Construct as specified type.
		//! Not attached to hierarchy so derived constructor can complete before sending notification.
		explicit TREEVIEWNODE (TYPE type, const MISTRING& name);

		//! Get whether treeview notification is enabled for node.
		bool GetEnableNotify () const
			{ return (m_bEnableNotify); }

		//! Set whether treeview notification is enabled for node.
		void SetEnableNotify (
			bool bEnableNotify
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		void *m_pParent;
		TREEVIEWNODE *m_pPrevious;
		TREEVIEWNODE *m_pNext;
		TREEVIEWNODE *m_pFirstChild;
		TREEVIEWNODE *m_pLastChild;
		MISTRING m_name;
		TYPE m_type;
		bool m_bEnableNotify;

		//! Detach from hierarchy.
		void Detach (
			bool bNotify = true						//!< Whether to notify treeview
			);

		TREEVIEWNODE (const TREEVIEWNODE&);
		TREEVIEWNODE& operator= (const TREEVIEWNODE&);
		#endif
	};

//=====================================================================================================================

//! Base class for "object" in tree view.
class TREEVIEWOBJECT : public TREEVIEWNODE {
	public:

		//! Get treeview unique ID for object.
		UINT32 GetUniqueID () const
			{ return (m_UniqueID); }

		//! Determine if object has changed and needs update.
		bool IsChanged () const
			{ return (m_bChanged); }

		//! Determine if object is valid.
		bool IsValid () const
			{ return (m_bValid); }

		//! Notify as having changed and whether still valid.
		void NotifyChanged (
			bool bValid				//!< Whether object is valid
			);

	protected:

		//! Constructor.
		explicit TREEVIEWOBJECT (
			TREEVIEWNODE& node,							//!< Parent node
			const MISTRING& name,
			TREEVIEWNODE *pNodeInsertBefore = 0		//!< Node to insert before, NULL to insert at end of list
			);

		//! Destructor.
		virtual ~TREEVIEWOBJECT () = 0;

		//! Reset "changed" status.
		void ResetChanged ()
			{ m_bChanged = false; }

		//! Set object as valid without setting as changed.
		//! Generally for use by derived class constructor where object is initially valid.
		void SetAsValid ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_UniqueID;
		mutable bool m_bValid;
		mutable bool m_bChanged;
		#endif

		//! Called by NotifyChanged prior to notifying TreeView.
		//! Default implemention does nothing.
		virtual void v_NotifyChanged (bool bValid);
	};

//=====================================================================================================================

//! Base class for "tree view" of objects.
class TREEVIEW {
	public:

		//! Base interface class to receive notification of changes via treeview.
		class OBSERVER {
			public:

				//! Constructor.
				OBSERVER (TREEVIEW& treeview);

				//! Destructor.
				virtual ~OBSERVER ();

				//! Called when node added.
				//! Default implementation does nothing.
				virtual void v_OnNodeAdd (TREEVIEWNODE *pNode);

				//! Called when node name is changed.
				//! Default implementation does nothing.
				virtual void v_OnNodeChangeName (TREEVIEWNODE *pNode);

				//! Called when node position in hierarchy is changed.
				//! Default implementation does nothing.
				virtual void v_OnNodeChangePosition (TREEVIEWNODE *pNode);

				//! Called when node is removed.
				//! Default implementation does nothing.
				virtual void v_OnNodeRemove (TREEVIEWNODE *pNode);

				//! Called when existing object is changed (including if becomes invalid).
				//! Default implementation does nothing.
				virtual void v_OnObjectChange (TREEVIEWOBJECT *pObject);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				TREEVIEW *m_pTreeView;
				friend class TREEVIEW;
				#endif
			};

		//! Access node at root of hierarchy.
		TREEVIEWNODE& AccessRootNode ()
			{ return (m_NodeRoot); }

		//! Get node at root of hierarchy.
		const TREEVIEWNODE& GetRootNode () const
			{ return (m_NodeRoot); }

	protected:

		explicit TREEVIEW (const MISTRING& rootname);

		~TREEVIEW ();

		UINT32 GetLastUniqueID () const
			{ return (m_LastUniqueID); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef MILIST<OBSERVER*> OBSERVERLIST;

		OBSERVERLIST m_ObserverList;
		TREEVIEWNODE m_NodeRoot;
		mutable UINT32 m_LastUniqueID;

		UINT32 GetNextUniqueID () const
			{ return (++m_LastUniqueID); }

		void NotifyNodeAdd (TREEVIEWNODE *pNode) const;
		void NotifyNodeChangeName (TREEVIEWNODE *pNode) const;
		void NotifyNodeChangePosition (TREEVIEWNODE *pNode) const;
		void NotifyNodeRemove (TREEVIEWNODE *pNode) const;
		void NotifyObjectChange (TREEVIEWOBJECT *pObject) const;

		TREEVIEW (const TREEVIEW&);
		TREEVIEW& operator= (const TREEVIEW&);

		friend class TREEVIEWNODE;
		friend class TREEVIEWOBJECT;
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_TREEVIEW_H
