/**
 * \file rvc/dbase.h 
 * \brief interface for the RVC::DBASE class.
 *
 * \if NODOC
 * $Id: dbase.h_v 1.65 2015/10/27 17:35:59 mju Exp $
 *
 * $Log: dbase.h_v $
 * Revision 1.65  2015/10/27 17:35:59  mju
 * Need copyflag_FreezeComputed after all.
 *
 * Revision 1.64  2015/10/27 17:11:30  mju
 * Undo previous change.
 *
 * Revision 1.63  2015/10/27 17:03:11  mju
 * Add copyflag_FreezeComputedFields.
 *
 * Revision 1.62  2015/05/15 15:31:20  mju
 * Change some tableCopy methods to use elemnumtrans.
 *
 * Revision 1.61  2015/05/13 21:57:15  mju
 * Add static checkLikeFields.
 *
 * Revision 1.60  2015/05/12 21:43:20  mju
 * Use dbtabletrans in various copy/join methods.
 *
 * Revision 1.59  2015/05/08 19:09:15  mju
 * Use elemnumtrans.
 *
 * Revision 1.58  2015/04/28 17:47:54  mju
 * Add elemtranslist.addElement with elemnum parms.
 *
 * Revision 1.57  2015/04/27 15:16:35  bcolombe
 * Filter - return label dbase
 *
 * Revision 1.56  2014/04/10 21:17:06  mju
 * Add copyflag_DemoteElementIDTable and IgnoreStyleTables.
 *
 * Revision 1.55  2014/03/11 21:57:38  mju
 * JOINFLAG_AttachDest flags and mask were not corresponding.
 *
 * Revision 1.54  2013/11/22 22:32:53  mju
 * Add getRelationshipDistance using table numbers.
 *
 * Revision 1.50  2010/11/08 17:40:19  bcolombe
 * Add COPYFLAG_CopyElementIDTable
 *
 * Revision 1.47  2008/10/02 22:00:51  scowan
 * Added assignForeignKeyRelationships method.
 *
 * Revision 1.45  2007/09/10 14:09:23  dwilliss
 * Added another CopyTo method for copying from a plain database to a
 * dbase_element given a source table and translation list
 *
 * Revision 1.44  2007/09/06 16:53:30  dwilliss
 * Added simple copyDatabaseTo
 *
 * Revision 1.25  2005/04/06 15:32:51  scowan
 * Added isTableValid methods.
 *
 * Revision 1.17  2003/12/03 21:28:03  scowan
 * Added methods to compare structures of database tables for a match.
 *
 * Revision 1.9  2003/09/02 17:29:43  dwilliss
 * Added COPYTABLE_FreezeComputedFields flag
 *
 * Revision 1.1  2001/05/23 20:36:09  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_DBASE_H
#define INC_RVC_DBASE_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif	

#ifndef INC_RVC_DBFIELDTYPE_H
	#include <rvc/dbfieldtype.h>
#endif

#ifndef INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef INC_MI32_RVCDBASE_H
	#include <mi32/rvcdbase.h>
#endif

#ifndef INC_RVC_GETOBJFILTER_H
	#include <rvc/getobjfilter.h>
#endif

#ifndef INC_MI32_UNSIGNEDSET_H
	#include <mi32/unsignedset.h>
#endif

#ifndef INC_MI32_ELEMNUMTRANS_H
	#include <mi32/elemnumtrans.h>
#endif

#ifndef INC_MI32_UNITCONV_H
	#include <mi32/unitconv.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class ELEMNUMTRANS;
namespace SML {
	class CONTEXT;
	}
namespace RVC {
	struct DBCALLBACKSTRUCT;
	class DBASE_ELEMENT;
	class DBTABLE;
	class DBTABLE_STYLE;
	class DATABASEHANDLE;
	class DBTABLETRANS;
	class DBTRANSINFO;
	}
#endif

namespace RVC {

//! Table translation list.
typedef MILIST<DBTABLETRANS> DBTABLETRANSLIST;

//=====================================================================================================================

//! Manages RVC Databse objects.
class DBASE : public RVC::OBJECT {
	public:

		enum COPYFLAG {
			COPYFLAG_None = 						0x00000000,
			COPYFLAG_TableLinks = 				0x00000004,		//!< Copy table links instead of the tables themselves
			COPYFLAG_DemoteIdentityTables = 	0x00000008,		//!< Demote identity tables because they can no longer exist
			COPYFLAG_MinimizeTableSize = 		0x00000020,		//!< Minimize table sizes (convert Identity and OneToOne to OneRecordPerElement)
			COPYFLAG_DontRemoveRecords = 		0x00000200,		//!< Do not remove records while copying
			COPYFLAG_RemoveStdAttribTables = 0x00000800,		//!< Remove standard attribute tables
			COPYFLAG_IgnoreStdAttribTables = 0x00040000,		//!< Ignore standard attribute tables
			COPYFLAG_FreezeComputedFields =	0x00080000,		//!< Freeze computed fields when copy table
			COPYFLAG_CopyThemeTable = 			0x01000000,		//!< Copy theme tables, only if not joining tables
			COPYFLAG_CopyElementIDTable = 	0x02000000,		//!< Copy element id table
			COPYFLAG_DemoteElementIDTable = 	0x04000000,		//!< Change ElementID table into user table
			COPYFLAG_IgnoreStyleTables =		0x08000000,		//!< Don't copy style assignment tables
			};
		
		enum COPYTABLE {
			COPYTABLE_None =		 				0x00000000,
			COPYTABLE_CopyLinks = 				0x00000004,
			COPYTABLE_RemoveStdAttTables = 	0x00000800,
			COPYTABLE_FreezeComputedFields =	0x00080000
			};
			
		//! Search flags for GetAttachedRecordList()
		enum FINDFLAG {
			FINDFLAG_AllAttached = 0,
			FINDFLAG_NoInternal = 1,
			FINDFLAG_OnlyDirect = 2,
			FINDFLAG_FirstMatch = 8
			};

		enum JOINFLAGS {
			JOINFLAG_None = 									0x00000000,
			JOINFLAG_CopyThemeTable = 						0x01000000,		//!< Copy theme tables, only if not joining tables
			JOINFLAG_OuterRight = 							0x02000000,		//!< Right Outer (for Join)
			JOINFLAG_Union = 									0x04000000,		//!< Union (DBTABLE::Combine only)
			JOINFLAG_Natural = 								0x08000000,		//!< Natural Join (DBTABLE::Combine only)
			JOINFLAG_SameStrc = 								0x10000000,		//!< Only table structure has to match
			JOINFLAG_SameNmSt = 								0x30000000,		//!< Table structure and name has to match
			JOINFLAG_RemoveDups =	 						0x40000000,		//!< Remove duplicate records when joining
			JOINFLAG_Outer = 									0x80000000,		//!< Outer Union or Left Outer Join
			JOINFLAG_AttachDestIfEitherSrcAttached =	0x00000000,
			JOINFLAG_AttachDestIfSrc2Attached =			0x00400000,
			JOINFLAG_AttachDestIfSrc1Attached =			0x00800000,
			JOINFLAG_AttachDestIfBothSrcAttached =		0x00C00000,
			JOINFLAG_NoAttachDest =							0x00020000,
			JOINFLAG_AttachDestMASK =						0x00C00000,
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class BULKMODIFY;
		class GETOBJFILTER_ISCAPABLE;
		class GETOBJFILTER_ISSELECTABLE;
		class INVERSEDBLISTSUSPEND;
		class OBSERVER;
		class TABLEITERATOR;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Constant iterator for tables.
		//! Use with TableBegin and TableEnd to iterate through tables in database.
		class CONST_TABLEITERATOR {
			public:
				//! Default constructor
				CONST_TABLEITERATOR (
					) : 
					m_TableNum(TABLE_InternalNode),
					m_CurrentObj(0)
					{}

				//! Internal constructor used by TableEnd() methods
				CONST_TABLEITERATOR (
					const DBASE* dbase,
					INT32 TableNum
					) : 
					m_TableNum(TableNum),
					m_CurrentObj(dbase)
					{}

				//! Copy constructor
				CONST_TABLEITERATOR (
					const TABLEITERATOR &rhs
					) : 
					m_TableNum(rhs.m_TableNum),
					m_CurrentObj(rhs.m_CurrentObj)
					{}

				//! Interanl constructor used by TableBegin() methods
				CONST_TABLEITERATOR (
					const DBASE* dbase
					);

				//! Destructor
				virtual ~CONST_TABLEITERATOR (
					) {}

				//! Assignment
				CONST_TABLEITERATOR& operator= (
					const CONST_TABLEITERATOR &rhs
					) {
					if (this != &rhs) {
						m_TableNum = rhs.m_TableNum;
						m_CurrentObj = rhs.m_CurrentObj;
						}
					return (*this);
					}

				//! Dereference
				const INT32& operator* (
					) const { return (m_TableNum); }

				//! Dereference arrow
				const INT32* operator-> (
					) const { return (&**this); }

				//! Pre-increment
				CONST_TABLEITERATOR& operator++ (
					);

				//! Comparison for equality
				bool operator== (
					const CONST_TABLEITERATOR &rhs
					) const { return (m_TableNum == rhs.m_TableNum && m_CurrentObj == rhs.m_CurrentObj); }

				//! Comparison for inequality
				bool operator!= (
					const CONST_TABLEITERATOR &rhs
					) const { return (!(*this == rhs)); }

			protected:
				INT32 m_TableNum;
				const DBASE *m_CurrentObj;
			};

		//! Iterator for tables.
		//! Use with TableBegin and TableEnd to iterate through tables in database.
		class TABLEITERATOR : public CONST_TABLEITERATOR {
			public:

				//! Default constructor
				TABLEITERATOR (
					) : CONST_TABLEITERATOR() {}

				//! Copy constructor
				TABLEITERATOR (
					CONST_TABLEITERATOR &rhs
					) : CONST_TABLEITERATOR(rhs) {}

				//! Internal constructor used by TableBegin() methods
				TABLEITERATOR (
					const DBASE* dbase
					) : CONST_TABLEITERATOR(dbase) {}

				//! Internal constructor used by TableEnd() methods
				TABLEITERATOR (
					const DBASE* dbase,
					INT32 TableNum
					) : CONST_TABLEITERATOR(dbase, TableNum) {}

				//! Dereference
				const INT32& operator* (
					) const { return (m_TableNum); }

				//! Dereference arrow
				const INT32* operator-> (
					) const { return (&**this); }

				//! Pre-increment
				TABLEITERATOR& operator++ (
					) {
					CONST_TABLEITERATOR::operator++();
					return (*this);
					}

				//! Comparison for equality
				bool operator== (
					const TABLEITERATOR &rhs
					) const { return (m_TableNum == rhs.m_TableNum && m_CurrentObj == rhs.m_CurrentObj); }

				//! Comparison for inequality
				bool operator!= (
					const TABLEITERATOR &obj
					) const { return (!(*this == obj)); }
			};

		//! Default constructor
		DBASE ();

		//! Copy constructor
		DBASE (								//! Copy Constructor
			const DBASE &rhs
			);

		//! Destructor
		virtual ~DBASE ();

		//! Assignment
		DBASE& operator= (
			const DBASE &rhs
			);
			
		//! Checks to see if a table could be created. 
		//! \return 0 if table creation is possible, error code if not.
		//! Allows to report error BEFORE the user designs table.
		ERRVALUE CanCreateTables () const;

		//! Copy tables from this database to another database, keeping the relationships.
		ERRVALUE CopyDatabaseTo (
			DBASE &ObjDbTgt,									//!< Target database
			DBTABLETRANSLIST *pTableTransList = 0,		//!< Table translation list, if not specified all tables will be copied
			COPYFLAG CopyFlags = COPYFLAG_None
			);

		//! Copy tables from this database to another database, keeping the relationships.
		ERRVALUE CopyDatabaseTo (
			DBASE_ELEMENT &ObjDbTgt,						//!< Target database
			const DBTABLE &SourceTable,				//!< Source table associated with elements
			const ELEMNUMTRANS &SrcRecToTgtElem,		//!< Mapping from source record number to target element number
			ELEMTYPE ElemType = ELEMTYPE_Empty,			//!< Used for databases associated with multiple element types
			DBTABLETRANSLIST *pTableTransList = 0,		//!< Table translation list, if not specified all tables will be copied
			COPYFLAG CopyFlags = COPYFLAG_None
			);

		//! Copy a table from the source database to the destination database
		//! @return Table number of copied table or error if < 0	
		INT32 CopyTableTo (
			INT32 SrcTableNum,
			DBASE &ObjDbTgt,
			COPYTABLE flags = COPYTABLE_None
			) const;

		//! Copy a table from the source database to the destination database
		//! @return Table number of copied table or error if < 0	
		INT32 CopyTableTo (
			INT32 SrcTableNum,
			DBASE &ObjDbTgt,
			DESCRIPTOR &DestTableDescriptor,
			SIMPLE_ARRAY<INT32> *pRecordTrans = 0,	//!< SrcToDest record translation table RETURNED if not NULL
			COPYTABLE flags = COPYTABLE_None
			) const;

		//! Create an SML context for the given database class instance
		ERRVALUE CreateSMLContext (
			SML::CONTEXT **pcontext					//!< SML context allocated and RETURNED
			);

		//! Extract the tables that are related to the 'BaseTable' to the specified LINKTYPE.
		//! if GEOMETRIC::HasCapability(GEOMETRIC::CAPABILITY_MultipleTableSupport) == false, this routine will return.
		//! If the 'BaseTable' has a primary key field, the 'BaseTable' will be extracted to a new table.
		ERRVALUE ExtractTablesFrom (
			const DBASE &SrcDbObj,						//!< The source database that contains the tables related to 'BaseTable'
			const DESCRIPTOR &BaseTable,				//!< Name of the base table for export
			const MISTRING &ConnectString,			//!< Connection string for the type of database to extract to, empty string refers to internal
			bool IsGeometricExtract = false			//!< This is an extraction where the base table can be incoporated with the spatial component
			);
			
		//! Compare table structures between two databases to determine which tables structurally
		//! match.  Returns the set of exact matching tables In the TableMatch array if the entry != -1
		ERRVALUE FindTableMatches (
			const DBASE &rhs,
			SIMPLE_ARRAY<INT32> &TableMatch		//!< Translation from lhs table number to rhs table number, -1 if no other table matches
			) const;

		//! Flush cached records and build indexes if necessary
		ERRVALUE Flush ();

		//! Get records attached to the specified element for all tables
		ERRVALUE GetAttachedRecordList (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			SIMPLE_ARRAY<DBLISTELMT> &dblist,	//!< List of attachments allocated and returned
			FINDFLAG FindFlag = FINDFLAG_AllAttached,
			INT32 BaseTableNum = -1
			) const;
			
		//! Get list of field names that are key fields
		//! @return Number of key field names
		ERRVALUE GetKeyFieldNames (
			SIMPLE_ARRAY<OBJECTNAME> &FieldNames
			) const;

		//! Determine the number of tables in the database
		//! @return Number of tables in the database
		INT32 GetNumTables (
			) const;
			
		//! Get correct OBJTYPESET for all database OBJTYPE's
		//! @return OBJTYPESET for all database OBJTYPE's
		static OBJTYPESET GetObjTypeSet (
			);

		// Returns the "distance" between related tables in # of hops it takes to get from one to the other.
		INT32 GetRelationshipDistance (
			const DBTABLE &SrcTable,
			const DBTABLE &DestTable
			);
			
		// Returns the "distance" between related tables in # of hops it takes to get from one to the other.
		INT32 GetRelationshipDistance (
			INT32 SrcTableNum,
			INT32 DestTableNum
			);
			
		//! Build an inverse DB list for all DBlist entries
		ERRVALUE InverseDBListBuild ();

		//! Resume maintainence of the inverse DB list from a InverseDBListSuspend() call
		void InverseDBListResume ();

		//! Suspend maintainence of the inverse DB lists.  Used if doing multiple database changes
		void InverseDBListSuspend ();

		//! Determine if the table number refers to an internal (built-in) table
		//! @return 'True' if table number refers to an internal table, 'false' if not
		bool IsTableInternal (
			INT32 TableNum
			) const;

		//! Determine if the table number refers to a valid table
		//! @return 'True' if table number refers to a valid table, 'false' if not
		bool IsTableValid (
			INT32 TableNum
			) const;

		//! Determine if the table name refers to a valid table
		//! @return 'True' if table name refers to a valid table, 'false' if not
		bool IsTableValid (
			const OBJECTNAME &name
			) const;

		//! Make a new database object
		ERRVALUE Make (
	    	const OBJECT &parent,				//!< Parent object to create database under
			const DESCRIPTOR &Descriptor,	//!< Name of database object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS	//!< User credentials passed in, NULL for default behavior
			);

		//! Make a new database object
		ERRVALUE Make (
	    	OBJITEM &ObjItem,					//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,			//!< Source of new style object, default is application name
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,			//!< Source of new style object, default is application name
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Remove duplicate records from all tables (except for ATTACHMENT_Identity and ATTACHMENT_OneToOne) tables
		ERRVALUE RemoveDuplicateRecords ();
			
		//! Remove unattached records from all tables
		ERRVALUE RemoveUnattachedRecords ();
			
		//! Get a beginning constant table iterator 
		CONST_TABLEITERATOR TableBegin (
			) const {
			m_NumTables = (IsOpen()) ? GetNumTables() : 0;
			return (CONST_TABLEITERATOR(this));
			}

		//! Get a beginning table iterator 
		TABLEITERATOR TableBegin (
			) {
			m_NumTables = (IsOpen()) ? GetNumTables() : 0;
			return (TABLEITERATOR(this));
			}

		//! Get a ending constant table iterator 
		CONST_TABLEITERATOR TableEnd (
			) const { return (CONST_TABLEITERATOR(this, m_NumTables)); }

		//! Get a ending table iterator 
		TABLEITERATOR TableEnd (
			) { return (TABLEITERATOR(this, m_NumTables)); }
			
		//! Validate the database
		ERRVALUE Validate (
			MISTRING &LogString
			);

		//! Validate new table and field names against existing tables and fields
		ERRVALUE ValidateTableName (
			OBJECTNAME &name,						//!< Table name to validate, can be changed
			SIMPLE_ARRAY<OBJECTNAME> &finfo	//!< Field names to validate, can be changed
			) const;

	protected:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 AppendDBList (
			ELEMTYPE ElemType, 
			INT32 ElemNum,
			const SIMPLE_ARRAY<DBLISTELMT> &DBList
			);

		INT32 AppendDBList (
			ELEMTYPE ElemType, 
			INT32 ElemNum,
			const DBLISTELMT &item
			);

		ERRVALUE CleanDBList (
			ELEMTYPE ElemType
			);

		INT32 GetDBListItemSize (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			bool ExcludeInternal = false
			) const;

		INT32 GetDBListMaxItemSize (
			ELEMTYPE ElemType
			) const;

		INT32 GetDBListNumElements (
			ELEMTYPE ElemType
			) const;

		bool IsNodeAPoint (
			INT32 NodeNum,
			SIMPLE_ARRAY<DBLISTELMT> &DBList
			) const;
					
		ERRVALUE JoinLow (
			const DBASE &ObjDbSrc,
			DBTABLETRANSLIST *pTableTransList,
			const ELEMNUMTRANS *pElemNumTrans,
			const ELEMNUMTRANS *pElemNumTransNode,
			INT32 &NextElemID,		// Next ElemID RETURNED
			ELEMTYPE SrcElemType = ELEMTYPE_Empty,
			ELEMTYPE DestElemType = ELEMTYPE_Empty,
			COPYFLAG CopyFlags = COPYFLAG_None,
			JOINFLAGS JoinFlags = JOINFLAG_None,
			const DBTABLE *pSourceTable = 0
			);
	
		//! Open the database.  If a database object does not exist, make one
		ERRVALUE OpenAsSubobject (
			const OBJECT &parent,				//!< Parent object to find and/or create database under
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE mode,
			MDLGPARENT widget,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE mode = OPENMODE_Read,
			MDLGPARENT widget = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		INT32 ReadDBList (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			SIMPLE_ARRAY<DBLISTELMT> &DBList
			) const;

		ERRVALUE ResizeDBList (
			ELEMTYPE ElemType,
			INT32 NumItems
			);

		ERRVALUE ResizeDBListItem (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			INT32 NumItems
			);

		ERRVALUE SwapDBList (
			ELEMTYPE ElemType,
			INT32 Item1,
			INT32 Item2
			);

		INT32 WriteDBList (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			const SIMPLE_ARRAY<DBLISTELMT> &DBList
			);

		INT32 UnattachDBList (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			const SIMPLE_ARRAY<DBLISTELMT> &DBList
			);

		ERRVALUE UpdateDBListNumElements (
			ELEMTYPE ElemType, 
			INT32 NumElements
			);

		DATABASEHANDLE* GetDBHandle (
			) const {
			return m_DBHandle;
			}

		#endif // GENERATING_DOXYGEN_OUTPUT
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		mutable INT32 m_NumTables;
		DATABASEHANDLE* m_DBHandle;

		static int CheckLikeFields (DBFIELDTYPE fieldtype1, DBFIELDTYPE fieldtype2, DBFIELDTYPE &BestType);

		static void DatabaseCallbackFunc (DBCALLBACKSTRUCT *cb, void *data);

		static INT32 TestIsJoinable (
			const DBTABLE &src,
			const DBTABLE &dest,
			DBTABLETRANS &TableTrans,
			DBASE::COPYFLAG CopyFlags,
			DBASE::JOINFLAGS JoinFlags
			);
			
		ERRVALUE CopyTableElementId (
			DBTABLE &SrcTable,
			DBTABLETRANS &TableTrans,
			const ELEMNUMTRANS *pElemNumTrans,
			INT32 &NextElemID
			);

		ERRVALUE CopyTableElementIdList (
			DBTABLE &SrcTable,
			DBTABLETRANS &TableTrans,
			INT32 &NextElemID
			);

		ERRVALUE CopyTableIdentity (
			DBTABLE &SrcTable,
			DBTABLETRANS &TableTrans,
			const ELEMNUMTRANS &ElemNumTrans
			);

		ERRVALUE DuplicateDatabaseRecords (
			INT32 ElemNum,
			SIMPLE_ARRAY<DBLISTELMT> &list
			);

		INT32 FindTable (const MISTRING &name) const;
		INT32 FindTable (const DESCRIPTOR &name) const;

		INT32 TableCombineUnion (
			DBTABLE &SrcTable1,
			DBTABLETRANS &TableTrans1,
			const DBTABLE &SrcTable2,
			DBTABLETRANS &TableTrans2,
			DBTRANSINFO &TransInfo,
			const DESCRIPTOR *pDescriptor,
			DBTABLE &DestTable,
		   JOINFLAGS flags
			);
			
		INT32 TableNaturalJoin (
			const DBTABLE &SrcTable1,
			DBTABLETRANS &TableTrans1,
			const DBTABLE &SrcTable2,
			DBTABLETRANS &TableTrans2,
			const DESCRIPTOR &Descriptor,
			DBTABLE &DestTable,
		   JOINFLAGS flags
			);
			
		//! Implementation of OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();					//! Close the open object
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsMultiObject () const;
		virtual bool v_IsObjectOpen () const;				//!< Is this object opened?
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);
		virtual bool v_ValidateObjectType (const OBJTYPE otype) const;			//!< Validate the object type this class can handle

		friend class DBTABLE;
		friend class DBTABLE_STYLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DBASE::COPYFLAG)
DEFINE_ENUM_OP_BITWISE(DBASE::COPYTABLE)
DEFINE_ENUM_OP_BITWISE(DBASE::FINDFLAG)
DEFINE_ENUM_OP_BITWISE(DBASE::JOINFLAGS)
#endif

//=====================================================================================================================

//! Sentry for bulk modification on database.
class DBASE::BULKMODIFY {
	public:

		//! Constructor -- calls BulkModifyBegin on the database.
		BULKMODIFY (DBASE &dbase);

		//! Destructor -- calls BulkModifyEnd on the database.
		~BULKMODIFY();
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DBASE &m_dbase;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
			
//=====================================================================================================================

//! Database object selection filter to select either a database object or a geospatial object capable of holding a database subobject (existing or not)
class DBASE::GETOBJFILTER_ISCAPABLE : public GETOBJFILTER, private OBJECT::FILTER {
	public:
		GETOBJFILTER_ISCAPABLE () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool Filter (const OBJECT &Object);
		virtual bool v_IsSelectable (const OBJITEM &objitem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};	
	
//=====================================================================================================================

//! Database object selection filter to determine if the database exists and is selectable under the parent object
class DBASE::GETOBJFILTER_ISSELECTABLE : public GETOBJFILTER, private OBJECT::FILTER {
	public:
		GETOBJFILTER_ISSELECTABLE () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool Filter (const OBJECT &Object);
		virtual bool v_HasSelectableSubObjects (const OBJITEM &objitem);
		virtual bool v_IsSelectable (const OBJITEM &objitem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};	

//=====================================================================================================================

//! Sentry for automatic suspend and resume on inverse db list operations.
class DBASE::INVERSEDBLISTSUSPEND {
	public:

		//! Constructor
		INVERSEDBLISTSUSPEND (
			DBASE &DBase,						//!< Database to suspend
			bool Rebuild = false,				//!< Rebuild inverse DB lists after resume if true
			bool Condition = true				//!< Condition to suspend inverse DB lists, true to suspend
			) :
			m_DBase(DBase),
			m_Rebuild(Rebuild),
			m_Condition(Condition)
			{
			if (m_DBase.IsOpen() && Condition) m_DBase.InverseDBListSuspend();
			}
			
		~INVERSEDBLISTSUSPEND (
			) {
			if (m_DBase.IsOpen() && m_Condition) {
				m_DBase.InverseDBListResume();
				if (m_Rebuild) m_DBase.InverseDBListBuild();
				}
			}
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DBASE &m_DBase;
		bool m_Rebuild;
		bool m_Condition;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================
	
//! OBSERVER class for DBASE objects.
//! The SUBJECT class is DBASE.
class DBASE::OBSERVER : public OBJECTOBSERVER {
	public:

		//! Types of table structural changes made, used in DBTABLE::OBSERVER::OnStructChanged()
		enum STRUCTCHANGE {
			STRUCTCHANGE_Name = 					0x00000001,
			STRUCTCHANGE_Desc = 					0x00000002,
			STRUCTCHANGE_ViewMode = 			0x00000004,
			STRUCTCHANGE_Field = 				0x00000008,			//!< Something in one or more fields changed 
			STRUCTCHANGE_FieldsAdded = 		0x00000010,
			STRUCTCHANGE_Flags = 				0x00000020,
			STRUCTCHANGE_AttachmentType = 	0x00000040,
			STRUCTCHANGE_Encoding = 			0x00000080,
			STRUCTCHANGE_DBEditorUpdate = 	0x00000100,
			STRUCTCHANGE_DBEditorRebuild = 	0x00000200
			};	

		//! Constructor
		explicit OBSERVER (
			DBASE &dbase
			) : OBJECTOBSERVER(dbase) {}
			
		//! Destructor
		virtual ~OBSERVER (
			) {}
			
		//! Called before a multi-record change
		virtual void OnBulkModifyBegin (
			);
			
		//! Called after a multi-record change
		virtual void OnBulkModifyEnd (
			);
		
		//! Called if any change has occured, useful for setting IsDirty flags
		virtual void OnAnyChange (
			);

		//! Called if an element entry is added to the dblist
		virtual void OnDBListAdd (
			ELEMTYPE ElemType,				//!< Element type
			INT32 ElemNum						//!< Element number
			);

		//! Called if an element entry is modified in the dblist
		virtual void OnDBListChanged (
			ELEMTYPE ElemType,				//!< Element type
			INT32 ElemNum						//!< Element number
			);
			
		//! Called when table structure information has changed
		virtual void OnStructChanged (
			INT32 TableNum,					//!< Table number
			STRUCTCHANGE StructChange		//!< Part of structure changed
			);
			
		//! Called when a table is added to the database
		virtual void OnTableAdd (
			INT32 TableNum						//!< Table number
			);
			
		//! Called if a table is being dropped from the database
		virtual void OnTableDropBegin (
			INT32 TableNum						//!< Table number
			);
		
		//! Called if a table is dropped from the database
		virtual void OnTableDropEnd (
			INT32 TableNum						//!< Table number
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual void OnNotifyRaw (UINT32 reason, void *data);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

class DBELEMTYPEFILTER : public OBJECT::FILTER {
	public:

		DBELEMTYPEFILTER (
			ELEMTYPE ElemType
			) :
			m_ObjType(OBJTYPE_Database)
			{
			switch (ElemType) {
				case ELEMTYPE_Node: m_ObjType = OBJTYPE_All; break;
				case ELEMTYPE_Line: m_ObjType = OBJTYPE_Database_VectorLine; break;
				case ELEMTYPE_Point: m_ObjType = OBJTYPE_Database_VectorPoint; break;
				case ELEMTYPE_Polygon: m_ObjType = OBJTYPE_Database_VectorPolygon; break;
				case ELEMTYPE_Edge:
				case ELEMTYPE_Triangle: m_ObjType = OBJTYPE_Database_TIN; break;
				case ELEMTYPE_CAD: m_ObjType = OBJTYPE_Database_CAD; break;
				case ELEMTYPE_RasterCellValue: m_ObjType = OBJTYPE_Database_Raster; break;
				case ELEMTYPE_All: m_ObjType = OBJTYPE_Database_Shape; break;
				case ELEMTYPE_Label: m_ObjType = OBJTYPE_Database_VectorLabel; break;
				}
			}

		virtual bool Filter (
			const OBJECT &object
			) {
			if (m_ObjType == OBJTYPE_All) {			//! Vector or TIN node database
				OBJECT parent;
				if (object.GetParent(parent) < 0) return (false);
				m_ObjType = (parent.GetObjectType() == OBJTYPE_Vector) ? OBJTYPE_Database_VectorPoint : OBJTYPE_Database_TIN;
				}
			return (object.GetObjectType() == m_ObjType);
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		OBJTYPE m_ObjType;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}		// End of RVC namespace

#endif 	//INC_RVC_DBASE_H
