/**
 * \file rvc/dbfieldcontrol.h
 * \brief RVC::DBFIELDCONTROL definitions
 *
 * \if NODOC
 * $Id: dbfieldcontrol.h_v 1.2 2015/05/28 13:45:44 mju Exp $
 *
 * $Log: dbfieldcontrol.h_v $
 * Revision 1.2  2015/05/28 13:45:44  mju
 * Add defaultmode_FromSystem.
 *
 * Revision 1.1  2014/10/01 21:15:17  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_DBFIELDCONTROL_H
#define	INC_RVC_DBFIELDCONTROL_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_RVC_DBFIELDTYPE_H
#include <rvc/dbfieldtype.h>
#endif


#ifdef RVCSYSDLL
	#define CLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace RVC {

//=====================================================================================================================

//! Specification for controls on single database field.
//! Controls include default values, allowed range, case modification, matching key enforcement,
//! autocompletion and alternative labels.
// This class replaces and unifies various instances of legacy database "constraints" structures.
// Enumeration values are combined and stored in a single 32-bit unsigned value so cannot overlap or change.
class CLASSEXPORT DBFIELDCONTROL {
	public:

		//! Mode for default value.
		enum DEFAULTMODE {
			DEFAULTMODE_Null =				0,				//!< Default to empty for strings and null/invalid for other types
			DEFAULTMODE_Constant = 			0x00000010,	//!< Default to constant value
//			DEFAULTMODE_Previous =			0x00000020,	//!< Default to previous value if available
//			DEFAULTMODE_Blank =				0x00000040,	// LEGACY, do not reuse
			DEFAULTMODE_FromSystem =		0x00000080,	//!< Default to value obtained from system (for current date/time)
			};

		//! Mode for minimum numeric value checking.
		enum MINMODE {
			MINMODE_None =						0,				//!< No specific minimum (will still be limited by field type)
			MINMODE_Exclusive =				0x00000001,	//!< Value must be greater than minimum (not recommended, and same as Inclusive for floating-point)
			MINMODE_Inclusive =				0x00000004,	//!< Value must be greater or equal to minimum
			};

		//! Mode for maximum numeric value checking.
		enum MAXMODE {
			MAXMODE_None =						0,				//!< No specific maximum (will still be limited by field type)
			MAXMODE_Exclusive =				0x00000002,	//!< Value must be greater than maximum (not recommended, and same as Inclusive for floating-point)
			MAXMODE_Inclusive =				0x00000008,	//!< Value must be greater or equal to maximum
			};

		//! Mode for case of string fields.
		enum CASEMODE {
			CASEMODE_None =					0,
			CASEMODE_ForceUpperCase =		0x00000100,	//!< Make string all uppercase
			CASEMODE_ForceLowerCase =		0x00000200,	//!< Make string all lowercase
			CASEMODE_CapFirstLetter =		0x00000400,	//!< Capitalize First Letter Of Every Word
			CASEMODE_CapFirstWord =			0x00000800,	//!< Capitalize first letter of first word
			};

		//! Action for missing key value.
		enum KEYMISSING {
			KEYMISSING_None =					0,
			KEYMISSING_MustExist =			0x00008000,	//!< Record related by value must exist in primary key table
			KEYMISSING_AutoAdd =				0x00010000,	//!< If not in related primary key table, add it
			KEYMISSING_AutoAddPrompt =		0x00020000,	//!< If not in related primary key table, prompt to add
			};

		//! Interface for value related to primary key.
		enum KEYUI {
			KEYUI_Default =					0,				//!< Default user interface, may be simple edit or picklist
			KEYUI_ComboBox =					0x00040000,	//!< Editable text with popup list
			KEYUI_OptionMenu =				0x00080000,	//!< Popup Menu
			KEYUI_CSVListAllowDups =		0x01000000,	//!< Values stored comma-separated, chosen from list, duplicates allowed
			KEYUI_CSVListUnique =    		0x08000000,	//!< Values stored comma-separated, chosen from list, no duplicates
			};

		//! Default constructor.
		DBFIELDCONTROL ();

		//! Copy constructor.
		DBFIELDCONTROL (const DBFIELDCONTROL& rhs);

		//! Destructor.
		~DBFIELDCONTROL ();

		//! Assignment.
		DBFIELDCONTROL& operator= (const DBFIELDCONTROL& rhs);

		//! Equality.
		bool operator== (const DBFIELDCONTROL& rhs) const
			{ return (IsEqual(rhs)); }

		//! Inequality.
		bool operator!= (const DBFIELDCONTROL& rhs) const
			{ return (!IsEqual(rhs)); }

		//! Apply range controls.
		//! Apply range controls.
		//! @return true if value changed, false if not
		bool ApplyRange (
			double& value					//!< Value passed/returned
			) const;

		//! Clear all controls.
		void Clear ();

		//! Get whether to auto-complete value during entry.
		bool GetAutoComplete () const
			{ return (m_bAutoComplete); }

		//! Get case mode.
		CASEMODE GetCaseMode () const
			{ return (m_CaseMode); }

		//! Get default mode.
		DEFAULTMODE GetDefaultMode () const
			{ return (m_DefaultMode); }

		//! Get default numeric value.
		double GetDefaultNum () const
			{ return (m_DefaultNum); }

		//! Get default string value.
		const MISTRING& GetDefaultStr () const
			{ return (m_DefaultStr); }

		//! Get setting for missing key.
		KEYMISSING GetKeyMissing () const
			{ return (m_KeyMissing); }

		//! Get user interface type for field related to key.
		KEYUI GetKeyUI () const
			{ return (m_KeyUI); }

		//! Get label to display after field in interface (typically in place of units).
		const MISTRING& GetLabelAfter () const
			{ return (m_LabelAfter); }

		//! Get label to display before field in interface (typically in place of name).
		const MISTRING& GetLabelBefore () const
			{ return (m_LabelBefore); }

		//! Get mode for maximum value checking.
		MAXMODE GetMaxMode () const
			{ return (m_MaxMode); }

		//! Get maximum value.
		double GetMaxValue () const
			{ return (m_MaxValue); }

		//! Get mode for minimum value checking.
		MINMODE GetMinMode () const
			{ return (m_MinMode); }

		//! Get minimum value.
		double GetMinValue () const
			{ return (m_MinValue); }

		//! Determine if has default.
		bool HasDefault () const
			{ return (m_DefaultMode != DEFAULTMODE_Null); }

		//! \internal
		void IniRead (INIHANDLE inihnd, const MIUNICODE *fieldname);

		//! \internal
		void IniWrite (INIHANDLE inihnd, const MIUNICODE *fieldname) const;

		//! Compare for equality.
		bool IsEqual (
			const DBFIELDCONTROL& rhs
			) const;

		//! Set whether to auto-complete string value during entry.
		void SetAutoComplete (
			bool bAutoComplete
			) { m_bAutoComplete = bAutoComplete; }

		//! Get case mode.
		void SetCaseMode (
			CASEMODE CaseMode
			) { m_CaseMode = CaseMode; }

		//! Set default mode.
		void SetDefaultMode (
			DEFAULTMODE DefaultMode
			) { m_DefaultMode = DefaultMode; }

		//! Set default for numeric fields.
		//! Also sets DEFAULTMODE_Constant.			
		void SetDefaultNum (
			double DefaultNum
			) { m_DefaultNum = DefaultNum; m_DefaultMode = DEFAULTMODE_Constant; }
			
		//! Set default for string fields.
		//! Also sets DEFAULTMODE_Constant.			
		void SetDefaultStr (
			const MISTRING DefaultStr
			) { m_DefaultStr = DefaultStr; m_DefaultMode = DEFAULTMODE_Constant; }

		//! Set behavior for missing key.
		void SetKeyMissing (
			KEYMISSING KeyMissing
			) { m_KeyMissing = KeyMissing; }

		//! Set user interface type for field related to key.
		void SetKeyUI (
			KEYUI KeyUI
			) { m_KeyUI = KeyUI; }

		//! Set label to display after field in interface (typically in place of units).
		void SetLabelAfter (
			const MISTRING& label
			) { m_LabelAfter = label; }

		//! Set label to display before field in interface (typically in place of name).
		void SetLabelBefore (
			const MISTRING& label
			) { m_LabelBefore = label; }

		//! Set maximum value and optional mode.
		//! @return true if specified maximum value is less than previous enforced minimum, which will be adjusted, false otherwise
		bool SetMaximum (
			double MaxValue,
			MAXMODE MaxMode = MAXMODE_Inclusive
			);

		//! Set maximum value test mode.
		void SetMaxMode (
			MAXMODE MaxMode
			) { m_MaxMode = MaxMode; }
			
		//! Set minimum value and mode.
		//! @return true if specified minimum value is greater than previous enforced maximum, which will be adjusted, false otherwise
		bool SetMinimum (
			double MinValue,
			MINMODE MinMode = MINMODE_Inclusive
			);

		//! Set minimum value test mode.
		void SetMinMode (
			MINMODE MinMode
			) { m_MinMode = MinMode; }
			
		//! Set range (inclusive).
		void SetRange (
			double MinValue,
			double MaxValue
			) { SetMinimum(MinValue); SetMaximum(MaxValue); }

		//! Setup range based on field type and min/max modes.
		//! This will ensure that range is valid for field type, and if not floating-point
		//! will adjust min/max modes if necessary.
		bool SetupRange (
			DBFIELDTYPE fieldtype					//! Field type.
			);
					
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		static const UINT32 KEYUI_AutoComplete = 0x00400000;

		double m_DefaultNum;
		double m_MinValue;
		double m_MaxValue;
		MISTRING m_DefaultStr;
		MISTRING m_LabelBefore;
		MISTRING m_LabelAfter;
		DEFAULTMODE m_DefaultMode;
		MINMODE m_MinMode;
		MAXMODE m_MaxMode;
		KEYMISSING m_KeyMissing;
		KEYUI m_KeyUI;
		CASEMODE m_CaseMode;
		bool m_bAutoComplete;
		#endif
	};

//=====================================================================================================================

}	// End namespace RVC

#undef CLASSEXPORT

#endif	// INC_RVC_DBFIELDCONTROL_H
