/**
 * \file rvc/dbtabletrans.h
 * \brief RVC::DBTABLETRANS definition
 *
 * \if NODOC
 * $Id: dbtabletrans.h_v 1.6 2015/10/27 17:11:08 mju Exp $
 *
 * $Log: dbtabletrans.h_v $
 * Revision 1.6  2015/10/27 17:11:08  mju
 * Add get/setFreezeComputed.
 *
 * Revision 1.5  2015/08/04 15:06:35  mju
 * Add get/setCopyAllRecords.
 *
 * Revision 1.4  2015/06/02 17:04:45  mju
 * Add is/setDeRelated.
 *
 * Revision 1.3  2015/05/15 15:31:15  mju
 * Rename member.
 *
 * Revision 1.2  2015/05/13 21:20:38  mju
 * Add isSrcRecNumSelected.
 *
 * Revision 1.1  2015/05/12 21:43:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_DBTABLETRANS_H
#define	INC_RVC_DBTABLETRANS_H

#ifndef INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

namespace RVC {

//=====================================================================================================================

//! Table translation, used in copy and join operations for determining how table is translated to new database.
class DBTABLETRANS {
	public:

		//! Default constructor.
		DBTABLETRANS ();

		//! Copy constructor.
		DBTABLETRANS (
			const DBTABLETRANS &rhs
			);

		//! Destructor.
		~DBTABLETRANS ();

		//! Assignment.
		DBTABLETRANS& operator= (
			const DBTABLETRANS &rhs
			);

		//! Access set of source record numbers to be copied.
		BITSET& AccessSrcRecSetCopy ()
			{ return (m_SrcRecSetCopy); }

		//! Translate an old record number to a new record numbers and append to dblist.
		//! This may add more than one entry.
		void AddTranslatedEntryToDbList (
			SIMPLE_ARRAY<DBLISTELMT> &dblist,
			INT32 OldRecNum
			) const;

		//! Copy from a source table record to target record.
		ERRVALUE CopyRecordToNew (
			const DBTABLE::RECORD &SrcRecord,
			DBTABLE::RECORD &TgtRecord
			) const;

		//! Get whether to copy all records.
		bool GetCopyAllRecords () const
			{ return (m_bCopyAllRecords); }

		//! Get whether to "freeze" computed fields.
		bool GetFreezeComputed () const
			{ return (m_bFreezeComputed); }

		//! Get target record numbers.
		ERRVALUE GetRecNumTgt (
			INT32 RecNumSrc,
			SIMPLE_ARRAY<INT32> &RecNumTgt
			) const;

		//! Get record number translation.
		const ELEMNUMTRANS& GetRecNumTrans () const
			{ return (m_RecNumTrans); }

		//! Get set of source record numbers to be copied.
		const BITSET& GetSrcRecSetCopy () const
			{ return (m_SrcRecSetCopy); }

		//! Get source table name.
		const MISTRING& GetSrcTableName () const
			{ return (m_SrcTableName); }

		//! Get source table number.
		INT32 GetSrcTableNum () const
			{ return (m_SrcTableNum); }

		//! Get target table descriptor.
		const DESCRIPTOR& GetTgtTableDesc () const
			{ return (m_TgtTableDesc); }

		bool IsDeRelated () const
			{ return (m_bIsDeRelated); }

		//! Determine if source record is selected.
		bool IsSrcRecNumSelected (
			UINT32 RecNumSrc
			) const { return (!m_SrcRecSetCopy.HasEntries() || m_SrcRecSetCopy.GetBit(RecNumSrc)); }

		//! Get whether non-joinable.
		bool IsNonJoinable () const
			{ return (m_bNonJoinable || m_TgtTableNum < 0); }

		//! Set whether to copy all records.
		void SetCopyAllRecords (
			bool bCopyAllRecords = true
			) { m_bCopyAllRecords = bCopyAllRecords; }

		//! Set as having been "derelated" (changed from related to directly attached).
		void SetDeRelated (
			bool bIsDeRelated = true
			) { m_bIsDeRelated = bIsDeRelated; }

		//! Set field translation.
		void SetFieldTrans (
			UINT32 FieldNumSrc,			//! Source field number
			INT32 FieldNumTgt				//! Target field number
			) { if (FieldNumSrc < m_FieldTrans.GetNumItems()) m_FieldTrans[FieldNumSrc] = FieldNumTgt; }

		//! Set whether to "freeze" computed fields (make non-computed).
		void SetFreezeComputed (
			bool bFreezeComputed = true
			) { m_bFreezeComputed = bFreezeComputed; }

		//! Set whether non-joinable.
		void SetNonJoinable (
			bool bNonJoinable = true
			);

		//! Set record number translation from array.
		ERRVALUE SetRecNumTrans (
			const SIMPLE_ARRAY<INT32> &RecNumSrcToTgt
			);

		//! Set source table.
		ERRVALUE SetSrcTable (
			const DBTABLE &ObjTable
			);

		//! Set source table name.
		void SetSrcTableName (
			const MISTRING &TableName
			);

		//! Set target table.
		ERRVALUE SetTgtTable (
			const DBTABLE &ObjTable
			);

		//! Set target table descriptor.
		//! To set target to existing table, simply specify the name of the existing target table.
		void SetTgtTableDesc (
			const DESCRIPTOR &TableDesc
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_SrcTableName;
		DESCRIPTOR m_TgtTableDesc;
		ELEMNUMTRANS m_RecNumTrans;
		SIMPLE_ARRAY<INT32> m_FieldTrans;			// Source to target Field number translation
		SIMPLE_ARRAY<UNITCONV> m_FieldUnitConv;	// Unit conversion from source to target for each field
		BITSET m_SrcRecSetCopy;					// Which source records to copy, if empty copy all records
		INT32 m_SrcTableNum;
		INT32 m_TgtTableNum;
		INT32 m_SrcFieldNumElemIdCurrent;
		INT32 m_SrcFieldNumElemIdOriginal;
		bool m_bDoNotCopy;
		bool m_bCleanFieldJoin;
		bool m_bModFieldJoin;
		bool m_bNonJoinable;
		bool m_bDropDestTable;
		bool m_bIsIdentityTable;
		bool m_bIsRefToNode;
		bool m_bIsElemIdTable;
		bool m_bIsStdAttTable;
		bool m_bIsDeRelated;
		bool m_bCopyAllRecords;
		bool m_bFreezeComputed;

		friend class DBASE;
		friend class DBTABLE;
		#endif
	};

//=====================================================================================================================

}	// End namespace RVC

#endif	// INC_RVC_DBDBTABLETRANS_H
