/**
 * \file rvc/georefer.h 
 * \brief interface for the RVC::GEOREFERENCE class.
 *
 * \if NODOC
 * $Id: georefer.h_v 1.42 2017/02/24 14:53:11 mju Exp $
 *
 * $Log: georefer.h_v $
 * Revision 1.42  2017/02/24 14:53:11  mju
 * Adjust camera methods.
 *
 * Revision 1.41  2017/02/22 19:12:09  mju
 * Use cameraintrinsics instead of camera3d.
 *
 * Revision 1.40  2017/02/22 17:00:56  mju
 * Add get/setCameraIntrinsics methods and related members.
 *
 * Revision 1.39  2014/05/23 13:43:01  mju
 * Make setRationalPoly return errvalue.
 *
 * Revision 1.38  2014/02/26 22:32:00  mju
 * Add setMapTrans.
 *
 * Revision 1.37  2013/05/01 21:39:36  mju
 * Add rational polynomial methods.
 *
 * Revision 1.36  2011/12/23 15:37:14  mju
 * Use trans2daffine hdr.
 *
 * Revision 1.34  2007/09/06 17:18:48  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.33  2007/06/15 16:37:01  mju
 * Add setPointList from array/count.
 *
 * Revision 1.7  2001/09/05 21:05:33  scowan
 * Added conversion methods on the source coordinates.
 *
 * Revision 1.1  2000/09/29 16:25:29  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_GEOREFER_H
#define INC_RVC_GEOREFER_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_TRANS2DAFFINE_H
	#include <mi32/trans2daffine.h>
#endif

#ifndef INC_MI32_TRANSMOD_H
	#include <mi32/transmod.h>
#endif

#ifndef INC_MI32_SPATREF_H
	#include <mi32/spatref.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_MAPGEN;
class RATIONALPOLY;
class CAMERAINTRINSICS;
class CAMERA3D;
namespace RVC {
	class GEOREFHANDLE;
	}
#endif


namespace RVC {

//=====================================================================================================================

//! Manage RVC Georeference objects.
//!
//!	Object Types that this class supports: RVC::OBJTYPE_Georeference.
//!
//!	Valid parents for this class: Any derived from RVC::GEOSPATIAL
//!
//!	Read() and Write() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods
class GEOREFERENCE : public SIMPLE {  
	public:

		//! Type of georeference.
		enum GEOREFTYPE {
			GEOREFTYPE_Implied =			5,			//!< Georeference information is implied in the data
			GEOREFTYPE_AffineMatrix =	6,			//!< Affine transformation matrix
			GEOREFTYPE_CtrlPoint =		7,			//!< List of CTRLPT3D's
			GEOREFTYPE_Manifold = 		8			//!< Set of CTRLPT3D's to create a Manifold on
			};

		struct CTRLPTEDGE {
			INT32 m_StartCtrlPtID;
			INT32 m_EndCtrlPtID;
			};
	
		//! Default Constructor
		GEOREFERENCE ();

		//! Copy Constructor
		GEOREFERENCE (
			const RVC::GEOREFERENCE &rhs					
			);										

		//! Destructor
		virtual ~GEOREFERENCE ();										

		//! Assignment
		RVC::GEOREFERENCE& operator= (
			const RVC::GEOREFERENCE &rhs
			);

		//! Add a georeference boundary control point array to the list of boundary control point arrays
		//! Only works for extented georeference objects
		//! @return Number of the added boundary control point array
		INT32 AddBoundary (
			const SIMPLE_ARRAY<INT32> &Boundary
			);
			
		//! Add a georeference control point to the list of control points
		//! Works for either basic or extented georeference objects
		//! @return Number of the added control point
		INT32 AddCtrlPoint (
			const CTRLPT3D &CtrlPoint
			);
			
		//! Add a georeference control point edge entry to the list of control points edge entries
		//! Only works for extented georeference objects
		//! @return Number of the added control point edge entry
		INT32 AddCtrlPointEdge (
			const CTRLPTEDGE &CtrlPtEdge
			);

		//! Clear out all georeference control point array entries
		//! Only works for extented georeference objects
		ERRVALUE ClearBoundaries ();
			
		//! Clear out all georeference control point entries
		//! Works for either basic or extented georeference objects.
		ERRVALUE ClearCtrlPoints ();
			
		//! Clear out all georeference control point edge entries
		//! Only works for extented georeference objects
		ERRVALUE ClearCtrlPointEdges ();
			
		//! Convert control point list 'source' coordinates using an affine transformation
		//! Only works for internally stored ctrl point lists, does not affect disabled ctrl points
		ERRVALUE ConvertForward (
			const TRANS2D_AFFINE &Affine
			);
		
		//! Convert control point list 'source' coordinates using an affine transformation
		//! Only works for internally stored ctrl point lists, does not affect disabled ctrl points
		ERRVALUE ConvertInverse (
			const TRANS2D_AFFINE &Affine
			);
		
		//! Convert control point list 'source' coordinates using an TRANS2D_MAPGEN transformation
		//! Only works for internally stored ctrl point lists, does not affect disabled ctrl points
		ERRVALUE ConvertForward (
			const TRANS2D_MAPGEN &tmg
			);
		
		//! Convert control point list 'source' coordinates using an TRANS2D_MAPGEN transformation
		//! Only works for internally stored ctrl point lists, does not affect disabled ctrl points
		ERRVALUE ConvertInverse (
			const TRANS2D_MAPGEN &tmg
			);

		//! Convert a basic georeference object to an extented georeference object		
		//! Only works for basic georeference objects
		ERRVALUE ConvertToExtended ();
			
		//! Delete a control point at the location specified
		//! Maintains the order of the control point list
		//! Works for either basic or extented georeference objects.
		ERRVALUE DeleteCtrlPoint (
			INT32 CtrlPointNum
			);
			
		//! Delete a control point edge entry at the location specified
		//! Maintains the order of the control point edge list
		//! Only works for extented georeference objects
		ERRVALUE DeleteCtrlPointEdge (
			INT32 CtrlPointEdgeNum
			);
			
		//! Get georeference affine matrix.  Only value if GetGeorefType() == GEOREFTYPE_AffineMatrix
		//! @return Affine matrix
		const TRANS2D_AFFINE& GetAffineMatrix () const
			{ return (m_TransAffine); }

		//! Obtain the calibration / translation model from the rvc georeference object
		//! @return calibration / translation model class instance
		TRANSMODEL GetCalibModel () const
			{ return (m_TransModel); }

		//! Get camera parameters if available.
		//! @return TRUE if available, FALSE if not, < 0 if error.
		int GetCameraValues (
			CAMERAINTRINSICS &CameraIntrinsics,		//!< Camera intrinsic parameters returned
			CAMERA3D &Camera3D							//!< Camera 3D transformation returned
			);

		//! Get georeference Coordinate reference system parameters
		//! @return Coordinate reference system parameters to use
		const SPATREF::COORDREFSYS& GetCoordRefSys () const
			{ return (m_CoordRefSys); }

		//! Obtain the georeference type
		//! @return Georeference Type
		GEOREFTYPE GetGeorefType () const;

		//! Generate a TRANS2D_MAPGEN from the georeference instance
		ERRVALUE GetMapTrans (
			TRANS2D_MAPGEN &tmg,							//!< Generic transformation generated and RETURNED
			bool DoOutput = false,						//!< Place transformation on output side if true
			TRANSMODEL model = 0							//!< Translation model, zero sets up the default model
			) const;

		//! Get the number of boundary control point array items
		//! Only works for extented georeference objects
		//! @return Number of boundary control point array entries
		INT32 GetNumBoundaries () const;
			
		//! Get the number of control points
		//! Works for either basic or extented georeference objects
		//! @return Number of control points
		INT32 GetNumCtrlPoints () const;
			
		//! Get the number of control points edge entries
		//! Only works for extented georeference objects
		//! @return Number of control point edge entries
		INT32 GetNumCtrlPointEdges () const;
			
		//! Generate a control point list from the georeference
		//! This method will generate control points if the georeference is not a control point georeference
		ERRVALUE GetPointList (
			SIMPLE_ARRAY<CTRLPT3D> &CtrlList		//!< Control point list allocated and returned
			) const;

		//! Get rational polynomial definition if available.
		//! @return TRUE if available, FALSE if not, < 0 if error.
		int GetRationalPoly (
			RATIONALPOLY &RationalPoly					//!< Rational polynomial definition read
			);

		//! Insert a control point at the location specified
		//! Maintains the order of the control point list
		//! Works for either basic or extented georeference objects.
		ERRVALUE InsertCtrlPoint (
			INT32 CtrlPointNum,
			const CTRLPT3D &CtrlPoint
			);
			
		//! Insert a control point edge entry at the location specified
		//! Maintains the order of the control point edge list
		//! Works for extented georeference objects only.
		ERRVALUE InsertCtrlPointEdge (
			INT32 CtrlPointEdgeNum,
			const CTRLPTEDGE &CtrlPtEdge
			);
			
		//! Determine if the georeference object is basic or extented
		//! @return 'True' if the georeference object is basic, 'false' if extented
		bool IsBasicFormat () const;
			
		//! Determine if the referenced control point is enabled
		//! Works for either basic or extented georeference objects.  Standard georeference objects always return 'true'
		//! @return 'True' if the control point is enabled, 'false' if not
		bool IsCtrlPointEnabled (
			INT32 CtrlPointIndex
			) const;

		//! Determine if the two georeference objects are equivalent
		//! @return 'True' if the georeference information is equivalent, 'false' if not
		bool IsEquivalentTo (
			const RVC::GEOREFERENCE &GeorefObj
			) const;
			
		//! Read a specific georeference boundary control point array entry
		//! Only works for extented georeference objects
		ERRVALUE ReadBoundary (
			INT32 BoundaryIndex,
			SIMPLE_ARRAY<INT32> &Boundary
			);
			
		//! Read a specific georeference control point
		//! Works for either basic or extented georeference objects.
		ERRVALUE ReadCtrlPoint (
			INT32 CtrlPointIndex,
			CTRLPT3D &CtrlPoint
			);
			
		//! Read a specific georeference control point edge entry
		//! Only works for extented georeference objects
		ERRVALUE ReadCtrlPointEdge (
			INT32 CtrlPointEdgeIndex,
			CTRLPTEDGE &CtrlPtEdge
			);
			
		//! Read a specific georeference control point ID entry
		//! Enhanced georeference objects can store an ID value associated with the control point
		//! Only works for extented georeference objects
		INT32 ReadCtrlPointID (
			INT32 CtrlPointIndex
			);
			
		//! Read a specific georeference control point name entry
		//! Enhanced georeference objects can store a name value associated with the control point
		//! Only works for extented georeference objects
		ERRVALUE ReadCtrlPointName (
			INT32 CtrlPointIndex,
			MISTRING &CtrlPointName
			);

		//! Set the number of control point edge entries 
		//! Only works for extented georeference objects
		ERRVALUE ResizeCtrlPointEdges (
			INT32 NumCtrlPointEdges
			);
			
		//! Set the number of control points 
		//! Works for either basic or extented georeference objects.
		ERRVALUE ResizeCtrlPoints (
			INT32 NumCtrlPoints
			);
			
		//! Set georeference to be an affine transformation
		void SetAffine (
			const TRANS2D_AFFINE &taf					//!< Affine transformation to set in georeference
			);

		//! Set transformation model.
		void SetCalibModel (						
			const TRANSMODEL &model							//!< Transformation model to use
			);

		//! Set camera parameters.
		//! Note, this does not change the georeference type or model.
		ERRVALUE SetCameraValues (
			const CAMERAINTRINSICS &CameraIntrinsics,	//!< Camera intrinsic parameters
			const CAMERA3D &Camera3D						//!< Camera 3D transformation
			);

		//! Set georeference Coordinate reference system parameters
		ERRVALUE SetCoordRefSys (
			const SPATREF::COORDREFSYS &CoordRefSys	//!< Coordinate reference system parameters to use
			);

		//! Set the specified control point to be enabled or disabled
		//! Only works for extented georeference objects
		ERRVALUE SetCtrlPointEnabled (
			INT32 CtrlPointIndex,
			bool IsEnabled
			);
			
		//! Set georeference to be an implied transformation
		void SetImplied ();

		//! Set georeference to be a Manifold georeference
		void SetManifold ();

		//! Set transformation from TRANS2D_MAPGEN.
		//! Sets model, CRS, control points (if any).
		ERRVALUE SetMapTrans (
			const TRANS2D_MAPGEN &trans,					//!< Transformation to set from.
			bool bFromOutput = false						//!< Whether to use "output" side of transformation
			);
			
		//! Set georeference to be an control point transformation
		ERRVALUE SetPointList (
			const SIMPLE_ARRAY<CTRLPT3D> &CtrlList	//!< Control point list to use
			);
			
		//! Set georeference to be an control point transformation
		ERRVALUE SetPointList (
			const CTRLPT3D *CtrlPtList,					//!< Control point list to use
			int NumCtrlPts										//!< Number of control points
			);

		//! Set rational polynomial values.
		//! Note, this does not change the georeference type or model.
		ERRVALUE SetRationalPoly (
			const RATIONALPOLY &RationalPoly				//!< Values to set
			);

		//! Write a specific georeference boundary control point array entry
		//! Only works for extented georeference objects
		ERRVALUE WriteBoundary (
			INT32 BoundaryIndex,
			const SIMPLE_ARRAY<INT32> &Boundary
			);
			
		//! Write a specific georeference control point.  Use AddCtrlPoint() to add control points to the list
		//! Works for either basic or extented georeference objects.
		ERRVALUE WriteCtrlPoint (
			INT32 CtrlPointIndex,
			const CTRLPT3D &CtrlPoint
			);
			
		//! Write a specific georeference control point edge entry
		//! Only works for extented georeference objects
		ERRVALUE WriteCtrlPointEdge (
			INT32 CtrlPointEdgeIndex,
			const CTRLPTEDGE &CtrlPtEdge
			);
			
		//! Write a specific georeference control point name entry
		//! Enhanced georeference objects can store an ID value associated with the control point
		//! Only works for extented georeference objects
		ERRVALUE WriteCtrlPointID (
			INT32 CtrlPointIndex,
			INT32 CtrlPointID
			);
			
		//! Write a specific georeference control point name entry
		//! Enhanced georeference objects can store a name value associated with the control point
		//! Only works for extented georeference objects
		ERRVALUE WriteCtrlPointName (
			INT32 CtrlPointIndex,
			const MISTRING &CtrlPointName
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		
		SPATREF::COORDREFSYS m_CoordRefSys;		// Coordinate reference system of the georeference
		SIMPLE_ARRAY<CTRLPT3D> m_CtrlList;		// Control point list if georeference has one
		TRANS2D_AFFINE m_TransAffine;				// Affine transformation if georeference has one
		TRANSMODEL m_TransModel;					// Transformation model
		GEOREFHANDLE *m_pGeorefHandle;
		RATIONALPOLY *m_pRationalPoly;
		CAMERAINTRINSICS *m_pCameraIntrinsics;
		CAMERA3D *m_pCamera3D;
	 	GEOREFTYPE m_GeorefType;					// Type of georeference used
		bool m_bChangedType;							// Type of projection (Affine, Implied, Ctrl Point) and or projection name has changed
		bool m_bChangedRationalPoly;
		bool m_bChangedCameraValues;

		ERRVALUE ReadStandardGeorefData ();
		ERRVALUE UpdateHeader ();
		ERRVALUE WriteCameraValues ();
		ERRVALUE WriteRationalPoly ();

		// Implementation of RVC::OBJECT virtual methods.
		virtual ERRVALUE v_CloseObject ();
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;					
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		// Implementation of RVC::SIMPLE virtual methods.
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT &parent, DESCRIPTOR &Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS &ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS &ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS &ObjOpenParms);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

}		// End of RVC namespace

#endif	// INC_RVC_GEOREFER_H
