/**
 * \file rvc/opbuffer.h
 * \brief Definitions for RVC::OP::BUFFER class.
 *
 * \if NODOC
 * $Id: opbuffer.h_v 1.5 2014/05/12 20:27:13 mju Exp $
 *
 * $Log: opbuffer.h_v $
 * Revision 1.5  2014/05/12 20:27:13  mju
 * For setDistanceMultiEqual specify number of steps instead of end distance.
 *
 * Revision 1.4  2014/04/10 16:29:01  mju
 * Add setProcessSet using bitset_unowned and getProcessSet.
 *
 * Revision 1.3  2014/04/09 17:31:01  mju
 * Add setColorFill, remove method setting.
 *
 * Revision 1.2  2014/04/07 18:00:32  mju
 * Use elemset for process set.
 *
 * Revision 1.1  2014/04/04 20:59:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef  INC_RVC_OPBUFFER_H
#define  INC_RVC_OPBUFFER_H

#ifndef	INC_RVC_GEOTABLE_H
#include <rvc/geotable.h>
#endif

#ifndef	INC_RVC_ELEMSET_H
#include <rvc/elemset.h>
#endif

#ifndef  INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifndef	INC_MI32_UOM_H
#include <mi32/uom.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MIJOB;
class BITSET_UNOWNED;
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Class for buffer zone generation.
class BUFFER {
	public:

		//! Which element type(s) to buffer.
		enum BUFFERELEMS {
			BUFFERELEM_None =				0,
			BUFFERELEM_Point = 			0x10,
			BUFFERELEM_Line =				0x20,
			BUFFERELEM_Node =				0x40,
			BUFFERELEM_Area =				0x80,
			BUFFERELEM_MASK =				0xF0
			};

		//! How areas are buffered.
		enum AREAMODE {
			AREAMODE_Expand =				0,
			AREAMODE_Shrink =				1,
			AREAMODE_Outer =				2,
			AREAMODE_Inner =				3,
			AREAMODE_Border =				4,
			AREAMODE_MASK =				0x0F
			};

		//! How buffers are merged.
		enum MERGE {
			MERGE_None = 0,
			MERGE_All,
			MERGE_ByAttribute
			};

		//! Default constructor.
		BUFFER ();

		//! Destructor.
		~BUFFER ();

		//! Get array of distances.
		//! If distance is by script, array will contain single value of 0.
		const DOUBLE_ARRAY<double>& GetDistances () const
			{ return (m_Distances); }

		//! Get distance unit.
		const UOM::UNIT& GetDistanceUnit () const
			{ return (m_DistanceUnit); }

		//! Get set of elements to be processed.
		const ELEMSET& GetProcessSet () const
			{ return (m_ProcessSet); }

		//! Determine if has specfied set of elements to process.
		bool HasProcessSet () const;

		//! Read settings from job.
		ERRVALUE JobRead (
			MIJOB& job
			);

		//! Write all settings to job.
		ERRVALUE JobWrite (
			MIJOB& job,
			INT32 DistanceIndex = -1
			);

		//! Run buffer operation.
		ERRVALUE Run (
			const RVC::OBJITEM& ObjItemSrc,			//!< Source object location
			RVC::OBJITEMLIST& ObjItemListTgt			//!< Output object locations passed/returned
			) const;

		//! Set area buffering mode.
		void SetAreaMode (
			AREAMODE AreaMode
			) { m_AreaMode = AreaMode; }

		//! Set which element types to buffer.
		void SetBufferElems (
			BUFFERELEMS BufferElems
			) { m_BufferElems = BufferElems; }

		//! Set color to fill buffer areas.
		void SetColorFill (
			const COLOR& color							//!< Color to use.
			) { m_ColorFill = color; }

		//! Set distance as all-same.
		ERRVALUE SetDistanceAllSame (
			double distance								//!< Distance to use, must be positive
			);

		//! Set list of distance values.
		ERRVALUE SetDistanceList (
			const DOUBLE_ARRAY<double> distances	//!< Distances to use, all must be positive
			);

		//! Set multiple equal distances.
		ERRVALUE SetDistanceMultiEqual (
			double start,									//!< Starting distance, if <= 0 will use "interval" as starting
			double interval,								//!< Distance interval, must be positive
			int steps										//!< Number of steps, must be positive
			);

		//! Set distance script.
		ERRVALUE SetDistanceScript (
			const MISTRING& script						//!< Distance script.
			);

		//! Set distance table and field.
		ERRVALUE SetDistanceTableField (
			const MISTRING& TableName,					//!< Table name
			const MISTRING& FieldName					//!< Field name
			);

		//! Set distance unit.
		void SetDistanceUnit (
			UINT8 DistanceUnit
			);

		//! Set geometric table parameters.
		void SetGeoTable (
			const RVC::GEOTABLE& GeoTable
			);

		//! Set merge option.
		void SetMerge (
			const MERGE merge
			) { m_Merge = merge; }

		//! Set whether to merge adjoining (vector) polygons.
		void SetMergeAdjoiningPolygons (
			bool bMergeAdjoiningPolygons
			) { m_bMergeAdjoiningPolygons = bMergeAdjoiningPolygons; }

		//! Set to merge by attribute based on specified table and field.
		void SetMergeAttribute (
			const MISTRING& table,
			const MISTRING& field
			) { m_Merge = MERGE_ByAttribute; m_MergeTableName = table; m_MergeFieldName = field; }

		//! Set subset of elements to process.
		ERRVALUE SetProcessSet (
			const UNSIGNEDSET *pElemSet,
			ELEMTYPE ElemType
			);

		//! Set subset of elements to process.
		ERRVALUE SetProcessSet (
			const BITSET_UNOWNED& ElemBitSet,
			ELEMTYPE ElemType
			);

		//! Set whether to transfer attributes from source.
		void SetTransferAttributes (
			bool bTransferAttributes
			) { m_bTransferAttributes = bTransferAttributes; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class SOURCE_OBJECT;
		class SOURCE_GEOMETRIC;
		class SOURCE_TABLE;
		class SOURCE_REGION;
		class ENGINE;

		MISTRING m_DistanceScript;
		MISTRING m_DistanceTableName;
		MISTRING m_DistanceFieldName;
		MISTRING m_MergeTableName;
		MISTRING m_MergeFieldName;
		DOUBLE_ARRAY<double> m_Distances;
		UOM::UNIT m_DistanceUnit;
		ELEMSET m_ProcessSet;
		RVC::GEOTABLE *m_pGeoTable;
		COLOR m_ColorFill;
		BUFFERELEMS m_BufferElems;
		AREAMODE m_AreaMode;
		MERGE m_Merge;
		bool m_bTransferAttributes;
		bool m_bMergeAdjoiningPolygons;

		void ResetDistance ();

		BUFFER (const BUFFER&);
		BUFFER& operator= (const BUFFER&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(BUFFER::BUFFERELEMS)
#endif

//=====================================================================================================================

}  // end namespace OP
}  // end namespace RVC


#endif  // INC_RVC_OPBUFFER_H
