/**
 * \file rvc/opimagegeometricstatistics.h
 * \brief RVC::OP::IMAGE_GEOMETRIC_STATISTICS definition
 *
 * \if NODOC
 * $Id: opimagegeometricstatistics.h_v 1.7 2015/02/18 16:01:28 mju Exp $
 *
 * $Log: opimagegeometricstatistics.h_v $
 * Revision 1.7  2015/02/18 16:01:28  mju
 * Add setNumBinsGLCM.
 *
 * Revision 1.6  2014/09/02 20:47:50  mju
 * Add range/variance stats.
 *
 * Revision 1.5  2014/09/02 19:32:24  mju
 * Add glcm stats.
 *
 * Revision 1.4  2014/08/26 20:05:25  mju
 * Add getStatName and stat values for glcm.
 *
 * Revision 1.3  2014/08/21 16:09:55  mju
 * Use relative not normalized mean.
 *
 * Revision 1.2  2014/08/19 13:31:17  mju
 * Use objitem for output table locations.
 *
 * Revision 1.1  2014/08/07 17:10:30  mju
 * Initial revision
 *	\endif
**/

#ifndef	INC_RVC_IMAGEGEOMETRICSTATISTICS_H
#define	INC_RVC_IMAGEGEOMETRICSTATISTICS_H

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_RVC_OPIMAGEREGIONSTATISTICS_H
	#include <rvc/opimageregionstatistics.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_UNSIGNEDSET_H
	#include <mi32/unsignedset.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class MIJOB;
class XMLNODE;
namespace RVC {
	class DBASE_ELEMENT;
	}
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Compute raster properties for a geometric object.
class IMAGE_GEOMETRIC_STATISTICS {
	public:

		//! Statistics to compute.
		enum STATS {
			STAT_None =							0,
			STAT_Count =						0x000001,	//!< Cell count
			STAT_Minimum =						0x000002,
			STAT_Maximum =						0x000004,
			STAT_Range =						0x000008,
			STAT_Mean =							0x000010,
			STAT_Median =						0x000020,
			STAT_Mode =							0x000040,
			STAT_StdDev =						0x000080,
			STAT_Variance =					0x000100,
			STAT_RelStdDevPct =				0x000200,
			STAT_RelativeMean =				0x000400,
			STAT_GLCM_Contrast =				0x001000,
			STAT_GLCM_Dissimilarity =		0x002000,
			STAT_GLCM_Homogeneity =			0x004000,
			STAT_GLCM_ASM =					0x008000,	//!< Angular Second Moment
			STAT_GLCM_Entropy =				0x010000,
			STAT_GLCM_Mean =					0x020000,
			STAT_GLCM_Variance =				0x040000,
			STAT_GLCM_Correlation =			0x080000,
			STAT_Histogram =					0x100000,
			STAT_Value =						0x200000,	//!< Cell value for points
			STAT_MASK_GLCM =					0x0FF000,
			STAT_MASK_PointByElement =		STAT_Value,
			STAT_MASK_PointByAttribute =	0x0007FF,
			STAT_MASK_Line =					0x0007FF,
			STAT_MASK_Polygon =				0x1FF7FF,
			STATS_AllNoHistogram =			0x0FF7FF
			};

		//! Get name string for statistic.
		static MISTRING GetStatName (
			STATS stat,
			bool bAbbr = false
			);

		//! Install SML interface.
		static void SML_Install (
			SML::CONTEXT *context
			);

		//! Constructor.
		IMAGE_GEOMETRIC_STATISTICS ();

		//! Destructor.
		~IMAGE_GEOMETRIC_STATISTICS ();

		//! Add image to process.
		//! If image is multiband or hyperspectral, the band name/number will be appended to corresponding field names.
		//! To specify table in text file, leave name blank and set description to encoded filepath.
		ERRVALUE AddObjImage (
			const OBJITEM& ObjItemImage,
			const OBJITEM *pObjItemTableByElementStat = 0,			//!< Location of output table for statistics by element
			const OBJITEM *pObjItemTableByElementHistogram = 0,	//!< Location of output table for histogram by element
			const OBJITEM *pObjItemTableByAttributeStat = 0,		//!< Location of output table for statistics by attribute
			const OBJITEM *pObjItemTableByAttributeHistogram = 0	//!< Location of output table for histogram by attribute
			);

		//! Read all parameters from job file.
		ERRVALUE JobRead (
			MIJOB& job
			);

		//! Write parameters to job file.
		ERRVALUE JobWrite (
			MIJOB& job
			);

		//! Run the operation.
		ERRVALUE Run ();

		//! Set whether to allow spaces in field names.
		void SetAllowSpaceInFieldName (
			bool bAllowSpaceInFieldName
			) { m_bAllowSpaceInFieldName = bAllowSpaceInFieldName; }

		//! Set how boundary cells are handled.
		void SetBoundaryCells (
			IMAGE_REGION_STATISTICS::BOUNDARYCELLS BoundaryCells
			) { m_BoundaryCells = BoundaryCells; }
		
		//! Set field for element id.
		void SetElementIdField (
			const MISTRING& TableName,
			const MISTRING& FieldName
			) { m_ElementIdTableName = TableName; m_ElementIdFieldName = FieldName; }

		//! Set element id to use element number.
		void SetElementIdUseElemNum ();

		//! Set number of bins for GLCM.
		//! Will use 16 bins if not specified.
		void SetNumBinsGLCM (
			UINT16 NumBins								//!< Number of bins (2-64) in each direction
			) { m_NumBinsGLCM = NumBins; }

		//! Set geometric object and general element type to use.
		ERRVALUE SetObjGeometric (
			const RVC::OBJITEM& ObjItem,		//!< Geometric object location
			ELEMTYPE ElemType,					//!< General element type (Point, Line or Polygon)
			const UNSIGNEDSET *pElemSet = 0	//!< Set of elements to process, NULL for all
			);

		//! Set to copy geometric object to specified location before adding tables.
		void SetObjItemCopyTo (
			const RVC::OBJITEM& ObjItemCopyTo
			) { m_ObjItemCopyTo = ObjItemCopyTo; m_Target = TARGET_Copy; }

		//! Set output table name / location with fields for each raster when summarize by attribute.
		void SetOutputTableByAttribute (
			const RVC::OBJITEM& descriptor	//!< Location of table
			) { m_ObjItemTableByAttribute = descriptor; }

		//! Set output table name / location with fields for each raster when summarize by element.
		void SetOutputTableByElement (
			const RVC::OBJITEM& descriptor	//!< Location of table
			) { m_ObjItemTableByElement = descriptor; }

		//! Set which statistics to compute by attribute.
		void SetStatsByAttribute (
			STATS stats									//!< Statistics to compute, STAT_None if not computing by attribute
			) { m_StatsByAttribute = stats; }

		//! Set which statistics to compute by element.
		void SetStatsByElement (
			STATS stats									//!< Statistics to compute, STAT_None if not computing by element
			) { m_StatsByElement = stats; }

		//! Set field for summary by attribute.
		void SetSummaryAttribute (
			const MISTRING& TableName,
			const MISTRING& FieldName
			) { m_SummaryTableName = TableName; m_SummaryFieldName = FieldName; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ENGINE;

		enum TARGET {
			TARGET_Existing,
			TARGET_Copy,
			TARGET_Text
			};

		class IMAGEITEM {
			public:
				IMAGEITEM ();
				IMAGEITEM (
					const OBJITEM& ObjItemImage,
					const OBJITEM *pObjItemTableByElementStat,
					const OBJITEM *pObjItemTableByElementHistogram,
					const OBJITEM *pObjItemTableByAttributeStat,
					const OBJITEM *pObjItemTableByAttributeHistogram
					);
				~IMAGEITEM ();

				const OBJITEM& GetObjItemImage () const
					{ return (m_ObjItemImage); }

				const OBJITEM& GetObjItemTableByAttributeHistogram () const
					{ return (m_ObjItemTableByAttributeHistogram); }

				const OBJITEM& GetObjItemTableByAttributeStat () const
					{ return (m_ObjItemTableByAttributeStat); }

				const OBJITEM& GetObjItemTableByElementHistogram () const
					{ return (m_ObjItemTableByElementHistogram); }

				const OBJITEM& GetObjItemTableByElementStat () const
					{ return (m_ObjItemTableByElementStat); }

				void GetXml (XMLNODE *node) const;

				ERRVALUE SetXml (const XMLNODE *node);

			private:
				OBJITEM m_ObjItemImage;
				OBJITEM m_ObjItemTableByElementStat;
				OBJITEM m_ObjItemTableByElementHistogram;
				OBJITEM m_ObjItemTableByAttributeStat;
				OBJITEM m_ObjItemTableByAttributeHistogram;
			};
		typedef MILIST<IMAGEITEM> IMAGEITEMLIST;

		RVC::OBJITEM m_ObjItemGeometric;
		RVC::OBJITEM m_ObjItemCopyTo;
		UNSIGNEDSET m_ElemSetGeometric;
		MISTRING m_ElementIdTableName;
		MISTRING m_ElementIdFieldName;
		MISTRING m_SummaryTableName;
		MISTRING m_SummaryFieldName;
		OBJITEM m_ObjItemTableByElement;
		OBJITEM m_ObjItemTableByAttribute;
		IMAGEITEMLIST m_ImageItemList;
		ELEMTYPE m_ElemType;
		TARGET m_Target;
		IMAGE_REGION_STATISTICS::BOUNDARYCELLS m_BoundaryCells;
		STATS m_StatsByElement;
		STATS m_StatsByAttribute;
		UINT16 m_NumBinsGLCM;
		bool m_bAllowSpaceInFieldName;
		bool m_bMultiImageFields;
	
		static void SML_Compute (int numargs, SML::ARG *args, UINT8 *argtypes, SML::CONTEXT *context);

		ERRVALUE CheckSettings ();
		ERRVALUE RunSML (SML::CONTEXT *context, SML::ARG *args, int numargs);

		IMAGE_GEOMETRIC_STATISTICS (const IMAGE_GEOMETRIC_STATISTICS&);
		IMAGE_GEOMETRIC_STATISTICS& operator= (const IMAGE_GEOMETRIC_STATISTICS&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(IMAGE_GEOMETRIC_STATISTICS::STATS)
#endif

//=====================================================================================================================

} // End OP namespace
} // End RVC namespace

#endif	// INC_RVC_IMAGEGEOMETRICSTATISTICS_H
