/**
 * \file rvc/opimagesetnullcells.h
 * \brief RVC::OP::IMAGE_SETNULLCELLS definition
 *
 * \if NODOC
 * $Id: opimagesetnullcells.h_v 1.1 2015/01/26 15:23:54 mju Exp $
 *
 * $Log: opimagesetnullcells.h_v $
 * Revision 1.1  2015/01/26 15:23:54  mju
 * Initial revision
 *
 *	\endif
**/

#ifndef	INC_RVC_IMAGESETNULLCELLS_H
#define	INC_RVC_IMAGESETNULLCELLS_H

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_RVC_IMAGE_H
	#include <rvc/image.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_ANYRASTV_H
	#include <mi32/anyrastv.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class MIJOB;
class XMLNODE;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Operation to set null cells of image.
//! Null cells can be set by specific values, ranges and using a mask. The specified image can be processed in-place
//! or a new image created. Existing null cells can be left as null or treated as "valid", and all new null cells can
//! be changed to a specified value or marked as null via the null mask. The operation automatically creates a null
//! mask or sets a null value depending on the output.
class IMAGE_SETNULLCELLS {
	public:

		//! Install SML interface.
		static void SML_Install (
			SML::CONTEXT *context
			);

		//! Constructor.
		IMAGE_SETNULLCELLS ();

		//! Destructor.
		~IMAGE_SETNULLCELLS ();

		//! Add range.
		void AddRange (
			double minValue,					//!< Minimum value in range
			double maxValue,					//!< Maximum value in range
			bool bMinInclusive,				//!< Whether minimum is inclusive
			bool bMaxInclusive,				//!< Whether maximum is inclusive
			bool bInsideIsValid				//!< Whether values "inside" range are valid (true) or null (false)
			);

		//! Add single numeric value.
		//! If 'bUseForOutput' is false, only the raster definition and null mask will be updated, the actual values
		//! stored with the raster will remain unchanged. If 'bUseForOutput' is true, the stored raster cell values
		//! will be updated such that any cells that are made null (or are null already) will be changed to the
		//! specified value, and any existing null mask will then be removed.. Note that only the last value having
		//! bUseForOutput=true specified will be used.
		void AddValue (
			double value,						//!< Value to add
			bool bUseForOutput = false		//!< If true, all null cells in output will be changed to this value
			);

		//! Add single value of general type.
		void AddValue (
			const ANYRASTVALUE &value,		//!< Value to add
			bool bUseForOutput = false		//!< If true, all null cells in output will be changed to this value
			);

		//! Read all parameters from job file.
		ERRVALUE JobRead (
			MIJOB &job
			);

		//! Write parameters to job file.
		ERRVALUE JobWrite (
			MIJOB &job
			);

		//! Run the operation.
		ERRVALUE Run ();

		//! Set whether to treat existing null cells as valid instead of null
		//! By default, existing null cells are considered to be null.
		void SetExistingNullsAsValid (
			bool bExistingNullsAsValid = true		//!< Whether existing null cells treated as valid instead of null
			) { m_bExistingNullsAsValid = bExistingNullsAsValid; }

		//! Set image raster to process.
		ERRVALUE SetObjItemImage (
			const OBJITEM &ObjItem			//!< Image raster location
			);

		//! Set optional mask raster to use.
		ERRVALUE SetObjItemMask (
			const OBJITEM &ObjItem			//!< Mask raster location
			);

		//! Set output raster location.
		//! If no output location is specified will process raster in place.
		ERRVALUE SetObjItemOutput (
			const OBJITEM &ObjItem			//!< Output raster location
			);

		//! Set mode for computing pyramid tiers.
		void SetPyramidMode (
			IMAGE::PYRAMID PyramidMode
			) { m_PyramidMode = PyramidMode; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ENGINE;

		class RANGE {
			public:

				RANGE ();
				RANGE (double minValue, double maxValue, bool bMinInclusive, bool bMaxInclusive, bool bInsideIsValid);

				bool ContainsValue (double value) const;

				void GetXml (XMLNODE *node) const;

				bool InsideIsValid () const
					{ return (m_bInsideIsValid); }

				ERRVALUE SetXml (const XMLNODE *node);

			private:
				double m_minValue;
				double m_maxValue;
				bool m_bMinInclusive;
				bool m_bMaxInclusive;
				bool m_bInsideIsValid;
			};
		typedef MILIST<RANGE> RANGELIST;

		OBJITEM m_ObjItemImage;
		OBJITEM m_ObjItemMask;
		OBJITEM m_ObjItemOutput;
		RANGELIST m_RangeList;
		SIMPLE_ARRAY<double> m_ValuesDouble;
		SIMPLE_ARRAY<ANYRASTVALUE> m_ValuesAny;
		double m_OutputValueDouble;
		ANYRASTVALUE m_OutputValueAny;
		IMAGE::CELLTYPE m_CellType;
		IMAGE::PYRAMID m_PyramidMode;
		bool m_bExistingNullsAsValid;
		bool m_bUseOutputValueDouble;
		bool m_bUseOutputValueAny;
		bool m_bInPlace;
	
		ERRVALUE CheckSettings ();

		IMAGE_SETNULLCELLS (const IMAGE_SETNULLCELLS&);
		IMAGE_SETNULLCELLS& operator= (const IMAGE_SETNULLCELLS&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

} // End OP namespace
} // End RVC namespace

#endif	// INC_RVC_IMAGESETNULLCELLS_H
