/**
 * \file rvc/oprecordpopulate.h
 * \brief RVC::OP::RECORDPOPULATE definitions
 *
 * \if NODOC
 * $Id: oprecordpopulate.h_v 1.5 2015/04/30 13:50:00 mju Exp $
 *
 * $Log: oprecordpopulate.h_v $
 * Revision 1.5  2015/04/30 13:50:00  mju
 * Use elemidx.
 *
 * Revision 1.4  2015/04/30 13:36:37  mju
 * Use elemidx hdr.
 *
 * Revision 1.3  2014/12/22 21:48:50  mju
 * Add method_Sequential.
 *
 * Revision 1.2  2013/12/20 17:58:03  bcolombe
 * Support null or zero
 *
 * Revision 1.1  2013/11/22 22:33:52  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_OPRECORDPOPULATE_H
#define	INC_RVC_OPRECORDPOPULATE_H

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

#ifndef	INC_RVC_ELEMIDX_H
#include <rvc/elemidx.h>
#endif

#ifndef	INC_VECTOR
#include <vector>
#define	INC_VECTOR
#endif


namespace RVC {
namespace OP {

//=====================================================================================================================

//! Service for populating records in database table.
class RECORDPOPULATE {
	public:

		//! Conditions for when to populate field.
		enum CONDITION {
			CONDITION_Never =			0x0000,			//!< Never populate field
			CONDITION_Null =			0x0001,	//!< If field is empty or null
			CONDITION_Zero =			0x0002,	//!< If field value is zero (numeric)
			CONDITION_Always=			0x8000	//!< Always populate field, all other flags ignored if specified
			};

		//! Method for populating field.
		enum METHOD {
			METHOD_Unspecified,
			METHOD_Value,			//!< Use specified value
			METHOD_Field,			//!< Use field in related (or same) table
			METHOD_Sequential,	//!< Sequential (for auto-increment field with no value set)
//			METHOD_Script,
			};

		//! Results from populating record.
		enum RESULT {
			RESULT_NoChange =		0x00,			//!< Record was not changed
			RESULT_Changed =		0x01,			//!< Record was changed
			RESULT_NoRelated =	0x02,			//!< No related record found when METHOD_Field used
			RESULT_Mixed =			(RESULT_Changed | RESULT_NoRelated)
			};

		//! Parameters for populating field.
		class FIELDPARM {
			public:

				//! Constructor.
				FIELDPARM ();

				//! Destructor.
				~FIELDPARM ();

				CONDITION GetCondition () const
					{ return (m_condition); }

				METHOD GetMethod () const
					{ return (m_method); }

				UINT32 GetSrcFieldNum () const
					{ return (m_SrcField); }

				INT32 GetSrcTableNum () const
					{ return (m_SrcTable); }

				const DBTABLE::FIELDVALUE GetValue () const
					{ return (m_value); }

				void SetCondition (
					CONDITION condition
					) { m_condition = condition; }

				void SetMethod (
					METHOD method
					) { m_method = method; }

				void SetSrcTableField (
					INT32 TableNum,
					UINT32 FieldNum
					) { m_SrcTable = TableNum; m_SrcField = FieldNum; }

				void SetValue (
					const DBTABLE::FIELDVALUE& value
					) { m_value = value; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				CONDITION m_condition;
				METHOD m_method;
				INT32 m_SrcTable;
				UINT32 m_SrcField;
				DBTABLE::FIELDVALUE m_value;
				#endif
			};

		//! Constructor.
		RECORDPOPULATE ();

		//! Destructor.
		~RECORDPOPULATE ();

		//! Determine if specified field can be populated.
		bool CanPopulateField (
			UINT32 fieldnum
			) const;

		//! Get label for specified condition and field.
		TEXTID GetConditionLabel (
			UINT32 fieldnum,
			CONDITION condition
			) const;

		//! Get label for current condition of specified field.
		TEXTID GetConditionLabel (
			UINT32 fieldnum
			) const { return (GetConditionLabel(fieldnum,m_FieldParms[fieldnum].GetCondition())); }

		const DBTABLE::FIELDINFO& GetFieldInfo (
			UINT32 fieldnum
			) const { return (m_FieldInfo[fieldnum]); }

		//! Get parameters for specified field number.
		const FIELDPARM& GetFieldParm (
			UINT32 fieldnum
			) const { return (m_FieldParms[fieldnum]); }

		//! Get label for specified method.
		TEXTID GetMethodLabel (
			METHOD method
			) const;

		//! Get number of fields in table.
		UINT32 GetNumFields () const
			{ return (m_FieldParms.size()); }

		//! Get object for table.
		const DBTABLE& GetObjTable () const
			{ return (m_ObjTable); }

		//! Determine if any condition other than CONDITION_Never set.
		bool HasAnyCondition () const;

		//! Initialize and setup defaults for all fields.
		//! Must be called before using any other methods.
		ERRVALUE Initialize (
			const DBTABLE& ObjTable
			);

		//! Begin processing operation, must be called before ProcessRecord.
		ERRVALUE ProcessBegin ();

		//! End processing operation.
		void ProcessEnd ();

		//! Process specified record using element for relation.
		//! @return RESULT value or error < 0.
		INT32 ProcessRecord (
			DBTABLE::RECORD& record,				//!< Record passed/returned
			const DBTABLE::BOOKMARK& bookmark	//!< Bookmark
			);

		//! Process specified record using element for relation.
		//! @return RESULT value or error < 0.
		INT32 ProcessRecord (
			DBTABLE::RECORD& record,				//!< Record passed/returned
			const RVC::ELEMIDX &elemidx			//!< Associated element
			);

		//! Set condition value for specified field.
		void SetFieldCondition (
			UINT32 fieldnum,
			CONDITION condition
			);

		//! Set method value for specified field.
		void SetFieldMethod (
			UINT32 fieldnum,
			METHOD method
			);

		//! Set parameters for specified field number.
		ERRVALUE SetFieldParm (
			UINT32 fieldnum,
			const FIELDPARM& FieldParm
			);

		//! Set source table/field for specified field.
		void SetFieldSrcTableField (
			UINT32 fieldnum,
			INT32 SrcTableNum,
			UINT32 SrcFieldNum
			);

		//! Set value for specified field.
		void SetFieldValue (
			UINT32 fieldnum,
			const DBTABLE::FIELDVALUE& value
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DBTABLE m_ObjTable;
		DBTABLE::FIELDINFOARRAY m_FieldInfo;
		std::vector<FIELDPARM> m_FieldParms;
		DBASE m_ObjDb;
		SIMPLE_ARRAY<bool> m_DidField;
		DBTABLE m_ObjTableRelated;
		SIMPLE_ARRAY<INT32> m_RecNumRelated;
		DBTABLE::RECORD m_RecRelated;
		SIMPLE_ARRAY<DBLISTELMT> m_dblist;
		ELEMTYPE m_ElemTypeRel;

		INT32 ProcessField (UINT32 fieldnum, DBTABLE::RECORD& record, const RVC::ELEMIDX &elemidx, bool bHaveRecRelated);

		RECORDPOPULATE (const RECORDPOPULATE&);
		RECORDPOPULATE& operator= (const RECORDPOPULATE&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(RECORDPOPULATE::CONDITION)
DEFINE_ENUM_OP_BITWISE(RECORDPOPULATE::RESULT)
#endif

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPRECORDPOPULATE_H
