/**
 * \file rvc/uiformobjectlist.h
 * \brief RVC::UI::FORM_OBJECT_LIST class definitions
 *
 * \if NODOC
 * $Id: uiformobjectlist.h_v 1.19 2015/07/09 21:35:37 mju Exp $
 *
 * $Log: uiformobjectlist.h_v $
 * Revision 1.19  2015/07/09 21:35:37  mju
 * Add fields for image scale, offset, units.
 * Add setFieldVisible.
 *
 * Revision 1.18  2015/03/20 14:46:38  mju
 * Add listfield_Image_Range.
 *
 * Revision 1.17  2015/02/12 20:35:43  mju
 * Add predefined list fields and methods to work with.
 * Add ability to create separate toolbar instead of buttons in form.
 * Change parameters for create and default to multi-selection.
 *
 * Revision 1.16  2015/02/09 15:31:03  mju
 * Use private class for internals.
 * Add getSelectedObjItemPtr.
 *
 * Revision 1.15  2014/12/29 18:07:33  mju
 * Doxygen exclusion for deprecated items.
 *
 * Revision 1.14  2014/09/12 14:38:20  mju
 * Add objitem parm to delegate on removed.
 *
 * Revision 1.13  2014/07/14 19:32:57  mju
 * Add setFilter.
 *
 * Revision 1.12  2013/06/24 14:53:29  mju
 * In onBtnRemove handle multiple selection.
 * Don't force item to be selected in multi-selection mode.
 *
 * Revision 1.11  2013/06/21 21:38:51  mju
 * Add formstyle to allow multi-selection.
 *
 * Revision 1.9  2013/05/09 15:07:14  mju
 * Add create method with pointer to filter to take ownership.
 *
 * Revision 1.8  2013/05/09 14:51:55  mju
 * Support mgui::dlg_getobject_filter and deprecate use of rvc::getobjfilter.
 *
 * Revision 1.7  2012/07/27 14:03:25  mju
 * Add getObjTypeSet.
 *
 * Revision 1.4  2007/05/16 21:38:23  scowan
 * Added setObjectList method that only works before calling create method.
 *
 * Revision 1.1  2006/07/21 21:19:41  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMOBJECTLIST_H
#define	INC_RVC_UIFORMOBJECTLIST_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
#include <mgui/dlggetobjectfilter.h>
#endif

#ifndef	INC_MGUI_DLGGETOBJECTSTYLE_H
#include <mgui/dlggetobjectstyle.h>
#endif

#ifndef	INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

#ifndef	INC_RVC_OTYPESET_H
#include <rvc/otypeset.h>
#endif

#ifndef	INC_RVC_GETOBJFLAGS_H
#include <rvc/getobjflags.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace MGUI {
	class CTRL_TOOLBAR;
	class FORM_LIST;
	}
namespace RVC {
	class GETOBJFILTER;
	}
#endif

namespace RVC {
namespace UI {
	
//===================================================================================================================

//! Form containing basic object selection list with "add", "remove", and "remove all" buttons.
class FORM_OBJECT_LIST : public MGUI::FORM_COMPOSITE {
	public:

		//! Style for form creation.
		enum FORMSTYLE {
			FORMSTYLE_ListSelectMulti =		0x0000,	//!< Allow multiple items to be highlighted (default)
			FORMSTYLE_ListSelectSingle =		0x0001,	//!< Allow single item to be highlighted (typically used if have object-specific settings)
			FORMSTYLE_NoDftFields =				0x0002,	//!< Don't create default list fields based on allowed object type(s)
			FORMSTYLE_NoButtons =				0x0004,	//!< Don't create add/remove buttons (use if will call CreateToolbar or AddToolbarButtons)
			FORMSTYLE_NoShowHideFields =		0x0008,	//!< Don't allow user to show/hide fields
			};

		//! Predefined list fields.
		enum LISTFIELD {
			LISTFIELD_None =						0,
			LISTFIELD_Created =					0x00000001,
			LISTFIELD_Modified =					0x00000002,
			LISTFIELD_DataType =					0x00000010,	//!< Image cell type, geometric point type and topology
			LISTFIELD_CRS =						0x00000020,
			LISTFIELD_ExtentsX =					0x00000040,
			LISTFIELD_ExtentsY =					0x00000080,
			LISTFIELD_Extents =					LISTFIELD_ExtentsX | LISTFIELD_ExtentsY,
			LISTFIELD_Image_Dimensions =		0x00001000,
			LISTFIELD_Image_CellSize =			0x00002000,
			LISTFIELD_Image_NullSpec =			0x00004000,
			LISTFIELD_Image_Range =				0x00008000,
			LISTFIELD_Image_Scale =				0x00010000,
			LISTFIELD_Image_Offset =			0x00020000,
			LISTFIELD_Image_Units =				0x00040000,
			LISTFIELD_Geometric_Elements =	0x00100000,
			LISTFIELD_Geometric_Points =		0x00200000,
			LISTFIELD_Geometric_Nodes =		0x00400000,
			LISTFIELD_Geometric_Lines =		0x00800000,
			LISTFIELD_Geometric_Polygons =	0x01000000,
			LISTFIELD_Geometric_Labels =		0x02000000,
			LISTFIELD_MASK_Image =				0x000FF000 | LISTFIELD_DataType,
			LISTFIELD_MASK_Geometric =			0x0FF00000 | LISTFIELD_DataType,
			LISTFIELD_MASK_Geospatial = 		0x0FFFFFF0,
			LISTFIELD_MASK_NeedGeoref =		LISTFIELD_CRS | LISTFIELD_Extents | LISTFIELD_Image_CellSize,
			};

		//!< Delegate specifying OBJITEM and position in list.
		typedef FastDelegate<void(const RVC::OBJITEM&, INT32)> DELEGATE_OBJITEM_LISTPOS;
	
		//! Constructor.
		FORM_OBJECT_LIST ();

		//! Destructor.
		virtual ~FORM_OBJECT_LIST ();

		//! Access FORM_LIST.
		//! Should only be used to add custom fields. Do not set delegates on the list, only set delegates on specific fields created.
		MGUI::FORM_LIST& AccessList ();

		//! Add default fields based on allowed object types.
		void AddDftFields ();

		//! Add predefined field with application-specific label.
		//! If field has already been added, its label will be changed.
		//! Added field will always be visible.
		void AddField (
			LISTFIELD field,					//!< Field to add
			const MISTRING &label			//!< Label
			);

		//! Add predefined fields.
		//! May be called multiple times to add fields in specific order.
		void AddFields (
			LISTFIELD fields,					//!< Fields to add
			bool bForceVisible = false		//!< Force fields to be visible, if true user will not have option to hide specified fields
			);
			
		//! Add buttons to specified toolbar.
		void AddToolbarButtons (
			MGUI::CTRL_TOOLBAR &toolbar,						//!< Toolbar 
			const MISTRING &TypeName							//!< Type name for objects being added, should generally be plural
			);

		//! Clears the list.
		void Clear (
			bool notify							//!< Send notification
			);

		//! Create form with filter reference.
		//! The Ini key used to save field visibility should be specified for library implementations or if application creates multiple lists.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,				//!< Parent pane
			const MISTRING &promptstr,							//!< Prompt string to use with DlgGetObjects
			const MGUI::DLG_GETOBJECT_FILTER &filter,		//!< Object selection filter reference (MUST REMAIN VALID)
			FORMSTYLE FormStyle = FORMSTYLE_ListSelectMulti,	//!< Form style
			int ListHeight = 5,									//!< Initial list box height
			MGUI::DLG_GETOBJECT_STYLE DlgStyle = MGUI::DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Flags for DlgGetObjects
			const char *IniKey = 0,								//!< Ini key for saving field visibility
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Create form and take ownership of filter.
		//! The form will take ownership of the specified filter pointer and the value will be returned as NULL.
		//! The Ini key used to save field visibility should be specified for library implementations or if application creates multiple lists.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,				//!< Parent pane
			const MISTRING &promptstr,							//!< Prompt string to use with DlgGetObjects
			MGUI::DLG_GETOBJECT_FILTER *& pFilter,			//!< Object selection filter pointer (must not be NULL)
			FORMSTYLE FormStyle = FORMSTYLE_ListSelectMulti,	//!< Form style
			int ListHeight = 5,									//!< Initial list box height
			MGUI::DLG_GETOBJECT_STYLE DlgStyle = MGUI::DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Flags for DlgGetObjects
			const char *IniKey = 0,								//!< Ini key for saving field visibility
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Create form with legacy filter.
		DEPRECATED ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,				//!< Parent pane
			const MISTRING &promptstr,							//!< Prompt string to use with DlgGetObjects
			const RVC::OBJTYPESET &ObjTypeSet,				//!< Set of object types that are selectable
			GETOBJFLAGS GetObjFlags = GETOBJFLAG_None,	//!< Flags for DlgGetObjects
			RVC::GETOBJFILTER* GetObjFilter = 0,			//!< Filter for DlgGetObjects
			int ListHeight = 5,									//!< Initial list box height
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);
		#endif

		//! Create toolbar.
		void CreateToolbar (
			MGUI::LAYOUT_PANE_BASE &pane,						//!< Pane to contain toolbar
			const MISTRING &TypeName							//!< Type name for objects being added, should generally be plural
			);

		//! Get pane containing standard buttons.
		//! Do not use if FORMSTYLE_NoButtons was specified on creation.
		MGUI::LAYOUT_PANE& GetBtnPane ();

		//! Get pointer to OBJITEM corresponding to list item number.
		//! @return Pointer to OBJITEM, NULL if none corresponding.
		//! Intended for use by custom field delegates.
		const RVC::OBJITEM* GetObjItemFromItemNum (
			int itemnum												//!< List item number (not necessarily position in OBJITEMLIST)
			) const;
			
		//! Get the list of objects in the list			
		const RVC::OBJITEMLIST& GetObjectList () const;

		//! Get set of allowed object types.
		const RVC::OBJTYPESET& GetObjTypeSet () const;

		//! Get OBJITEM* for current (highlighted) item.
		//! @return Pointer to OBJITEM or NULL if there is not exactly one item highlighted.
		const RVC::OBJITEM* GetSelectedObjItemPtr () const;

		//! Get list of objects highlighted.
		void GetSelectedObjItemPtrList (
			MILIST<const RVC::OBJITEM*> &ObjItemPtrList	//!< List returned
			) const;

		//! Is the list empty?			
		bool IsEmpty () const;

		//! Select objects via form.
		//! Behaves same as when 'Select' button pressed.
		ERRVALUE Select (
			bool notify = true						//!< Send notification if list changed
			);
			
		//! Set delegate to call for any change to objects in list.
		void SetDelegateOnListChanged (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call for each object added to list.
		void SetDelegateOnObjectAdded (
			DELEGATE_OBJITEM_LISTPOS delegate
			);

		//! Set delegate to call when single object removed from list.
		void SetDelegateOnObjectRemoved (
			DELEGATE_OBJITEM_LISTPOS delegate
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Set delegate to call when single object selected (highlighted) in list.
		DEPRECATED_MSG("Use SetDelegateOnSelection") void SetDelegateOnObjectSelected (
			DELEGATE_OBJITEM_LISTPOS delegate
			);
		#endif

		//! Set delegate to call when objects added to list.
		void SetDelegateOnObjectsAdded (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when all objects removed from list.
		void SetDelegateOnObjectsAllRemoved (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when items highlighted in list.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate		//!< Delegate
			);

		//! Set whether field is visible.
		void SetFieldVisible (
			LISTFIELD field,							//!< Field to set
			bool bVisible								//!< Whether to make visible
			);

		//! Set new filter.
		//! Form will take ownership of new filter and value will be returned as NULL. Any currently selected object not allowed
		//! by the new filter will be removed from the list. The additional parameters affect whether the user will be prompted
		//! for a new selection and/or whether to call the delegate to notify. If the user cancels the selection (or is not prompted)
		//! the existing selection will be cleared. If no object is currently selected, the user will not be prompted and the delegate
		//! will not be called.
		void SetFilter (
			MGUI::DLG_GETOBJECT_FILTER *& pFilter,			//!< Object selection filter pointer (must not be NULL)
			bool bSelectIfNoCurrentAllowed,					//!< Reselect if no current object is allowed
			bool bNotifyIfChanged								//!< Notify if selection list changed
			);

		//! Set the list of objects in the list
		//! Must be called before Create() method.
		void SetObjectList (
			const RVC::OBJITEMLIST &ObjItemList
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_OBJECT_LIST (const FORM_OBJECT_LIST&);
		FORM_OBJECT_LIST& operator= (const FORM_OBJECT_LIST&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_OBJECT_LIST::FORMSTYLE)
DEFINE_ENUM_OP_BITWISE(FORM_OBJECT_LIST::LISTFIELD)
#endif
	
//===================================================================================================================

}		// End namespace UI
}		// End namespace RVC

#endif	// INC_RVC_UIFORMOBJECTLIST_H
