/**
 * \file rvc/uiimagemultibandextractor.h
 * \brief RVC::UI::IMAGE_MULTIBAND_EXTRACTOR
 *
 * \if NODOC
 * $Id: uiimagemultibandextractor.h_v 1.1 2015/02/17 20:18:16 mju Exp $
 *
 * $Log: uiimagemultibandextractor.h_v $
 * Revision 1.1  2015/02/17 20:18:16  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIIMAGEMULTIBANDEXTRACTOR_H
#define	INC_RVC_UIIMAGEMULTIBANDEXTRACTOR_H

#ifndef	INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================
//		IMAGE_MULTIBAND_EXTRACTOR definition - Begin

//! Helper to extract OBJITEM sets of coincident image bands from OBJITEMLIST.
class IMAGE_MULTIBAND_EXTRACTOR {
	public:

		enum FLAGS {
			FLAG_Default =					0,
			FLAG_Strict =					0x01,		//!< Return error if any non-suitable item found
			FLAG_AllowPalette =			0x02,		//!< Allow rasters with color palette subobject (implied by FLAG_Strict)
			FLAG_NeedSameCellType =		0x04,		//!< Require all rasters to have the same cell type
			FLAG_ReturnNumSingle =		0x10,		//!< Return number of rasters to be processed singly instead of calling DelegateOnSingle
			// Note, flag values must match any corresponding IMAGEMULTIBAND::FLAGS
			};

		//! Delegate called when multiband image used.
		//! Parameters are:
		//! UINT32 SetId, selection from allowed set IDs.
		//! List of all image bands in set, in original order chosen by user.
		//! Index into list for each component band in set.
		typedef fastdelegate::FastDelegate<ERRVALUE(UINT32, const OBJITEMLIST &, const INT16 *)> DELEGATE_ONMULTI;

		//! Delegate called when specific band set used and remaining bands not included.
		//! Parameters are:
		//! UINT32 SetId, selection from allowed set IDs.
		//! List of all image bands in set, in position order for set type.
		typedef fastdelegate::FastDelegate<ERRVALUE(UINT32, const OBJITEMLIST &)> DELEGATE_ONSET;

		//! Delegate called when single band used.
		typedef fastdelegate::FastDelegate<ERRVALUE(const OBJITEM &)> DELEGATE_ONSINGLE;

		//! Constructor.
		IMAGE_MULTIBAND_EXTRACTOR ();

		//! Destructor.
		~IMAGE_MULTIBAND_EXTRACTOR ();

		//! Add bandset definition.
		ERRVALUE AddBandSet (
			UINT32 SetId,							//!< Application-specific set ID, must be unique or error will occur
			const MISTRING &SetName,			//!< Set name
			int NumComponents,					//!< Number of components in set, must be > 1
			const TEXTID *ComponentNames		//!< Names of components in set
			);

		//! Clear previous image definition history.
		void ClearHistory ();

		//! Clear all set definitions and image definition history.
		void ClearSets ();

		//! Process list, calling assigned delegates until completion.
		//! Depending user selections, multiple dialogs may appear in succession.
		//! Processing will continue until the list is empty, a non-raster object is encountered, or an error occurs.
		ERRVALUE Process (
			MDLGPARENT dlgparent,						//!< Parent for dialog
			RVC::OBJITEMLIST &ObjItemListInput,		//!< List to process, items processed will be removed
			RVC::OBJITEMLIST::ITERATOR &itInput		//!< Iterator for first input to process, passed/returned
			);

		//! Set delegate to call for multi-band image with all bands included.
		void SetDelegateOnMulti (
			DELEGATE_ONMULTI delegate
			);

		//! Set delegate to call for exact set of band.
		void SetDelegateOnSet (
			DELEGATE_ONSET delegate
			);

		//! Set delegate to call for single-band image.
		void SetDelegateOnSingle (
			DELEGATE_ONSINGLE delegate
			);

		//! Set multiband image restrictions.
		void SetFlags (
			FLAGS flags
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		IMAGE_MULTIBAND_EXTRACTOR (const IMAGE_MULTIBAND_EXTRACTOR &);
		IMAGE_MULTIBAND_EXTRACTOR& operator= (const IMAGE_MULTIBAND_EXTRACTOR &);
		#endif
	};

//		IMAGE_MULTIBAND_EXTRACTOR definition - End
//=====================================================================================================================

} // End namespace UI
} // End namespace RVC

#endif	// INC_RVC_UIIMAGEMULTIBANDEXTRACTOR_H
