/**
 * \file rvc/uistyle.h
 * \brief RVC User Interface controls for working with styles.
 *
 * \if NODOC
 * $Id: uistyle.h_v 1.12 2015/07/17 21:23:06 mju Exp $
 *
 * $Log: uistyle.h_v $
 * Revision 1.12  2015/07/17 21:23:06  mju
 * Add styletype parm for dlgEditPattern.
 *
 * Revision 1.11  2015/03/20 22:45:07  bcolombe
 * Allow specifying active stylenum
 *
 * Revision 1.10  2014/01/21 13:08:25  mju
 * Add editstyleflag_AllowScaleFixed.
 *
 * Revision 1.9  2013/12/19 17:32:37  mju
 * Add editstyleflag_NoFrame.
 *
 * Revision 1.8  2013/11/15 17:24:25  mju
 * Move form_edit_style to own hdr.
 *
 * Revision 1.3  2007/02/13 18:11:44  dwilliss
 * changed parameters to icon selection dialog
 *
 * Revision 1.1  2006/09/20 15:10:58  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UISTYLE_H
#define	INC_RVC_UISTYLE_H

#ifndef	INC_RVC_STYLETYPE_H
#include <rvc/styletype.h>
#endif

#ifndef	INC_RVC_OBJECTNAME_H
#include <rvc/objectname.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class MISTRING;
struct COLOR;
struct POINTSTYLE;
struct LINESTYLE;
struct MULTISTYLE;
struct POLYSTYLE;
struct TEXTSTYLE;
namespace RVC {
	class OBJITEM;
	class STYLE;
	}
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Enumeration of flags for style editor forms and dialogs
enum EDITSTYLEFLAGS {
	EDITSTYLEFLAG_None =							0x00000000,	//!< No flags specified
	EDITSTYLEFLAG_DefaultOnly =				0x00000001,	//!< Only editing the "default" style
	EDITSTYLEFLAG_CAD =							0x00000002,	//!< Editing CAD styles
	EDITSTYLEFLAG_SingleType =					0x00000004,	//!< Only allow editing of specified style type
	EDITSTYLEFLAG_NoFrame =						0x00000008,	//!< Don't include frame (for DefaultOnly)
	EDITSTYLEFLAG_NoRenameStyles =			0x00000010,	//!< Don't allow styles to be renamed
	EDITSTYLEFLAG_NoNewDelStyles =			0x00000020,	//!< Don't allow styles to be created or deleted
	EDITSTYLEFLAG_NoElemHeight =				0x00000040,	//!< Don't include "Use element height" togglebutton
	EDITSTYLEFLAG_NoTypeButtons =				0x00000080,	//!< Don't include buttons for switching style type
	EDITSTYLEFLAG_NoScaleToLayout =			0x00000100,	//!< Don't allow scaling to "layout"
	EDITSTYLEFLAG_NoScaleOptions =			0x00000200,	//!< Don't give user options for scale-to
	EDITSTYLEFLAG_NoSample =					0x00000400,	//!< No not show "sample"
	EDITSTYLEFLAG_AllowScaleFixed =			0x00000800,	//!< Allow scaling of "fixed"
	EDITSTYLEFLAG_AllowScaleToNone =			0x00000800,	//!< Allow scaling to "none" (same as AllowScaleFixed)
	EDITSTYLEFLAG_SymbolForceSquare =		0x00010000,	//!< Force symbol "width" to match "height"
	EDITSTYLEFLAG_SymbolNoNone =				0x00020000,	//!< Don't allow "no symbol" option
	EDITSTYLEFLAG_TextAllowHTML =				0x00100000,	//!< Allow HTML text formatting options
	EDITSTYLEFLAG_TextAllowSpecialFormat =	0x00200000,	//!< Allow "Special Format" options for text styles
	EDITSTYLEFLAG_TextAllowVertAlignment =	0x00400000,	//!< Add option for text vertical alignment
	EDITSTYLEFLAG_TextForceWordWrap =		0x00800000,	//!< Force word wrap
	EDITSTYLEFLAG_TextNoBlockWidth =			0x01000000,	//!< Don't allow setting block width
	EDITSTYLEFLAG_TextNoWordWrap =			0x02000000,	//!< Disallow setting word wrap
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(EDITSTYLEFLAGS)
#endif

//=====================================================================================================================

//!\addtogroup DialogFunctions 
//!@{

//! Dialog allowing selection of single icon from style object.
//! Same as DlgGetPattern except that it assumes Bitmap patterns, and uses
//!	icons.ref instead of stdstyle.rvc for the "standard" styles.
//! @return Pattern index or error < 0
int DlgGetIcon (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const MISTRING &PromptStr,				//!< Prompt to show at top of dialog
	RVC::OBJITEM &StyleObjItem,			//!< Style object item the icon was selected from
	const COLOR &SampleColor				//!< "Variable" color for sample displays
	);

//! Dialog allowing selection of single pattern from style object.
//! @return Pattern index or error < 0
int DlgGetPattern (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const MISTRING &PromptStr,				//!< Prompt to show at top of dialog
	RVC::PATTERNTYPE PatternType,			//!< Type of pattern
	const RVC::OBJITEM &StyleObjItem,	//!< Style object item to select pattern from
	const COLOR &SampleColor				//!< "Variable" color for sample displays
	);

//! Dialog allowing selection of patterns from style object.
ERRVALUE DlgGetPatterns (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const MISTRING &PromptStr,				//!< Prompt to show at top of dialog
	RVC::PATTERNTYPE PatternType,			//!< Type of pattern
	RVC::OBJITEM &StyleObjItem,			//!< Style object item passed/returned
	SIMPLE_ARRAY<INT32> &PatternNums,	//!< Patterns selected
	const COLOR &SampleColor				//!< "Variable" color for sample displays
	);

//! Dialog to edit styles in style object.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &ObjItemStyle,	//!< Style object item
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags,					//!< Flags
	RVC::STYLETYPE StyleType = STYLETYPE_NumTypes,	//!< Type of style to edit, TYPE_NumTypes for all
	INT32 ActiveStyleNum = -1				//!< Initial active style number
	);

//! Dialog to edit point style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &ObjItemStyle,	//!< Style object item
	POINTSTYLE &DftStyle,					//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags,						//!< Flags
	INT32 ActiveStyleNum = -1				//!< Initial active style number
	);

//! Dialog to edit line style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &ObjItemStyle,	//!< Style object item
	LINESTYLE &DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags,						//!< Flags
	INT32 ActiveStyleNum = -1				//!< Initial active style number
	);

//! Dialog to edit polygon style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &ObjItemStyle,	//!< Style object item
	POLYSTYLE &DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags,						//!< Flags
	INT32 ActiveStyleNum = -1				//!< Initial active style number
	);

//! Dialog to edit text style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &ObjItemStyle,	//!< Style object item
	TEXTSTYLE &DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags,						//!< Flags
	INT32 ActiveStyleNum = -1				//!< Initial active style number
	);

//! Dialog to edit multi style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &ObjItemStyle,	//!< Style object item
	MULTISTYLE &DftStyle,					//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags,						//!< Flags
	RVC::STYLETYPE StyleType = STYLETYPE_NumTypes,	//!< Type of style to edit, TYPE_NumTypes for all
	INT32 ActiveStyleNum = -1				//!< Initial active style number
	);

//! Dialog allowing "pattern" in style object to be edited.
ERRVALUE DlgStyleEditPattern (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &StyleObjItem,	//!< Style object item
	RVC::PATTERNTYPE PatternType,			//!< Type of pattern
	INT32 PatternNum,							//!< Pattern number to edit, -1 for "new" pattern
	const COLOR &varcolor,					//!< "Variable" color for sample displays
	RVC::STYLETYPE StyleType = STYLETYPE_NumTypes	//!< Type of style pattern is associated with, TYPE_NumTypes for default
	);

//! Dialog allowing "pattern set" in style object to be edited.
ERRVALUE DlgStyleEditPatternSet (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &StyleObjItem,	//!< Style object item
	RVC::PATTERNTYPE PatternType,			//!< Type of pattern
	const COLOR &varcolor,					//!< "Variable" color for sample displays
	INT32 PatternNum = -1					//!< Default pattern number
	);

//! Dialog for editing 'style assignment' table for point style.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByAttribute (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &dbobjitem,		//!< Database object item reference
	ELEMTYPE elemtype,						//!< Element type
	RVC::OBJECTNAME &tablename,			//!< Style assignment table name passed/returned
	POINTSTYLE &DftStyle,					//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	const OBJITEM *pDftStyleObjItem = 0	//!< Default style object if any
	);

//! Dialog for editing 'style assignment' table for line style.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByAttribute (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &dbobjitem,		//!< Database object item reference
	ELEMTYPE elemtype,						//!< Element type
	RVC::OBJECTNAME &tablename,			//!< Style assignment table name passed/returned
	LINESTYLE &DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	const OBJITEM *pDftStyleObjItem = 0	//!< Default style object if any
	);

//! Dialog for editing 'style assignment' table for polygon style.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByAttribute (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &dbobjitem,		//!< Database object item reference
	ELEMTYPE elemtype,						//!< Element type
	RVC::OBJECTNAME &tablename,			//!< Style assignment table name passed/returned
	POLYSTYLE &DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	const OBJITEM *pDftStyleObjItem = 0	//!< Default style object if any
	);

//! Dialog for editing 'style assignment' table for MULTISTYLE.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByAttribute (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &dbobjitem,		//!< Database object item reference
	RVC::OBJECTNAME &tablename,			//!< Style assignment table name passed/returned
	MULTISTYLE &DftStyle,					//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	const OBJITEM *pDftStyleObjItem = 0,//!< Default style object if any
	RVC::STYLETYPE DftStyleType = RVC::STYLETYPE_NumTypes	//!< Default style type
	);

//! Dialog for editing 'style assignment' table for style by theme.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByTheme (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM &dbobjitem,		//!< Database object item reference
	ELEMTYPE elemtype,						//!< Element type
	RVC::OBJECTNAME &tablename				//!< Style assignment table name passed/returned
	);

//!@}

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UISTYLE_H
