/**
 * \file spatmod/imageautoregister.h
 * \brief SPATMOD::IMAGE::AUTOREGISTER definitions
 *
 * \if NODOC
 * $Id: imageautoregister.h_v 1.17 2016/05/23 20:44:28 mju Exp $
 *
 * $Log: imageautoregister.h_v $
 * Revision 1.17  2016/05/23 20:44:28  mju
 * Rename maxModel to georefModel.
 *
 * Revision 1.16  2015/12/04 21:42:55  mju
 * Add settings for maximum affine shear and projective tilt.
 *
 * Revision 1.15  2015/12/03 18:38:29  mju
 * Use int32 not int so mac doesn't complain.
 *
 * Revision 1.14  2015/12/03 17:24:20  mju
 * Add settings for components to match.
 *
 * Revision 1.13  2015/12/01 13:59:00  mju
 * Add setting for detector type.
 *
 * Revision 1.12  2015/11/20 18:55:12  mju
 * Add settings to use components for keypoints.
 *
 * Revision 1.11  2015/11/19 23:05:29  mju
 * Add allFromSinglePoint method for testing.
 *
 * Revision 1.10  2015/11/18 18:38:44  mju
 * Add keypoint method.
 *
 * Revision 1.9  2011/10/11 13:57:24  mju
 * Remove unneeded hdr.
 *
 * Revision 1.8  2011/02/23 18:09:39  mju
 * Remove max gcps setting.
 *
 * Revision 1.7  2011/01/17 17:55:29  mju
 * Add settings for predefined and using existing gcps.
 *
 * Revision 1.5  2011/01/04 21:26:38  mju
 * Remove maxRunTime, combine settings.
 *
 * Revision 1.1  2010/12/13 17:51:56  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEAUTOREGISTER_H
#define	INC_SPATMOD_IMAGEAUTOREGISTER_H

#ifndef	INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

#ifndef	INC_SPATMOD_GEOREFERENCE_H
#include <spatmod/georeference.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#include <utility>

// Forward declarations
#ifndef GENERATING_DOXYGEN_OUTPUT
class MIJOB;
class XMLNODE;
namespace SPATMOD {
	class GEOREFERENCE;
	namespace IMAGE {
		class STAGE;
		}
	}
namespace RVC {
	class OBJITEMLIST;
	}
#endif


namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Perform automatic registration of input image to reference.
class AUTOREGISTER {
	public:

		enum METHOD {
			METHOD_Automatic = 0,					//!< Automatic based on initial georeference
			METHOD_CrossCorrelate,					//!< Cross-correlate, requires good initial estimate
			METHOD_KeyPoint,							//!< Match keypoints at maximum common resolution
			METHOD_EstimateFromSinglePoint,		//!< Match from single-point (typically GPS at center) using reduce resolution keypoints
			METHOD_AllFromSinglePoint,				//!< All keypoints starting from single point (for testing only)
			};

		//! Pair of components, Input is "first", Reference is "second".
		typedef std::pair<INT32,INT32> COMPONENTPAIR;

		//! Array of component pairs.
		typedef SIMPLE_ARRAY<COMPONENTPAIR> COMPONENTPAIRS;

		//! Settings used for Run.
		class SETTINGS {
			public:

				//! Constructor, reads setting from Ini file.
				SETTINGS ();

				//! Copy constructor.
				SETTINGS (const SETTINGS &rhs);

				//! Destructor.
				~SETTINGS ();

				//! Assignment.
				SETTINGS& operator= (const SETTINGS &rhs);

				//! Get whether to adjust existing GCPs if kept.
				bool GetAdjustExistingGCPs () const
					{ return (m_AdjustExistingGCPs); }

				//! Get whether to automatically generate GCPs.
				bool GetAutoGenerateGCPs () const
					{ return (m_AutoGenerateGCPs); }

				//! Get components to use in cross-correlation.
				const COMPONENTPAIRS& GetComponentsCorrelate () const
					{ return (m_ComponentsCorrelate); }

				//! Get components to use in keypoint matching.
				const COMPONENTPAIRS& GetComponentsKeyPoint () const
					{ return (m_ComponentsKeyPoint); }

				//! Get autocorrelation window size in input image cells.
				UINT16 GetCorrPatchSize () const
					{ return (m_CorrPatchSize); }

				//! Get estimated cell size (meters).
				double GetEstimatedCellSize () const
					{ return (m_EstCellSizeM); }

				//! Get GCP spacing in input image cells.
				double GetGCPSpacing () const
					{ return (m_GCPSpacing); }

				//! Set georeference transformation model.
				GEOREFERENCE::MODEL GetGeorefModel () const
					{ return (m_GeorefModel); }

				//! Get initial accuracy in input image cells.
				double GetInitialAccuracy () const
					{ return (m_InitialAccuracy); }

				//! Get whether to keep existing GCPs.
				bool GetKeepExistingGCPs () const
					{ return (m_KeepExistingGCPs); }

				//! Get keypoint detector subtype.
				INT32 GetKeyPointDetectorSubType () const
					{ return (m_KeyPointDetectorSubType); }

				//! Get keypoint detector type.
				//! @return RVC::FEATURELOCATOR::DETECTORTYPE.
				INT32 GetKeyPointDetectorType () const
					{ return (m_KeyPointDetectorType); }

				//! Get maximum affine shear angle (degrees).
				double GetMaxAffineShearAngle () const
					{ return (m_MaxAffineShear); }

				//! Get maximum individual GCP residual in input image cells.
				double GetMaxGCPResidual () const
					{ return (m_MaxGCPResidual); }

				//! Get maximum projective tilt angle (degrees).
				double GetMaxProjectiveTiltAngle () const
					{ return (m_MaxProjectiveTilt); }

				//! Get autoregister method.
				METHOD GetMethod () const
					{ return (m_Method); }

				//! Get list of objects containing predefined GCPs.
				const RVC::OBJITEMLIST* GetPredefinedGCPs () const
					{ return (m_pObjItemListPredefinedGCPs); }

				//! Get resample method.
				RESAMPLEMETHOD GetResampleMethod () const
					{ return (m_ResampleMethod); }

				//! Read settings from Ini file, specific to process.
				void IniRead ();

				//! Write settings to Ini file, specific to process.
				void IniWrite () const;

				//! Read from job file.
				ERRVALUE JobRead (
					const MIJOB &job,
					const XMLNODE *nodeParent
					);

				//! Write to job file.
				ERRVALUE JobWrite (
					MIJOB &job,
					XMLNODE *nodeParent
					) const;

				//! Reset settings to default values.
				void ResetDefaults ();

				//! Set to automatically generate GCPs.
				void SetAutoGenerateGCPs (
					bool AutoGenerateGCPs
					) { m_AutoGenerateGCPs = AutoGenerateGCPs; }

				//! Set components to use in cross correlation.
				//! If not specified, the first component of the input and reference will be matched.
				void SetComponentsCorrelate (
					const INT32 *ComponentsInput,		//!< Input component(s) (for composite) or band indices (for multiple)
					const INT32 *ComponentsReference,	//!< Corresponding reference component(s) (for composite) or band indices (for multiple)
					UINT32 NumComponents					//!< Number of components to correlate
					);

				//! Set components to use in cross-correlation.
				void SetComponentsCorrelate (
					const COMPONENTPAIRS &components
					) { m_ComponentsCorrelate = components; }

				//! Set components to use in keypoint location.
				//! If not specified, the average (for composite) or first (for multiple) will be used.
				//! Usually only a single set of keypoints for each image is matched. However, if no valid match is found
				//! additional components may be matched in any combination. Each will be tried in turn until a match is found.
				void SetComponentsKeyPoint (
					const INT32 *ComponentsInput,		//!< Input component(s) (for composite) or band indices (for multiple)
					const INT32 *ComponentsReference,	//!< Corresponding reference component(s) (for composite) or band indices (for multiple)
					UINT32 NumComponents = 1			//!< Number of components to correlate
					);

				//! Set components to use in keypoint location.
				void SetComponentsKeyPoint (
					const COMPONENTPAIRS &components
					) { m_ComponentsKeyPoint = components; }

				//! Set autocorrelation window size in input image cells.
				void SetCorrPatchSize (UINT16 CorrPatchSize)
					{ m_CorrPatchSize = CorrPatchSize; }

				void SetEstimatedCellSize (double EstCellSizeM)
					{ m_EstCellSizeM = EstCellSizeM; }

				//! Set GCP spacing in input image cells.
				void SetGCPSpacing (double GCPSpacing)
					{ m_GCPSpacing = GCPSpacing; }

				//! Set georeference transformation model.
				void SetGeorefModel (GEOREFERENCE::MODEL Model)
					{ m_GeorefModel = Model; }

				//! Set initial accuracy in input image cells.
				void SetInitialAccuracy (double InitialAccuracy)
					{ m_InitialAccuracy = InitialAccuracy; }

				//! Set keypoint detector type to use.
				void SetKeyPointDetectorType (
					INT32 DetectorType,						//!< RVC::FEATURELOCATOR::DETECTORTYPE
					INT32 DetectorSubType = 0
					) { m_KeyPointDetectorType = DetectorType; m_KeyPointDetectorSubType = DetectorSubType; }

				//! Set maximum affine shear angle.
				void SetMaxAffineShearAngle (double MaxAffineShear)
					{ m_MaxAffineShear = MaxAffineShear; }

				//! Set maximum individual GCP residual in input image cells.
				void SetMaxGCPResidual (double MaxGCPResidual)
					{ m_MaxGCPResidual = MaxGCPResidual; }

				//! Set maximum projective tilt angle.
				void SetMaxProjectiveTiltAngle (double MaxProjectiveTilt)
					{ m_MaxProjectiveTilt = MaxProjectiveTilt; }

				//! Set autoregister method.
				void SetMethod (
					METHOD method
					) { m_Method = method; }

				//! Set predefined GCPs to use.
				void SetPredefinedGCPs (
					const RVC::OBJITEMLIST *pObjItemListPredefinedGCPs		//!< Object location, NULL to not use
					);

				//! Set resample method.
				void SetResampleMethod (RESAMPLEMETHOD ResampleMethod)
					{ m_ResampleMethod = ResampleMethod; }

				//! Set to use existing GCPs.
				void SetKeepExistingGCPs (
					bool KeepExisting,
					bool AdjustExisting
					) { m_KeepExistingGCPs = KeepExisting; m_AdjustExistingGCPs = AdjustExisting; }

			private:
				#ifndef	GENERATING_DOXYGEN_OUTPUT
				COMPONENTPAIRS m_ComponentsCorrelate;
				COMPONENTPAIRS m_ComponentsKeyPoint;
				double m_InitialAccuracy;
				double m_GCPSpacing;
				double m_MaxGCPResidual;
				double m_EstCellSizeM;			// Estimated cell size in meters
				double m_MaxAffineShear;
				double m_MaxProjectiveTilt;
				INT32 m_KeyPointDetectorType;
				INT32 m_KeyPointDetectorSubType;
				UINT16 m_CorrPatchSize;
				GEOREFERENCE::MODEL m_GeorefModel;
				METHOD m_Method;
				RESAMPLEMETHOD m_ResampleMethod;
				RVC::OBJITEMLIST *m_pObjItemListPredefinedGCPs;
				bool m_AutoGenerateGCPs;
				bool m_KeepExistingGCPs;
				bool m_AdjustExistingGCPs;

				void Copy (const SETTINGS &rhs);
				#endif	// GENERATING_DOXYGEN_OUTPUT
			};

		//! Constructor.
		AUTOREGISTER ();

		//! Destructor.
		~AUTOREGISTER ();

		//! Get georeference resulting from autoregistration.
		//! Use to access control points and model.
		const GEOREFERENCE* GetGeorefResult () const;

		//! Get RMS residual from last run.
		double GetRMSResidual () const;

		//! Run autoregistration.
		//! Will only return error if cannot determine any reasonable georeference for input.
		int Run (
			IMAGE::STAGE &StageInput,				//!< Input image, must have initial georeference
			IMAGE::STAGE &StageRef,					//!< Reference image, must have georeference
			const SETTINGS &Settings				//!< Run settings
			);

		//! Run autoregistration.
		ERRVALUE Run (
			const RVC::OBJITEMLIST &ObjItemListInput,	//!< Input image(s), if multiple must be coincident
			const GEOREFERENCE &GeorefInput,				//!< Initial georeference for input
			const RVC::OBJITEMLIST &ObjItemListRef,	//!< Reference image(s), if multiple must be coincident
			const SETTINGS &Settings						//!< Run settings
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		AUTOREGISTER (const AUTOREGISTER &);
		AUTOREGISTER& operator= (const AUTOREGISTER &);
		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEAUTOREGISTER_H
