/**
 * \file spatmod/imagefilterlensgeometry.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_LENS_GEOMETRY definition.
 *
 * \if NODOC
 * $Id: imagefilterlensgeometry.h_v 1.3 2015/08/20 21:14:17 mju Exp $
 *
 * $Log: imagefilterlensgeometry.h_v $
 * Revision 1.3  2015/08/20 21:14:17  mju
 * Remove crop factor parm from ctor.
 *
 * Revision 1.2  2015/08/20 15:22:08  mju
 * define_enum_op.
 *
 * Revision 1.1  2015/08/07 21:22:39  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERLENSGEOMETRY_H
#define	INC_SPATMOD_IMAGEFILTERLENSGEOMETRY_H

#ifndef	INC_SPATMOD_IMAGEFILTERLENS_H
#include <spatmod/imagefilterlens.h>
#endif

#ifndef	INC_SPATMOD_IMAGEHELPERRESAMPLE_H
#include <spatmod/imagehelperresample.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to perform lens corrections and geometry change.
//! Presently the geometry of the result will be "Rectilinear" so that straight lines in the scene appear straight
//! in the image.
//! Corrections must be applied in a specific order. This is done automatically when more than one correction
//! is specified. Use caution if some corrections have already been done or will be done later.
//! Note also that since the output of the filter will be Rectilinear, if the geometry of the lens is not
//! Rectilinear (ie, Fisheye) then no further corrections will be possible later. Also note that TCA and Vignetting
//! corrections should only be done on "raw" imagery. Applying these to JPEG, sRGB or other "processed" images will
//! produce incorrect results.
class FILTER_LENS_GEOMETRY : public FILTER_LENS {
	public:

		//! Correction to apply.
		enum CORRECT {
			CORRECT_None =			0,			//!< Perform no corrective modifications, may still perform geometry and scaling
			CORRECT_TCA =			0x01,		//!< Correct for traversal chromatic aberrations, should only be applied to "raw" (not jpeg) input, RGB only
			CORRECT_Distortion =	0x02,		//!< Correct for distortion
			CORRECT_All =			0x03		//!< Perform all corrections
			};

		//! Construct.
		FILTER_LENS_GEOMETRY (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const lfLens *lfLens,					//!< Lens data
			float FocalLengthMM,						//!< Focal length in millimeters
			float Aperture,							//!< Aperture (F-number)
			float FocusDistanceM,					//!< Approximate focus distance in meters
			CORRECT correct,							//!< Corrections to perform
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method
			);

		// Destructor.
		virtual ~FILTER_LENS_GEOMETRY ();

		//! Set scaling from source.
		//! By default, automatic scaling is performed, which eliminates null areas at the borders
		//! at the expense of removing some valid pixels (typically at the corners).
		//! May only be done before Initialize.
		void SetScale (
			float scale									//!< Scale, use 1 to keep all input pixels
			) { m_Scale = scale; }

		//! Set automatic scaling (the default).
		//! May only be done before Initialize.
		void SetScaleAuto ()
			{ m_Scale = 0; }

		//! Set zooming for result.
		//! By default the resulting image has the same dimensions as the previous stage.
		//! However, for some cases, such as fisheye lenses it may be desirable to zoom to make the result
		//! larger, especially when also setting scaling to 1, thus not trimming off the null border areas.
		//! May only be done before Initialize.
		void SetZoom (
			float zoom
			) { m_Zoom = zoom; m_ZoomInv = 1 / m_Zoom; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<float> m_CoordBuf;
		DOUBLE_ARRAY<DPOINT2D> m_SrcCoord;
		HELPER_RESAMPLE m_Resampler;
		float m_Scale;
		float m_Zoom;
		float m_ZoomInv;
		CORRECT m_Correct;
		RESAMPLEMETHOD m_ResampleMethod;

		// BASE/STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR &iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// Unimplemented
		FILTER_LENS_GEOMETRY (const FILTER_LENS_GEOMETRY &);
		FILTER_LENS_GEOMETRY& operator= (const FILTER_LENS_GEOMETRY &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FILTER_LENS_GEOMETRY::CORRECT)
#endif

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERLENSGEOMETRY_H

