/**
 * \file spatmod/imagefilterrectify.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_RECTIFY definition.
 *
 * \if NODOC
 * $Id: imagefilterrectify.h_v 1.2 2016/01/12 17:14:53 mju Exp $
 *
 * $Log: imagefilterrectify.h_v $
 * Revision 1.2  2016/01/12 17:14:53  mju
 * Add camera3d support.
 * Add option to calculate distance from reference point.
 *
 * Revision 1.1  2014/05/27 20:36:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERRECTIFY_H
#define	INC_SPATMOD_IMAGEFILTERRECTIFY_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

#ifndef	INC_SPATMOD_IMAGESAMPLEITERATOR_H
#include <spatmod/imagesampleiterator.h>
#endif

#ifndef	INC_SPATMOD_IMAGEHELPERRESAMPLE_H
#include <spatmod/imagehelperresample.h>
#endif

#ifndef	INC_MI32_TRANS2DMAPGEN_H
#include <mi32/trans2dmapgen.h>
#endif

#ifndef	INC_MI32_RATIONALPOLY_H
#include <mi32/rationalpoly.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

class CAMERA3D;

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to rectify image using rational polynomial definition and terrain.
//! Note that source image will not be processed "sequentially", and the sections specified in PullBegin for the source
//! will overlap.  It is recommended that this filter be placed early in the pipeline or that a suitable caching filter
//! be placed ahead of it if the input requires significant computation or sequential iteration.
class FILTER_RECTIFY : public FILTER {
	public:

		//! Construct using RATIONALPOLY, with specified dimensions and georeference.
		FILTER_RECTIFY (
			STAGE *pStageSource,								//!< Pointer to source image stage
			bool TakeOwnershipSource,						//!< Filter takes ownership of source image stage and will delete when needed
			STAGE *pStageTerrain,							//!< Pointer to terrain image stage
			bool TakeOwnershipTerrain,						//!< Filter takes ownership of terrain image stage and will delete when needed
			const RATIONALPOLY &RationalPolySource,	//!< Rational polynomial definition for source image
			const DIMENSIONS &TgtDimensions,				//!< Resulting image dimensions
			const GEOREFERENCE &TgtGeoreference			//!< Georeference for resulting image
			);

		//! Construct using RATIONALPOLY, to match specified reference image.
		FILTER_RECTIFY (
			STAGE *pStageSource,								//!< Pointer to source image stage
			bool TakeOwnershipSource,						//!< Filter takes ownership of source image stage and will delete when needed
			STAGE *pStageTerrain,							//!< Pointer to terrain image stage
			bool TakeOwnershipTerrain,						//!< Filter takes ownership of terrain image stage and will delete when needed
			const RATIONALPOLY &RationalPolySource,	//!< Rational polynomial definition for source image
			const BASE &RefImage								//!< Reference image to match
			);

		//! Construct using CAMERA3D, to match specified dimensions and georeference.
		FILTER_RECTIFY (
			STAGE *pStageSource,								//!< Pointer to source image stage
			bool TakeOwnershipSource,						//!< Filter takes ownership of source image stage and will delete when needed
			STAGE *pStageTerrain,							//!< Pointer to terrain image stage
			bool TakeOwnershipTerrain,						//!< Filter takes ownership of terrain image stage and will delete when needed
			const CAMERA3D &CameraSource,					//!< Camera parameters for source image
			const DIMENSIONS &TgtDimensions,				//!< Resulting image dimensions
			const GEOREFERENCE &TgtGeoreference			//!< Georeference for resulting image
			);

		// Destructor.
		virtual ~FILTER_RECTIFY ();

		//! Set to calculate squared distance from center or reference point.
		//! May only be called before Initialize. If used, an additional (double) sample component will be added to the
		//! resulting image as the last sample.
		void SetCalcDistanceSquared (
			const DPOINT2D *pRefPointSource = 0			//!< Reference point in source image, nullptr for center
			);

		//! Set resample method for source.
		//! May only be called before Initialize. Default method is bicubic.
		void SetResampleMethodSource (
			RESAMPLEMETHOD ResampleMethod
			) { m_ResampleMethodSource = ResampleMethod; }

		//! Set resample method for terrain.
		//! May only be called before Initialize. Default method is bicubic. Terrain will only be resampled if does not match result.
		void SetResampleMethodTerrain (
			RESAMPLEMETHOD ResampleMethod
			) { m_ResampleMethodTerrain = ResampleMethod; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DOUBLE_ARRAY<DPOINT2D> m_SrcCoord;
		DOUBLE_ARRAY<DPOINT2D> m_TgtCoord;
		double m_ElevMin;
		double m_ElevMax;
		HELPER_RESAMPLE m_Resampler;
		RATIONALPOLY m_RationalPoly;
		CAMERA3D *m_pCamera;
		DRECT2D m_SrcRectTotal;
		TRANS2D_MAPGEN m_TransTgtToModel;
		DPOINT2D m_PointDistance;
		SIMPLE_ARRAY<UINT16> m_SampleIdxsNoDistance;
		RESAMPLEMETHOD m_ResampleMethodSource;
		RESAMPLEMETHOD m_ResampleMethodTerrain;
		bool m_bUseCamera;
		bool m_bCalcDistance;

		ERRVALUE ComputeSrcMaxSectionSize (
			const SECTION_ITERATOR &TgtIterator,
			DRECT2D &SrcRectAll,
			UINT32 &SrcMaxColumns,
			UINT32 &SrcMaxRows
			) const;

		ERRVALUE ComputeSrcRect (
			const DRECT2D &RectTgt,
			DRECT2D &RectSrc,
			double ElevMin,
			double ElevMax
			) const;

		// BASE/STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR &iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// Unimplemented
		FILTER_RECTIFY (const FILTER_RECTIFY&);
		FILTER_RECTIFY& operator= (const FILTER_RECTIFY&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERRECTIFY_H

