/**
 * \file spatmod/imagekeypoints.h
 * \brief SPATMOD::IMAGE::KEYPOINTS definitions
 *
 * \if NODOC
 * $Id: imagekeypoints.h_v 1.4 2015/11/18 18:19:44 mju Exp $
 *
 * $Log: imagekeypoints.h_v $
 * Revision 1.4  2015/11/18 18:19:44  mju
 * Add getDescriptorSizeBytes.
 *
 * Revision 1.3  2015/11/12 15:25:21  mju
 * Add block index ranges.
 *
 * Revision 1.2  2015/11/10 14:58:12  mju
 * Add getDescriptorMatCols/Type.
 *
 * Revision 1.1  2015/11/09 18:06:13  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEKEYPOINTS_H
#define	INC_SPATMOD_IMAGEKEYPOINTS_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef	INC_VECTOR
#define	INC_VECTOR
#include <vector>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace cv {
	class Mat;
	}
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Set image keypoints and corresponding descriptors.
class KEYPOINTS {
	public:

		typedef std::pair<UINT32,UINT32> IDXPAIR;

		//! Find matches between two sets of keypoints.
		//! For a tentative match to be included, the ratio between the second-closest and closest must be less than
		//! the specified distance ratio.
		static ERRVALUE Match (
			const KEYPOINTS &keypoints1,			//!< First set of keypoints
			const KEYPOINTS &keypoints2,			//!< Second set of keypoints
			std::vector<IDXPAIR> &matchpairs,	//!< Match pairs returned
			double distratio = .5					//!< Distance ratio
			);

		//! Constructor.
		KEYPOINTS ();

		//! Destructor.
		~KEYPOINTS ();

		//! Access descriptors, typically for reading into.
		cv::Mat& AccessDescriptors ();

		//! Access pointer to points, typically for reading into.
		double* AccessPtrPoints (
			UINT32 startidx = 0
			);

		//! Add 2D points and descriptors.
		//! An error will be returned if number of point dimensions is not 2.
		ERRVALUE Add (
			const DPOINT2D *points,			//!< Points
			const cv::Mat &descriptors		//!< Descriptors, number of rows specifies number of points also
			);

		//! Add 2D points and descriptors.
		//! An error will be returned if number of point dimensions is not 2.
		ERRVALUE Add (
			UINT32 NumItems,					//!< Number of items
			const DPOINT2D *points,			//!< Points
			const void *descriptors			//!< Descriptors
			);

		//! Add 3D points and descriptors.
		//! An error will be returned if number of point dimensions is not 3.
		ERRVALUE Add (
			const DPOINT3D *points,			//!< Points
			const cv::Mat &descriptors		//!< Descriptors, number of rows specifies number of points also
			);

		//! Add 3D points and descriptors.
		//! An error will be returned if number of point dimensions is not 3.
		ERRVALUE Add (
			UINT32 NumItems,					//!< Number of items
			const DPOINT3D *points,			//!< Points
			const void *descriptors			//!< Descriptors
			);

		//! Clear contents (does not free memory).
		void Clear ();

		//! Create.
		ERRVALUE Create (
			int cvMatType,								//!< OpenCV matrix type for descriptors
			int cvMatCols,								//!< Number of OpenCV descriptor matrix columns
			int numDimensions = 0,					//!< Number of point dimensions 0, 2 or 3, cannot be 0 if numReserve > 0
			UINT32 numReserve = 0,					//!< Initial number of items to reserve space for
			const LRECT2D *pBlockRange = 0		//!< Range of blocks for indexing (last values are exclusive)
			);

		//! Get beginning point index for block.
		UINT32 GetBlockIndexBegin (
			INT32 blockrow,							//!< Block row
			INT32 blockcol								//!< Block column
			) const;

		//! Get ending point index (exclusive) for block.
		UINT32 GetBlockIndexEnd (
			INT32 blockrow,							//!< Block row
			INT32 blockcol								//!< Block column
			) const;

		//! Get block range.
		const LRECT2D& GetBlockRange () const;

		//! Get number of descriptor matrix columns.
		int GetDescriptorMatCols () const;

		//! Get descriptor matrix type.
		int GetDescriptorMatType () const;

		//! Get matrix of descriptors for matching.
		const cv::Mat& GetDescriptors () const;

		//! Get descriptor size in bytes.
		int GetDescriptorSizeBytes () const;

		//! Get number of point/descriptor pairs.
		UINT32 GetNumItems () const;

		//! Get 2D point.
		const DPOINT2D& GetPoint2D (
			UINT32 idx							//!< Point index
			) const;

		//! Get general 2D point (with 'x' and 'y' members).
		template <typename _pt2d> void GetPoint2D (
			UINT32 idx,							//!< Point index
			_pt2d &pt							//!< Point returned
			) const { const DPOINT2D &dpt = GetPoint2D(idx); pt.x = dpt.x; pt.y = dpt.y; }

		//! Get 3D point.
		//! If number of point dimensions is not 3 resulting point values will be undefined/NAN.
		const DPOINT3D& GetPoint3D (
			UINT32 idx							//!< Point index
			) const;

		//! Get general 3D point (with 'x', 'y', 'z' members).
		//! If number of point dimensions is not 3 resulting point values will be undefined/NAN.
		template <typename _pt2d> void GetPoint3D (
			UINT32 idx,							//!< Point index
			_pt2d &pt							//!< Point returned
			) const { const DPOINT3D &dpt = GetPoint3D(idx); pt.x = dpt.x; pt.y = dpt.y; pt.z = dpt.z; }

		//! Get number point dimensions.
		//! @return 0 if not specified, 2 or 3
		int GetPointDimensions () const;

		//! Get pointer to points.
		const double* GetPtrPoints (
			UINT32 startidx = 0
			) const;

		//! Reserve space for specified maximum number of items.
		ERRVALUE Reserve (
			UINT32 MaxItems					//!< Total space to reserve for
			);

		//! Resize to specified number of items.
		//! If number of items is increased, new contents will be undefined.
		ERRVALUE Resize (
			UINT32 NumItems					//!< New number of items
			);

		//! Set point index range for block.
		void SetBlockIndexRange (
			INT32 blockrow,					//!< Block row within range specified in Create()
			INT32 blockcol,					//!< Block column within range specified in Create()
			UINT32 PointIdxBegin,			//!< Beginning index (inclusive)
			UINT32 PointIdxEnd				//!< Ending index (exclusive)
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		KEYPOINTS (const KEYPOINTS &);
		KEYPOINTS& operator= (const KEYPOINTS &);
		#endif
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEKEYPOINTS_H
