/**
 * \file spatmod/imagepassthru.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::PASSTHRU definition.
 *
 * \if NODOC
 * $Id: imagepassthru.h_v 1.1 2017/03/15 18:22:26 mju Exp $
 *
 * $Log: imagepassthru.h_v $
 * Revision 1.1  2017/03/15 18:22:26  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEPASSTHRU_H
#define	INC_SPATMOD_IMAGEPASSTHRU_H

#ifndef	INC_SPATMOD_IMAGESTAGE_H
#include <spatmod/imagestage.h>
#endif

#ifndef	INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base class for image pass-through stage.
//! A pass-through stage does not alter the image in any way. It is typically used to compute specific properties
//! of the image during processing.
class PASSTHRU : public STAGE {
	public:

		// Destructor.
		virtual ~PASSTHRU ();

	protected:

		//! Construct with specified input stage and ownership.
		PASSTHRU (
			STAGE *pInputStage,						//!< Pointer to input stage
			bool bTakeOwnership						//!< Whether to take ownership of input stage and will delete when needed
			);

		//! Get pointer to input stage in pipeline.
		STAGE* GetPreviousStage () const
			{ return (m_PrevStagePtr); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		STAGE_AUTOPTR m_PrevStagePtr;

		ERRVALUE v_GetDimensions (DIMENSIONS &dimensions, double zoom) const;
		ERRVALUE v_GetGeoreference (GEOREFERENCE &georeference, double zoom) const;
		void v_GetMetadata (MISTRING &result, METADATACONTENT content, METADATAFORMAT format) const;
		const PIXELTABLE* v_GetPixelLookupTable () const;
		ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double> &ZoomsAllowed) const;
		ERRVALUE v_PullBegin (const SECTION_ITERATOR &iterator);
		ERRVALUE v_PullEnd ();
		ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);
		bool v_QueryCapability (CAPABILITY capability) const;
		float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_LogCreate (const char *label, bool OnlyEnds, MILOG::MODE mode);

		// Unimplemented
		PASSTHRU (const PASSTHRU &);
		PASSTHRU& operator= (const PASSTHRU &);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// PASSTHRU overridables.

		//! Called after PullBegin is done on input stage.
		//! Default implementation does nothing.
		virtual ERRVALUE v_OnPullBegin (
			const SECTION_ITERATOR &iterator		//!< SubSection iterator to be used
			);

		//! Called before PullEnd on input stage.
		//! Default implementation does nothing.
		virtual ERRVALUE v_OnPullEnd ();

		//! Called after PullSubSection is done on input stage.
		virtual ERRVALUE v_OnPullSubSection (
			const SECTION &SubSection,				//!< Subsection being processed
			const BUFFER &buffer						//!< Buffer to be processed
			) = 0;

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEPASSTHRU_H

