/**
 * \file spatmod/imagesourcememory.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_SURFACEFROMPOINTS definition.
 *
 * \if NODOC
 * $Id: imagesourcesurfacefrompoints.h_v 1.1 2016/01/21 22:01:10 mju Exp $
 *
 * $Log: imagesourcesurfacefrompoints.h_v $
 * Revision 1.1  2016/01/21 22:01:10  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCESURFACEFROMPOINTS_H
#define	INC_SPATMOD_IMAGESOURCESURFACEFROMPOINTS_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Source which supports reading from memory.
class SOURCE_SURFACEFROMPOINTS : public SOURCE {
	public:

		//! Construct using reference image for dimensions and georeference.
		explicit SOURCE_SURFACEFROMPOINTS (
			const IMAGE::BASE &RefImage,				//!< Reference image for dimensions and georeference
			DATATYPE DataType = DATATYPE_FLOAT32	//!< Result data type
			);

		//! Construct using dimensions and georeference.
		SOURCE_SURFACEFROMPOINTS (
			const DIMENSIONS &dimensions,				//!< Image dimensions
			const GEOREFERENCE &georeference,		//!< Georeference
			DATATYPE DataType = DATATYPE_FLOAT32	//!< Result data type
			);

		// Destructor.
		virtual ~SOURCE_SURFACEFROMPOINTS ();

		//! Add point to surface.
		//! All points must be added before Initialize is called.
		//! @return 1 if point added, 0 if outside surface, < 0 if error.
		int AddPoint (
			const DPOINT3D &point						//!< Point in "georeference" CRS
			) { return (AddPoints(&point,1)); }

		//! Add points to surface.
		//! All points must be added before Initialize is called.
		//! @return Number of points added, 0 if all outside surface, < 0 if error.
		int AddPoints (
			const DPOINT3D *points,						//!< Points in "georeference" CRS
			int NumPoints									//!< Number of points
			);

		//! Add points in image coordinates to surface.
		//! All points must be added before Initialize is called.
		//! @return Number of points added, 0 if all outside surface, < 0 if error.
		int AddPointsInImageCoords (
			const DPOINT3D *points,						//!< Points in image coordinates
			int NumPoints									//!< Number of points
			);

		//! Get buffer to use directly.
		const IMAGE::BUFFER& GetBuffer () const
			{ return (m_Buffer); }

		//! Set whether to fill to edges of image based on nearby points.
		//! Default is to not fill.
		void SetFillToEdges (
			bool bFillToEdges = true
			) { m_bFillToEdges = bFillToEdges; }

		//! Set whether to use highest value if multiple points at same image cell.
		//! Lowest value is used by default.
		void SetUseHighestValue (
			bool bUseHighestValue = true
			) { m_bUseHighestValue = bUseHighestValue; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef SOURCE BASECLASS;

		IMAGE::BUFFER m_Buffer;
		UINT32 m_TotalPoints;
		bool m_bIsDouble;
		bool m_bUseHighestValue;
		bool m_bFillToEdges;

		int AddPointsLow (const DPOINT3D *points, int NumPoints);
		ERRVALUE CreateBuffer ();
		void SetFromClosest (INT32 col0, INT32 row0);

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR &iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);

		// Unimplemented
		SOURCE_SURFACEFROMPOINTS (const SOURCE_SURFACEFROMPOINTS&);
		SOURCE_SURFACEFROMPOINTS& operator= (const SOURCE_SURFACEFROMPOINTS&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCESURFACEFROMPOINTS_H

