/**
 * Demonstrate use of TNTsdk object display functions
 *
 * This program allows the user to view one or more spatial data layers.
 *
 * The user can select a text file to read x/y coordinates
 * of symbols to be plotted over the selected objects.  The map projection for
 * these symbols may also be set via the File / Projection option.
 *
 * If the points are to be read from a file generated by a GPS device the
 * function to read the file could be altered to match a specific format and
 * the projection option could be removed if the file format contains this
 * information or it is always a specific projection.
**/

#include <mgui/appmain.h>
#include <mgui/topshell.h>
#include <mgui/dlggetobject.h>
#include <mgui/toolcircle.h>

#include <gre/group.h>
#include <gre/system.h>
#include <gre/uiformview.h>
#include <gre/uiviewtoolsimple.h>

#include <mi32/coordtrans.h>
#include <mi32/mgd2.h>
#include <mi32/filetext.h>
#include <mi32/textid.h>

#include <memory>

#define	SYMBOLRADIUS	4


//=====================================================================================================================
//		MAINWND definition - Begin

class MAINWND : public MGUI::TOPSHELL {
	public:
		MAINWND ();
		~MAINWND ();

	private:
		typedef MAINWND THISCLASS;

		enum CMDID {
			CMDID_File_ReadPoints =		1,
			CMDID_File_GetCRS,
			CMDID_Exit
			};

		GRE::GROUP m_Group;
		GRE::UI::FORM_VIEW m_FormView;
		SIMPLE_ARRAY<DPOINT2D> m_SymbolPoint;
		SPATREF::COORDREFSYS m_CoordRefSys;
		FILEPATH m_SymbolFilePath;
		GRE::UI::VIEWTOOL_SIMPLE_T<MGUI::TOOL_CIRCLE> *m_pToolCircle;

		static void GRECB_Group (const GRE::MSG *pMsg, void *vpMainWnd);

		void OnGreMsg (const GRE::MSG &msg);
		void OnMenuFile (MGUI::ID cmdid, bool bToggleOn);
		void OnViewAddTools (GRE::UI::VIEW_CANVAS *pViewCanvas);

		// Overrides from TOPSHELL.
		int v_CreateContent ();
		void v_OnCloseRequest ();
	};

//		MAINWND definition - End
//---------------------------------------------------------------------------------------------------------------------
//		MAINWND public - Begiin

MAINWND::MAINWND (
	):
	TOPSHELL(HELPID__None),
	m_FormView(&m_Group),
	m_pToolCircle(0)
	{
	}

// Destructor.
MAINWND::~MAINWND (
	) {
	}

//		MAINWND public - End
//---------------------------------------------------------------------------------------------------------------------
//		MAINWND private static - Begin

// [static] Handle GRE messages.
void MAINWND::GRECB_Group (
	const GRE::MSG *pMsg,
	void *vpMainWnd
	) {
	static_cast<MAINWND*>(vpMainWnd)->OnGreMsg(*pMsg);
	}

//		MAINWND private static - End
//---------------------------------------------------------------------------------------------------------------------
//		MAINWND private - Begin

// Handle GRE messages.
void MAINWND::OnGreMsg (
	const GRE::MSG &msg
	) {
	ERRVALUE err = 0;
	COLOR color(COLOR::RED);

	if (m_SymbolPoint.GetNumItems() == 0) return;		// No symbols selected yet
	if (msg.GetObject()->GetType() == GRE::OBJECT::TYPE_View) {
		switch (msg.GetAction()) {
			case GRE::MSG::ACTION_DrawEnd:
			case GRE::MSG::ACTION_Expose:
			case GRE::MSG::ACTION_RestoreEnd: {
				if (msg.GetView()->GetDrawingDeviceScreen() == NULL) return;
				std::auto_ptr<MGD::CONTEXT> dc(msg.GetView()->GetDrawingDeviceScreen()->CreateContext());
				SPATREF::COORDTRANS TransToGroup;
				if ((err = TransToGroup.SetupAutoCRS(m_CoordRefSys,m_Group.GetUsedCoordRefSys())) < 0) break;
				dc->SetColor(color);
				for (UINT32 i = 0; (i < m_SymbolPoint.GetNumItems()); ++i) {
					DPOINT2D dpoint(m_SymbolPoint[i]);
					if ((err = TransToGroup.ConvertForward(&dpoint,1)) < 0) break;
					if ((err = msg.GetView()->TransGroupToDisp(&m_Group,&dpoint,&dpoint,1)) < 0) break;
					INT32 x = FAST_ROUND_LIMITED(dpoint.x);
					INT32 y = FAST_ROUND_LIMITED(dpoint.y);
					dc->FillCircle(x,y,SYMBOLRADIUS);
					}
				break;
				}
			}
		}

// Does not work to show error in callback as causes multiple messages to appear
//	if (err < 0) MdlgShowErrorCode(NULL,err);
	return;
	}

void MAINWND::OnMenuFile (
	MGUI::ID cmdid,
	bool // bToggleOn
	) {
	ERRVALUE err = 0;
	switch (cmdid) {
		case CMDID_File_ReadPoints: {
			if ((err = MGUI::DlgGetFile(GetDlgParent(),TEXTID_Prompt_File_Read,"*",m_SymbolFilePath)) < 0) return;

			FILE_TEXT file;
			STRUTF8 strutf8;
			if ((err = file.Open(m_SymbolFilePath)) < 0) break;
			m_SymbolPoint.Clear();

			while ((err = file.ReadLine(strutf8)) >= 0) {
				if (strutf8[0] == '#') continue;		// Used to indicate a comment
				DPOINT2D pt;
				if (sscanf(strutf8,"%lf %lf",&pt.x,&pt.y) == 2) {
					if ((err = m_SymbolPoint.Append(pt)) < 0) break;
					}
				}
			m_FormView.GetViewCanvas()->RestoreFromMemory();
			break;
			}
		case CMDID_File_GetCRS:
			if ((err = SPATREF::DlgGetCoordRefSys(GetDlgParent(),m_CoordRefSys)) < 0) break;
			if (m_SymbolPoint.GetNumItems() > 0) m_FormView.GetViewCanvas()->RestoreFromMemory();
			break;
		case CMDID_Exit:
			MGUI::APP_MAIN::Exit();
			break;
		}
	if (err < 0) MdlgShowErrorCode(GetDlgParent(),err);
	return;
	}

void MAINWND::OnViewAddTools (
	GRE::UI::VIEW_CANVAS *pViewCanvas
	) {
	m_pToolCircle = new GRE::UI::VIEWTOOL_SIMPLE_T<MGUI::TOOL_CIRCLE>(*pViewCanvas,TEXTID_Generic_Circle,ICONID_DRAW_CIRCLE_RED);
	return;
	}

//		MAINWND private - End
//---------------------------------------------------------------------------------------------------------------------
//		MAINWND overrides - Begin

// Create main window contents.
int MAINWND::v_CreateContent (
	) {
	m_CoordRefSys.Assign(SPATREF::COORDREFSYS::IDNUM_Geographic2D_WGS84_Deg);
	m_Group.CallbackAdd(GRECB_Group,this,GRE::OBJECT::TYPE_View);

	m_FormView.SetDelegateAddTools(GRE::UI::FORM_VIEW::DELEGATE_ADDTOOLS(this,&THISCLASS::OnViewAddTools));
	m_FormView.Create(GetMainPane(),0,0,"View",GRE::UI::FORM_VIEW::CREATEFLAG_AddLayerMgrButton);

	MGUI::MENU MenuFile;
	MenuFile.Create(MGUI::MENU::TYPE_SubMenu,MGUI::MENU::DELEGATE(this,&THISCLASS::OnMenuFile));
	MenuFile.Add(MISTRING("Read Points..."),CMDID_File_ReadPoints);
	MenuFile.Add(TEXTID_Generic_Projection_,CMDID_File_GetCRS);
	MenuFile.AddSeparator();
	MenuFile.Add(TEXTID_Generic_Exit,CMDID_Exit);

	MGUI::MENU MenuBar;
	MenuBar.Create(MGUI::MENU::TYPE_MenuBar);
	MenuBar.Add(TEXTID_Generic_File,MenuFile);
	m_FormView.MenuAddSubMenuView(MenuBar);
	m_FormView.MenuAddSubMenuTools(MenuBar);
	m_FormView.MenuAddSubMenuOptions(MenuBar);
	m_FormView.MenuAddSubMenuHotKeys(MenuBar);
	AttachMenuBar(MenuBar,true);

	return (TRUE);
	}

void MAINWND::v_OnCloseRequest (
	) {
	MGUI::APP_MAIN::Exit();
	return;
	}

//		MAINWND overrides - End
//=====================================================================================================================
//		APP_OBJVIEW - Begin

class APP_OBJVIEW : public MGUI::APP_BASE {
	public:
		APP_OBJVIEW ();
	private:
		MAINWND *m_pMainWnd;
		// Overrides from APP_BASE.
		virtual void v_ExitInstance ();
		virtual const char* v_GetProcessName () const;
		virtual bool v_InitInstance ();
	};

// Constructor.
APP_OBJVIEW::APP_OBJVIEW (
	):
	m_pMainWnd(0)
	{
	}

void APP_OBJVIEW::v_ExitInstance (
	) {
	delete m_pMainWnd;
	GRE::SYSTEM::StopX();
	return;
	}

const char* APP_OBJVIEW::v_GetProcessName (
	) const {
	return ("objview");
	}

bool APP_OBJVIEW::v_InitInstance (
	) {
	GRE::SYSTEM::InitX();
	m_pMainWnd = new MAINWND;
	m_pMainWnd->Create("Main",TEXTID_Generic_View);
	return (true);
	}

//		APP_OBJVIEW - End
//=====================================================================================================================
//		MAIN application instance class

class MAIN : public MGUI::APP_MAIN {
	public:
		MAIN () : m_pApp(new APP_OBJVIEW) {}
		~MAIN () { }
	private:
		APP_OBJVIEW *m_pApp;
	};

static MAIN MainApp;

//=====================================================================================================================
